#ifndef _ELOG_H
#define _ELOG_H
/*****************************************************************************
File:     elog.h
Contents: Definitions for GIST event log library.
System:   BBNACI Programming Environments.
Author:   Bob Thomas

Remarks:

Copyright 1988 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

/* $Header: /nfs/europa/usr/BF2USRSRC/include/RCS/elog.h,v 1.23 91/06/03 15:19:09 bthomas Exp $ */

#define ELOGNAMEMAX	255

#define MAXOPENELOGS	256
#define EVENTSIZE	3
#define OPENLOGSIZE	2

# define ELOGTRACENAMEMAX	16

#define FIRSTEVENTCODEDEF	(MAXOPENELOGS * OPENLOGSIZE + 1)

#define elogOFSTOPLOG	1
#define elogOFOUTPUT	2
#define elogOFCYCLE	3

/*****************************************************************
* Function prototypes 
*****************************************************************/

#if defined(__STDC__) || defined(c_plusplus)
extern elog_define(int code, char *name, char *format);
extern elog_define_to_file(char *elog_name, int code, char *name, char *format);
extern int elog_init(char *log_name, int clear);
extern int elog_setup(char *log_name, int tag, int buf_size);
extern int elog_setup_for_auto_output(char *log_name, int tag, int buf_size);
extern int elog_create(char *log_name);
extern elog_finalize(unsigned ev, unsigned evdata);
extern elog_output(void);
extern int elog_at_overflow(int action);
extern int UsElogSetup (char * name, int buf_size);
extern int UsElogSetupForAutoOutput (char * name, int buf_size);
extern UsElogAtOverflow (int action);
extern UsElogOutput (void);
/* ----- Helper functions useful for elog backtrace events ----- */
extern int caller(void);
extern int caller_caller(void);
extern int caller_caller_caller(void);
/* ----- system functions used by elog macros ----- */
extern int getrtc(void);
#else
extern elog_define ();
extern elog_define_to_file ();
extern int elog_init();
extern int elog_setup();
extern int elog_setup_for_auto_output();
extern int elog_create();
extern elog_finalize();
extern elog_output();
extern int elog_at_overflow();
extern int UsElogSetup ();
extern int UsElogSetupForAutoOutput ();
extern UsElogAtOverflow ();
extern UsElogOutput ();
/* ----- Helper functions useful for elog backtrace events ----- */
extern int caller();
extern int caller_caller();
extern int caller_caller_caller();
/* ----- system functions used by elog macros ----- */
extern int getrtc();
#endif

#ifdef ELOG

#define ELOG_DEFINE(code,name,S) elog_define (code, name, S)

#define ELOG_DEFINE_TO_FILE(elog_name,code,name,S) \
    elog_define (elog_name,code, name, S)

#define ELOG_INIT(name) elog_init (name, 1)

#define ELOG_SETUP(name,tag,buf_size) elog_setup (name, tag, buf_size)

# ifndef PSOS

#define ELOG_SETUP_FOR_AUTO_OUTPUT(name,tag,buf_size) elog_setup_for_auto_output (name, tag, buf_size)
# endif PSOS

#define ELOG_CREATE(name) elog_create (name)

#define ELOG_RESET() \
{   __elog_eptr = __elog_eptr_base; \
}

# ifdef BFLY2
# ifdef PSOS

# include <elog_trap.h>

# define ELOG_LOG(code,data) \
{ \
    int t; \
    unsigned int log_time [2];	/* = [time.hi, time.low] */ \
    if (__elog_eptr) \
    { \
	get64bitclock (log_time); \
	if (0 != (t = elog_trap (log_time[0], log_time[1], code, data))) \
	{ \
	    switch (t) \
	    { \
	    case elog_trap_ret_BUF_FULL: \
		elog_finalize (code, data); \
		break; \
	    case elog_trap_ret_BUF_BUSY: \
		do \
		{ \
		    if (0 == (t = elog_trap (log_time[0], log_time[1], code, data))) \
			break; \
		    if (elog_trap_ret_BUF_FULL == t) \
		    { \
			elog_finalize (code, data); \
			break; \
		    } \
		    tm_wkafter (1); \
		} while (1); \
		break; \
	    case elog_trap_ret_DISABLED: \
		break; \
	    default: \
		elog_log_error (t); \
		break; \
	    } \
	} \
    } \
}

# define ELOG_ISR_LOG(code,data) \
{ \
    unsigned int log_time [2];	/* = [time.hi, time.low] */ \
    if (__elog_eptr) \
    { \
	get64bitclock (log_time); \
	elog_trap (log_time[0], log_time[1], code, data); \
    } \
}

# else PSOS
#define ELOG_LOG(ev,evdata) \
    {   register int * xeptr = __elog_eptr; \
        if (xeptr < __elog_eptr_max) \
	{   get64bitclock (xeptr); \
	    xeptr += 2; \
	    *(xeptr++) = ev; \
	    *(xeptr++) = (int) (evdata); \
	    __elog_eptr = xeptr; \
	} \
	else \
	    elog_finalize (ev,evdata); \
    }
# endif PSOS

#define ELOG_FAST_LOG(ev,evdata) \
  { get64bitclock (__elog_eptr); \
    __elog_eptr += 2; \
    *(__elog_eptr++) = ev; \
    *(__elog_eptr++) = evdata; \
  }
# else BFLY2
#define ELOG_LOG(ev,evdata) \
  { if (__elog_eptr < __elog_eptr_max) \
    {	*(__elog_eptr++) = getrtc(); \
	*(__elog_eptr++) = ev; \
	*(__elog_eptr++) = evdata; \
    } \
    else \
	elog_finalize (ev,evdata); \
  }

#define ELOG_FAST_LOG(ev,evdata) \
  { *(__elog_eptr++) = getrtc(); \
    *(__elog_eptr++) = ev; \
    *(__elog_eptr++) = evdata; \
  }
# endif BFLY2

#define ELOG_OUTPUT() elog_output ()

#define ELOG_AT_OVERFLOW(action) elog_at_overflow (action)

#define US_ELOG_SETUP(name,buf_size) UsElogSetup (name,buf_size)

# ifndef PSOS

#define US_ELOG_SETUP_FOR_AUTO_OUTPUT(name,buf_size) UsElogSetupForAutoOutput (name,buf_size)
# endif PSOS

#define US_ELOG_AT_OVERFLOW(action) UsElogAtOverflow(action)

#define US_ELOG_OUTPUT() UsElogOutput ()

#else ELOG

#define ELOG_DEFINE(code,name,S)
#define ELOG_DEFINE_TO_FILE(filename,code,name,S)
#define ELOG_INIT(name)
#define ELOG_SETUP(name,tag,buf_size)
#define ELOG_CREATE(name)
#define ELOG_RESET()
#define ELOG_LOG(ev,evdata)
#define ELOG_FAST_LOG(ev,evdata)
#define ELOG_OUTPUT()
#define ELOG_AT_OVERFLOW(action)
#define US_ELOG_SETUP(name,buf_size)
# ifndef PSOS
#define ELOG_SETUP_FOR_AUTO_OUTPUT(name,tag,buf_size)
#define US_ELOG_SETUP_FOR_AUTO_OUTPUT(name,buf_size)
# endif PSOS
#define US_ELOG_AT_OVERFLOW(action)
#define US_ELOG_OUTPUT()
#ifdef PSOS
#define ELOG_ISR_LOG(ev,evdata)
#endif PSOS
#endif ELOG

extern int * __elog_eptr;
extern int * __elog_eptr_base;
extern int * __elog_eptr_max;

# ifdef BFLY2
typedef struct
{   unsigned		hi;
    unsigned		low;
} BF2_64BIT_TIME;

typedef struct
{   BF2_64BIT_TIME	time;
    int			code;
    int			data;
} EventLogEntry;
# else BFLY2
typedef struct
{   int		time;
    int		code;
    int		data;
} EventLogEntry;
# endif BFLY2

typedef struct
{   int		size;		/* Size of entry in bytes */
    int		code;		/* The event code */
    char	name [1];    	/* Name of the event (null terminated), followed */
				/* by an arbitrary string (format spec for data) */
} EventCodeDefn;

typedef struct
{   int			n_logs;
    long		flags;
#define elhMicroSecs		0x1
#define elh64BitTimes		0x2
#define elhSequentialFormat 0x80000000
    int			format_version;
	/* The first format version does not have an explicit version number.
	 * the version number is implicit in that the length of the header
	 * is only 8 bytes.  Version 2 and higher have an explicit version
	 * number in the header.
	 * Version History:
	 *     V1.  The original version.  Used on the Butterfly 1 (Butterfly
	 *	    Plus and GP1000) through Mach Release 2.0.
	 *     V2.  ...
	 */
} EventLogHeader;

/*
 * Declarations for new format (sequential) elog files.
 */

# define ELOG_FORMAT_VERSION	2

# define ELOG_HEADER_BLOCK	1
# define ELOG_DEF_BLOCK		2
# define ELOG_TRACE_BLOCK	3

typedef struct
{					/* Header common to all blocks */
    int			size;		/* Block size - exclusive of size field */
    unsigned		type;		/* Block type. */
} ElogHeaderHeader;

typedef struct
{					/* Elog file header block - must be first */
    ElogHeaderHeader	hdr;
    unsigned		flags;		/* Same as EventLogHeader.flags (above) */
    int			version;	/* Version of the format. */
} ElogHeaderBlock;

typedef struct
{					/* Defn block - defines strings for gist. */
    ElogHeaderHeader	hdr;
    int			code;
    char		strings [4];
} ElogDefBlock;

typedef struct
{					/* Trace block. */
    ElogHeaderHeader	hdr;
    unsigned		trace_id;	/* Trace id - set by elog_setup */
    unsigned		flags;
    unsigned		offset;		/* Offset of events from end of hdr.size */
    unsigned		data [1];
} ElogTraceBlock;

/* ----- end of file marker ----- */
#endif _ELOG_H

