/* $Header: /nfs/europa/u1/BFMACH.local/mk/bfly2/RCS/b2vme.h,v 1.39 91/01/28 11:36:01 gibson Exp $ */
#ifndef _B2VME_
#define _B2VME_

#ifdef KERNEL
#include "../bfly2/paddr.h"
#else KERNEL
#include <butterfly/paddr.h>
#endif KERNEL

/***********************************************************
* B2VME-SPECIFIC DEFINITIONS:
*
*	NB: This file does not include definitions relating
*	to Motorola hardware.
*	  MC88100 definitions are in psl.h (and elsewhere)
*	  MC88200 definitions are in mmu.h
***********************************************************/

/*
 *	ROM-specific definitions
 */

/* utility procedure vector in ROM */

#define Coldboot    0x09000         /* Enter XMD ROM as if at power up */
#define Uart_Init   0x09004         /* Initialize UART (both channels) */
#define Putc_Con    0x09008         /* Output char to console port */
#define Getc_Con    0x0900c         /* Input char from console port  */
#define Setb_Con    0x09010         /* Set baud rate of console port*/
#define Putc_Host   0x09014         /* Output char to host port */
#define Getc_Host   0x09018         /* Input char from host port */
#define Setb_Host   0x0901c         /* Set baud rate of host port */
#define Set_Light   0x09020         /* Set Light on iff r2 .ne. 0 */

#define BRG_300     0x44            /* Some rates for 2681, BRG set 2 */
#define BRG_9600    0xbb
#define BRG_19200   0xcc

/* Physical address allocation */

#define PA_LO_ROM_VEC   0x00000000  /* Rom vector table at zero */
#define PA_HI_ROM_VEC   0x00008000  /* Rom vector table if page zero is ram */

#define	B2VME_REGS_PADDR	0xE0700000
#define	B2VME_REGS_LARGE_PADDR	0xFF700000

/* 
 * note that the rest of the E07 addresses in this file 
 * are used as vaddrs only.  the regs vaddr doesn't change, only the paddr.
 */

/*
 *	LOCAL I/O: registers and definitions
 *		Note: Correct only for small machine
 *		(.lt. 64 switch port) systems
 */

    /* "User" registers */

#define _IO_CPU_INT      0x80002000  /* Interrupt request */
#define _IO_PROC_CFG     0xE0740000  /* Process Configuration */
#define _IO_LW_STEAL     0xE0741000  /* Lightweight Steal */
#define _IO_AUG_BLOCK    0xE0758000  /* Base of augmentation register block */
				     /* through E07E0000 */

    /* process configuration register definitions */
#define	PCR_SIGA_A	0x1
#define	PCR_SIGA_B	0x2

    /* T-bus priority bits */
#define	T_PRI_EXPR	0x0
#define	T_PRI_FORE	0x1
#define	T_PRI_BACK	0x2

#define	MACH_PCR_DEFAULT	 ((PCR_SIGA_A << 4) | (T_PRI_EXPR << 2))

    /* Augmentation register bits */

#define AR_STEAL        0x2000      /* Steal
#define AR_SYNC         0x1000      /* Synchronized access */
#define AR_SNEAK        0x0800      /* Sneak */
#define AR_LOCK         0x0400      /* Lock */
#define AR_EXC_ACT      0x0100      /* Exception action (2 bits) */
#define AR_DIS_INT      0x0080      /* Disable Interrupts */
#define AR_ABS_PRI      0x0010      /* Absolute Priority (2 bits) */
#define AR_BANK         0x0004      /* Bank number (2 bits) */

    /* Per Switch-port registers */

#define _IO_MACHINE_CFG  0xE0760000  /* Machine switch size and port */
#define _IO_NMI          0xE0760004  /* Non-maskable interrupt */
#define _IO_INT_PEND     0xE0760008  /* Interrupt Pending and Abort-retry timer */

    /* Machine configuration register definitions */
#define	MACHCFG_FUNCTION_MASK		0x7	/* low bits enable functions */
#define	MACHCFG_FASTPATH_ENABLE		4	/* only on Rev A, B, C boards*/
#define	MACHCFG_SHORTCIRCUIT_DISABLE	1	/* only on Rev A, B boards */
#define	MACHCFG_CMRAM_ENABLE		1	/* only on Rev C and up */
#define	MACHCFG_PROC_SHIFT		4	/* processor number (only on Rev A, B boards) */


    /* VME control registers */

#define _VME_SLAVE_MAP	       0xE0720000 /* Base of VME Slave mapper */
#define _VME_MBUS_TIMER_REG    0xE0760200 /* VME Master BERR timer */
#define _VME_CONFIG_REG	       0xE0760204 /* VME Configuration register */
#define _VME_SBUS_TIMER_REG    0xE0760208 /* VME System (slave) BERR timer */
#define _VME_ARBITER_TIMER_REG 0xE076020C /* VME Arbiter timer */
#define _VME_INT_VECTOR_REG    0xE0760800 /* VME Interrupt Gen control,B14-B8*/
#define _VME_INT_REQUEST_REG   0xE0760804 /* VME Interrupt Gen level, B14-B8 */

    /* VME CONFIGURATION REGISTER definitions */

#define VME_RESET		0x00000000
#define VME_NO_RESET		0x00002000

#define VME_RELEASE_WHEN_DONE	0x00001000
#define VME_RELEASE_ON_REQUEST	0x00000000

#define VME_STANDARD		0x00000800
#define VME_EXTENDED		0x00000000

#define VME_SYSFAIL		0x00008000	/* high true */

/*
 * VME Configuration register specifies locaton of "slave window".
 * Slave window starts at VME_SLAVE_START in vme space.
 */
#define VME_SLAVE_WINDOW	(VME_SLAVE_START >> 21)

    /* VME SLAVE MAP definitions */

#define VSM_AD32        0x0001      /* Address 33-32,           B1-B0 */
#define VSM_PATH        0x0004      /* Path bits,    		B3-B2 */
#define VSM_INTERLEAVE  0x0010      /* Interleave enable, 	B4    */
#define VSM_PRIORITY    0x0020      /* Switch priority, 	B6-B5 */
#define VSM_LOCK        0x0080      /* Locked accesses, 	B7    */
#define VSM_BYPASS      0x0100      /* Bypass access,           B8    */
#define VSM_AD13        0x2000      /* Address 31-13 (pages),   B13   */

/*
 * The number of slave map entries in use is determined by the VMEbus
 * Configuration Register standard/extended bit:
 * The specific set of 512 entries in the standard case is determined
 * by the slave window bits in the VMEbus Configuration Register.
 * 	standard:  512 entries
 *	extended: 2048 entries
 */

#define NUM_SMAP_ENTRIES	2048	/* Total # of slave map registers */
					/* (Enough to map 16 M) */

#define SMAP_ENTRY_SIZE		8192	/* Size of area mapped per register */
					   
   /* Slave area 4M of Butterfly physical memory into VME address space,
    * starting at 4M in VME address space.
    */
#define VME_SLAVE_START		0x400000
#define	VME_SLAVE_SIZE		0x400000
#define VME_SLAVE_END  		(VME_SLAVE_START + VME_SLAVE_AREA_SIZE)

/* These definitions are for dynamically allocated vme slave mapping ram
 * registers.  When not allocated, the registers are maintained in a
 * linked list running through the registers themselves.
/* Since the vme slave mapping ram registers aren't a full
 * 32 bits (some of the bits are don't cares),  we used only the good
 * bits, which implies masking off the garbage and doing index chaining.
 *
 * Currently, our invalid "end" value of 0x1FF gets us near to 4M.  That
 * may not be enough forever.
 */
#define	VME_MAP_CHAIN_MASK	0x1FF
#define	VME_MAP_CHAIN_END	0x1FF

/* These definitions describe the allocation method. */

#define VME_SLAVE_MAP_ALLOC_EMPTY -1
#define VME_SLAVE_MAP_ALLOC_OK	   0
#define VME_SLAVE_MAP_ALLOC_WAIT_I 0x1
#define VME_SLAVE_MAP_ALLOC_WAIT_P 0x2
#define VME_SLAVE_MAP_ALLOC_NOWAIT 0x4

/*
 * Currently, the VMEbus Slave Map RAM registers are set up as follows:
 *
 *  	bypass   	1	- for speed and to prevent bus errors
 *	lock T-bus	0	- If we eventually bypass locks, we don't
 *				  want to take a lock
 *	T_PRIORITY	0	- Access will be local, so switch message
 *				  priority doesn't apply
 *	interleaved	0	- For now, we have no interleaved memory
 *	T_PATH<1..0>   11	- Indicating local access
 */	
#define SLAVE_MAP_TEMPLATE	0x10C  /* bypass and appropriate path */

    /* VME MASTER MAP definitions */

#define VMM_NIACK       0x02000000  /* (not)IACK bit to VME bus B25   */
#define VMM_NMAP        0x400       /* Number of map entries, 1K      */
#define VMM_AM_EXT      0x00680000  /* Adr Mode Extended supervisory data */
#define VMM_AM_STD      0x01E80000  /* Adr Mode Standard supervisory data */
#define VMM_AM_SHORT    0x01680000  /* Adr Mode Short Supervisory     */

#define NO_IACK		0x02000000
#define IACK		0x00000000

#define MMAP_ENTRY_SIZE			8192
#define NUM_MMAP_ENTRIES		512
     /* 64K for 16 bit address space (first registers) */
#define NUM_MMAP_SHORT_ENTRIES		((64*1024)/MMAP_ENTRY_SIZE)
     /* Last 8K of VME space will be shadowed (used for IACK instead of VME) */
#define NUM_MMAP_IACK_ENTRIES		1
     /* Everything else is a standard entry */
#define NUM_MMAP_STANDARD_ENTRIES	\
 	(NUM_MMAP_ENTRIES - (NUM_MMAP_SHORT_ENTRIES + NUM_MMAP_IACK_ENTRIES))


    /* Interrupt Enable Mask Register definitions */

#define ALL_INTS	0xFF /* Intr enable mask: ALL interrupts */
#define ALL_VME_INTS	0xFE /* Intr enable mask: all BUT interrproc intr */
#define IPI_INTS	0x01 /* enable maskable InterProcessor Interrupts */
#define VME_LEVEL1_INTS	0x02 /* enable VMEbus level 1 interrupts */
#define VME_LEVEL2_INTS	0x04 /* enable VMEbus level 2 interrupts */
#define VME_LEVEL3_INTS	0x08 /* enable VMEbus level 3 interrupts */
#define VME_LEVEL4_INTS	0x10 /* enable VMEbus level 4 interrupts */
#define VME_LEVEL5_INTS	0x20 /* enable VMEbus level 5 interrupts */
#define VME_LEVEL6_INTS	0x40 /* enable VMEbus level 6 interrupts */
#define VME_LEVEL7_INTS	0x80 /* enable VMEbus level 7 interrupts */

    /* Supervisor registers */

#define _IO_CPU_LOCK_TMR     0xE0780000    /* CPU Lock timer (WR) */
#define _IO_AUG_REGISTER     0xE0780004    /* Augmentation Register */
#define _IO_AUG_MASK         0xE0780008    /* Augmentation Mask */
#define _IO_BERR_VEC         0xE078000C    /* Bus error vector */
#define _IO_PCR_DSBLE_MASK   0xE0780010    /* Process Configuration Register */
#define _IO_INT_MASK         0xE0780014    /* Interrupt Mask */
#define _INT_ENABLE_MASK_REG 0xE0780014    /* MACH name */
#define _IO_INT_SRC          0xE0780018    /* Interrupt Source register (RD) */
#define _INT_SOURCE_REG	     0xE0780018	   /* MACH name */
#define _IO_INT_DIS_TMR      _IO_INT_SRC   /* Interrupts Disabled Timer (WR) */
#define _IO_WRITE_ERR        0xE078001C    /* Write Error Source */
#define _IO_AUG_RD_AND_CLR   _IO_WRITE_ERR /* Augmentation read/clear (RD) */


    /* Interrupt source bits */

#define ISRC_VME        0x01            /* Source is VME bus */
#define ISRC_VME_LEV    0x0E            /* Three bits of level number */
#define ISRC_NMI        0x10            /* Source is inter-proc non-maskable */
#define ISRC_IPI        0x20            /* Source is inter-proc int req */
#define ISRC_RTC        0x40            /* Source is real time clock */
#define ISRC_ITMR       0x80            /* Source is int disabled timeout */

    /* Time of day constant for TONI interrupts */

#define TOD_INTERVAL    1000000         /* One second */

/*
 * Interleaver Loader:
 *
 *	The Interleaver Loader is used to load a) the interleaver mapping
 *	ram, b) the interleaver modulus ram, c) the cpu mapping ram,
 *	d) the vme master mapping ram.
 *
 *	(The interleaver loader is part of the SIGA.  For historical
 *	reasons, we aren't careful to distinguish that these are SIGA A
 *	registers.)
 */

#define _IAR		0xE07EA00C	/* Interleaver Address Register */
#define _IDR		0xE07EB000	/* Interleaver Data Register */
#define _WRITE_MMAP_RAM	0xE07EA008	/* IDR --> VME-Master-Map-RAM */
#define _READ_MMAP_RAM	0xE07EA028	/* VME-Master-Map-RAM --> IDR */
#define	_CMRAM_WRITE	0xE07EA010	/* IDR --> CPU-Mapping-RAM */
#define	_CMRAM_READ	0xE07EA030	/* CPU-Mapping-RAM --> IDR */

	/*
	 *	CPU MAPPING RAM DEFINITIONS
	 */
#define	CMRAM_ENTRY_SIZE	(8*1024*1024)

    /* CMRAM: IAR side */
#define	CMRAM_WRITE_MASK		(1 << 14)	/* set write half */
#define	CMRAM_READ_MASK			0		/* set read half */
#define CMRAM_LOW_BANK_ADDR_MASK	0x3F800000
#define CMRAM_LOW_BANK_SHIFT		2
#define CMRAM_LOW_BANK_MASK		(CMRAM_LOW_BANK_ADDR_MASK \
					 >> CMRAM_INDEX_SHIFT)
#define CMRAM_LOW_BANK2IAR_SHIFT	25

#define CMRAM_HIGH_BANK_ADDR_MASK	0xC0000000
#define CMRAM_HIGH_BANK_SHIFT		30
#define CMRAM_HIGH_BANK_MASK		(CMRAM_HIGH_BANK_ADDR_MASK \
					 >> CMRAM_INDEX_SHIFT)
#define CMRAM_HIGH_BANK2IAR_SHIFT	7

    /* CMRAM: IDR side */

#define	CMRAM_FASTPATH_BAR_SHIFT 31			/* fast path to local */
#define	CMRAM_IACC_BAR_SHIFT	30			/* intercept access */
#define CMRAM_INTERLEAVE_SHIFT	29			/* interleave */
#define	CMRAM_BYPASS_SHIFT	28			/* bypass */
#define	CMRAM_LOCAL_SHIFT	27			/* local memory */
#define CMRAM_BAY_SHIFT		24			/* bay number */
#define CMRAM_NODE_SHIFT	18			/* node within bay */
#define	CMRAM_BANK_SHIFT	16			/* bank bits offset */

#define	CMRAM_FASTPATH_BAR	(1 << CMRAM_FASTPATH_BAR_SHIFT)
#define	CMRAM_IACC_BAR		(1 << CMRAM_IACC_BAR_SHIFT)
#define CMRAM_INTERLEAVE	(1 << CMRAM_INTERLEAVE_SHIFT)
#define	CMRAM_BYPASS		(1 << CMRAM_BYPASS_SHIFT)
#define	CMRAM_LOCAL		(1 << CMRAM_LOCAL_SHIFT)
#define CMRAM_BANK_MASK		(0x3 << CMRAM_BANK_SHIFT)

#define	CMRAM_MASK		0xFFFF0000 /* screen don't care bits on read */

#ifndef ASSEMBLER
struct cmram {
	unsigned int
      		fastpath_bar: 	1,	/* On Rev D boards and up */
      		intercept_bar:	1,
      		interleave:	1,
      		bypass:		1,
      		local:		1,
	  	bay:		3,
	  	node:		6,
      		bank:		2,
      		reserved2:	16;
	
};
#endif ASSEMBLER

/* use node 1, bank 3, non-bypassed for "intercept-access" */
#define INTERCEPT_CMR_INDEX_SMALL 0xC1
/* large model uses cmr index 1 */
#define INTERCEPT_CMR_INDEX_LARGE 0x1

/* use "node 2, bank 3, non-bypassed" to remap user segment table bypassed */
#define USR_SEGTABLE_INDEX_SMALL 0xC2
/* large model uses cmr index 2 */
#define USR_SEGTABLE_INDEX_LARGE 0x2

/* use "node 3, bank 3, non-bypassed" to remap super seg table bypassed */
#define SUPER_SEGTABLE_INDEX_SMALL 0xC3
/* large model uses cmr index 3 */
#define SUPER_SEGTABLE_INDEX_LARGE 0x3

/* current in use definitions now reside in locore_vec.s */
#ifndef ASSEMBLER
extern short intercept_bank, usr_segtable_bank, super_segtable_bank;
#endif ASSEMBLER

/* T-bus address manipulation */
#define TBUS_PNODE_ADDR_SHIFT	25
#define PNODE2TADDRPK(pnode)	(pnode << (TBUS_PNODE_ADDR_SHIFT - 2))
/* This macro creates a tbus address shifted right 2 bits */
#define PADDR2TADDRPK(paddr)	(PNODE2TADDRPK(PADDR2PNODE(paddr))	\
				 | (paddr2memoffset(paddr) >> 2))

/*
 *	SIGA:	registers and definitions
 */

#define SIGA_A		0xE07E0000 /* SIGA-A Base Address */
#define SIGA_B          0xE07C0000 /* SIGA-B Base Address */

/* Offsets of SIGA registers from SIGA Base Address */
#define SIGA_TONIAC	0x0000 /* TONI-A Config reg */
#define SIGA_TONIA	0x0004 /* TONI-A */
#define SIGA_TONIBC	0x2000 /* TONI-B Config reg */
#define SIGA_TONIB	0x2004 /* TONI-B */
#define SIGA_POTC	0x8000 /* Protocol Timer Config */
#define SIGA_TTC	0x8004 /* Transmit Timer Config */
#define SIGA_PITC	0x8008 /* Priority Timer Config */
#define SIGA_RCA	0x800C /* Requestor Config A */
#define SIGA_RCB	0x8010 /* Requestor Config B */
#define SIGA_RT		0x8014 /* Requestor Test */
#define SIGA_RTC	0x8018 /* Real Time Clock */
#define SIGA_SCA	0x9000 /* Server Config A */
#define SIGA_SCB	0x9004 /* Server Config B */
#define SIGA_ST		0x9008 /* Server Test */
#define SIGA_WPR	0xA000 /* IDR->pool-ram (RO) */
#define SIGA_WMR	0xA004 /* IDR->modulus-ram (RO) */
#define SIGA_WVMM	0xA008 /* IDR->vme-master-map (RO) */
#define SIGA_IAR	0xA00C /* Interleaver Address register */
#define SIGA_WDR	0xA014 /* IDR->decision-ram (RO) */
#define SIGA_RPR	0xA020 /* pool-ram->IDR (RO) */
#define SIGA_RMR	0xA024 /* modulus-ram->IDR (RO) */
#define SIGA_RVMM	0xA028 /* vme-master-map->IDR (RO) */
#define SIGA_RDR	0xA034 /* decision-ram->IDR (RO) */
#define SIGA_IDR	0xB000 /* Interleaver Data Register */

/*
 *	SIGA REQUESTER CONFIGURATION REGISTER A (RCA).
 */

#define RCA_RQSLV_A 0x20000000 /* Requestor slave number for SIGA-A */
#define RCA_RQSLV_B 0x40000000 /* Requestor slave number for SIAG-B */

#define RCA_8x1		0x18000001 /* Modulo-8, One column switch */
#define RCA_8x2		0x18000000 /* Modulo-8, Two column switch */
#define RCA_8x3		0x10000000 /* Modulo-8, Three column switch */
#define RCA_16x1	0x08000001 /* Modulo-16, One column switch */
#define RCA_16x2	0x08000000 /* Modulo-16, Two column switch */
#define RCA_16x3	0x00000000 /* Modulo-16, Three column switch */
#define RCA_SWITCH_CONF	0x18000001 /* covers all three switch conf bits */

#define RCA_AUTODROP	0x04000000 /* Enable auto path teardown on error */

#define RCA_FQ1		0x00800000 /* Start XMIT on TBUS request */
#define RCA_FQ2		0x01000000 /* Start XMIT after word 1 */
#define RCA_FQ3		0x01800000 /* Start XMIT after word 2 */
#define RCA_FQ4		0x02000000 /* Start XMIT after word 3 */
#define RCA_FQ5		0x02800000 /* Start XMIT after word 4 */

#define RCA_STU0	0x00000000 /* STU Synchronization 0.0 clocks */
#define RCA_STU0H	0x00080000 /* STU Synchronization 0.5 clocks */
#define RCA_STU1	0x00100000 /* STU Synchronization 1.0 clocks */
#define RCA_STU1H	0x00180000 /* STU Synchronization 1.5 clocks */
#define RCA_STU2	0x00200000 /* STU Synchronization 2.0 clocks */
#define RCA_STU2H	0x00280000 /* STU Synchronization 2.5 clocks */
#define RCA_STU3	0x00300000 /* STU Synchronization 3.0 clocks */
#define RCA_STU3H	0x00380000 /* STU Synchronization 3.5 clocks */
#define RCA_STU4	0x00400000 /* STU Synchronization 4.0 clocks */
#define RCA_STU4H	0x00480000 /* STU Synchronization 4.5 clocks */
#define RCA_STU5	0x00500000 /* STU Synchronization 5.0 clocks */
#define RCA_STU5H	0x00580000 /* STU Synchronization 5.5 clocks */

#define RCA_BIU0	0x00000000 /* BIU Synchronization 0.0 clocks */
#define RCA_BIU0H	0x00008000 /* BIU Synchronization 0.5 clocks */
#define RCA_BIU1	0x00010000 /* BIU Synchronization 1.0 clocks */
#define RCA_BIU1H	0x00018000 /* BIU Synchronization 1.5 clocks */
#define RCA_BIU2	0x00020000 /* BIU Synchronization 2.0 clocks */
#define RCA_BIU2H	0x00028000 /* BIU Synchronization 2.5 clocks */
#define RCA_BIU3	0x00030000 /* BIU Synchronization 3.0 clocks */
#define RCA_BIU3H	0x00038000 /* BIU Synchronization 3.5 clocks */
#define RCA_BIU4	0x00040000 /* BIU Synchronization 4.0 clocks */
#define RCA_BIU4H	0x00048000 /* BIU Synchronization 4.5 clocks */
#define RCA_BIU5	0x00050000 /* BIU Synchronization 5.0 clocks */
#define RCA_BIU5H	0x00058000 /* BIU Synchronization 5.5 clocks */

#define RCA_ERROR_A	0x00000000 /* Error prefix for SIGA-A (really 0!)*/
#define RCA_ERROR_B	0x00000000 /* Error prefix for SIGA-B (ditto) */

#define RCA_65MS0	0x00000000 /* 65 Ms Pulse delay = 0 clocks */
#define RCA_65MS1	0x00000200 /* 65 Ms Pulse delay = 1 clocks */
#define RCA_65MS2	0x00000400 /* 65 Ms Pulse delay = 2 clocks */
#define RCA_65MS3	0x00000600 /* 65 Ms Pulse delay = 3 clocks */

#define RCA_CSUSLV	0x00000000 /* CSU slave number for all SIGA's */

#define RCA_PS40	0x00000026 /* Prescaler for 40 MHz switch clock */
#define RCA_PS32	0x0000001E /* Prescaler for 32 MHz switch clock */
#define RCA_PS20	0x00000012 /* Prescaler for 20 MHz switch clock */
#define	RCA_PS16	0x0000000E /* prescaler for 16 MHz */

#define RCA_RESET	0x00000000 /* Reset state */
#define RCA_DEFAULTA	(RCA_RQSLV_+RCA_8x2+RCA_FQ1+RCA_STU2+RCA_BIU2+RCA_ERROR_A+RCA65MS0+RCA_CSUSLV+RCA_PS40)
#define RCA_DEFAULTB	(RCA_RQSLV_+RCA_8x2+RCA_FQ1+RCA_STU2+RCA_BIU2+RCA_ERROR_B+RCA65MS0+RCA_CSUSLV+RCA_PS40)

#define	RCA_STU_SHIFT	19
#define	RCA_BIU_SHIFT	15
#define	RCA_PRE_SHIFT	1

/*
 *	SIGA REQUESTER CONFIGURATION REGISTER B (RCB).
 */

#define RCB_RAM		0xFF800000 /* Route address mask (1 = Random) */
#define	RCB_RAM1A	0x10000000 /* randomize one path, option A */
#define RCB_RAM1B	0x08000000 /* randomize one path, option B */
#define RCB_RAM1C	0x04000000 /* randomize one path, option C */
#define RCB_RAM2A	0x18000000 /* randomize two paths, option A */
#define RCB_RAM2B	0x0C000000 /* randomize two paths, option B */
#define RCB_RAM2C	0x14000000 /* randomize two paths, option C */
#define RCB_RAM_NONE	0x00000000

#define RCB_ESVE	0x00400000 /* Enable stolen verify error */
#define RCB_EMAE	0x00200000 /* Enable maintain absent error */
#define RCB_EMPE	0x00100000 /* Enable maintain present error */
#define RCB_ELAE	0x00080000 /* Enable lock address error */
#define RCB_EWTOE	0x00040000 /* Enable wait timeout error */
#define RCB_EITOE	0x00020000 /* Enable idle timeout error */
#define RCB_ERAE	0x00010000 /* Enable reject abort error */
#define RCB_ERTOE	0x00008000 /* Enable reject timeout error */
#define RCB_ECE		0x00004000 /* Enable check error */
#define RCB_ERVE	0x00002000 /* Enable reverse error */
#define RCB_ERME	0x00001000 /* Enable remote error */
#define RCB_ECSUFQE	0x00000800 /* Enable CSU function request error */
#define RCB_EPP		0x00000400 /* Enable priority promotion */
#define RCB_EI		0x00000200 /* Enable interleaver */
#define RCB_ERA		0x00000100 /* Enable reject abort */
#define RCB_ERT		0x00000080 /* Enable reject timer */
#define RCB_ECT		0x00000040 /* Enable connection timer */
#define RCB_ESF		0x00000020 /* Enable switch frame */
#define RCB_ERBIU	0x00000010 /* Enable requestor bus interface unit */
#define RCB_ERSTU	0x00000008 /* Enable requestor switch transmitter */
#define RCB_ERSRU	0x00000004 /* Enable requestor switch receiver */
#define RCB_ERCNT	0x00000002 /* Enable requestor counter unit */

#define RCB_RESET	0x00000000 /* Reset and disable all */
#define RCB_DEFAULT	0x007FFFFE /* Enable everything */

/*
 *	SIGA SERVER CONFIGURATION REGISTER A (SCA).
 */

#define SCA_EWHS	0x80000000 /* Enable write head start */
#define SCA_EBHS	0x40000000 /* Enable byte head start */

#define SCA_MHS0	0x00000000 /* Multiple head start (off) */
#define SCA_MHS1	0x10000000 /* Multiple head start (1 word) */
#define SCA_MHS2	0x20000000 /* Multiple head start (2 words) */
#define SCA_MHS3	0x30000000 /* Multiple head start (3 words) */
#define SCA_RICS	0x0F000000 /* Initial checksum value for this node */

#define SCA_EPA		0x000C0000 /* Error prefix code for SIGA-A */
#define SCA_EPB		0x000C0000 /* Error prefix code for SIGA-B */

#define SCA_EBIU	0x00020000 /* Enable bus interface unit */
#define SCA_ESRU	0x00010000 /* Enable server receiver */

#define SCA_STUF0	0x00000000 /* STU freed synchronization 0.0 clocks */
#define SCA_STUF0H	0x00001000 /* STU freed synchronization 0.5 clocks */
#define SCA_STUF1	0x00002000 /* STU freed synchronization 1.0 clocks */
#define SCA_STUF1H	0x00003000 /* STU freed synchronization 1.5 clocks */
#define SCA_STUF2	0x00004000 /* STU freed synchronization 2.0 clocks */
#define SCA_STUF2H	0x00005000 /* STU freed synchronization 2.5 clocks */
#define SCA_STUF3	0x00006000 /* STU freed synchronization 3.0 clocks */
#define SCA_STUF3H	0x00007000 /* STU freed synchronization 3.5 clocks */
#define SCA_STUF4	0x00008000 /* STU freed synchronization 4.0 clocks */
#define SCA_STUF4H	0x00009000 /* STU freed synchronization 4.5 clocks */
#define SCA_STUF5	0x0000A000 /* STU freed synchronization 5.0 clocks */
#define SCA_STUF5H	0x0000B000 /* STU freed synchronization 5.5 clocks */

#define SCA_STUD0	0x00000000 /* STU done synchronization 0.0 clocks */
#define SCA_STUD0H	0x00000100 /* STU done synchronization 0.5 clocks */
#define SCA_STUD1	0x00000200 /* STU done synchronization 1.0 clocks */
#define SCA_STUD1H	0x00000300 /* STU done synchronization 1.5 clocks */
#define SCA_STUD2	0x00000400 /* STU done synchronization 2.0 clocks */
#define SCA_STUD2H	0x00000500 /* STU done synchronization 2.5 clocks */
#define SCA_STUD3	0x00000600 /* STU done synchronization 3.0 clocks */
#define SCA_STUD3H	0x00000700 /* STU done synchronization 3.5 clocks */
#define SCA_STUD4	0x00000800 /* STU done synchronization 4.0 clocks */
#define SCA_STUD4H	0x00000900 /* STU done synchronization 4.5 clocks */
#define SCA_STUD5	0x00000A00 /* STU done synchronization 5.0 clocks */
#define SCA_STUD5H	0x00000B00 /* STU done synchronization 5.5 clocks */

				   /* BUS INTERFACE UNIT */
#define SCA_BIUF0	0x00000000 /* BIU free synchronization 0.0 clocks */
#define SCA_BIUF0H	0x00000010 /* BIU free synchronization 0.5 clocks */
#define SCA_BIUF1	0x00000020 /* BIU free synchronization 1.0 clocks */
#define SCA_BIUF1H	0x00000030 /* BIU free synchronization 1.5 clocks */
#define SCA_BIUF2	0x00000040 /* BIU free synchronization 2.0 clocks */
#define SCA_BIUF2H	0x00000050 /* BIU free synchronization 2.5 clocks */
#define SCA_BIUF3	0x00000060 /* BIU free synchronization 3.0 clocks */
#define SCA_BIUF3H	0x00000070 /* BIU free synchronization 3.5 clocks */
#define SCA_BIUF4	0x00000080 /* BIU free synchronization 4.0 clocks */
#define SCA_BIUF4H	0x00000090 /* BIU free synchronization 4.5 clocks */
#define SCA_BIUF5	0x000000A0 /* BIU free synchronization 5.0 clocks */
#define SCA_BIUF5H	0x000000B0 /* BIU free synchronization 5.5 clocks */

#define SCA_BIUX0	0x00000000 /* BIU XFER synchronization 0.0 clocks */
#define SCA_BIUX0H	0x00000001 /* BIU XFER synchronization 0.5 clocks */
#define SCA_BIUX1	0x00000002 /* BIU XFER synchronization 1.0 clocks */
#define SCA_BIUX1H	0x00000003 /* BIU XFER synchronization 1.5 clocks */
#define SCA_BIUX2	0x00000004 /* BIU XFER synchronization 2.0 clocks */
#define SCA_BIUX2H	0x00000005 /* BIU XFER synchronization 2.5 clocks */
#define SCA_BIUX3	0x00000006 /* BIU XFER synchronization 3.0 clocks */
#define SCA_BIUX3H	0x00000007 /* BIU XFER synchronization 3.5 clocks */
#define SCA_BIUX4	0x00000008 /* BIU XFER synchronization 4.0 clocks */
#define SCA_BIUX4H	0x00000009 /* BIU XFER synchronization 4.5 clocks */
#define SCA_BIUX5	0x0000000A /* BIU XFER synchronization 5.0 clocks */
#define SCA_BIUX5H	0x0000000B /* BIU XFER synchronization 5.5 clocks */

#define SCA_RESET	0          /* Reset state for Server config A */
#define SCA_DEFAULTA (SCA_EWHS+SCA_EBHS+SCA_MHS1+SCA_RICS+SCA_EPA+SCA_EBIU+SCA_ESRU+SCA_STUF2+SCA_STUD2+SCA_BIUF2+SCA_BIUX2)
#define SCA_DEFAULTB (SCA_EWHS+SCA_EBHS+SCA_MHS1+SCA_RICS+SCA_EPB+SCA_EBIU+SCA_ESRU+SCA_STUF2+SCA_STUD2+SCA_BIUF2+SCA_BIUX2)

#define	SCA_XSUM_SHIFT	24
#define	SCA_STUF_SHIFT	12
#define	SCA_STUD_SHIFT	8
#define	SCA_BIUF_SHIFT	4
#define	SCA_BIUX_SHIFT	0

/*
 *	SIGA SERVER CONFIGURATION REGISTER B (SCB).
 */

#define SCB_DF		0x00000020 /* Disable frame */
#define SCB_EF		0x00000000 /* Enable frame */

#define SCB_ESOC	0x00000010 /* Enable start of connection */

#define SCB_DCE		0x00000008 /* Disable checksum errors */
#define SCB_ECE		0x00000000 /* Enable checksum errors */

#define SCB_SN_A    	0x00000001 /* Server slave number for SIGA-A */
#define SCB_SN_B    	0x00000002 /* Server slave number for SIGA-B */

#define SCB_RESET	0	   /* Server config B reset state */
#define SCB_DEFAULTA (SCB_EF+SCB_ESOC+SCB_ECE+SCB_SN_A)
#define SCB_DEFAULTB (SCB_EF+SCB_ESOC+SCB_ECE+SCB_SN_B)

/*
 *	SIGA MESSAGE CLASSIFICATION REGISTER (MCR)
 *	(NB: MCR is the low 16 bits of protocol timer configuration register.)
 */

/* Possible strategies for messages */
#define	MCR_RAND1	3
#define	MCR_RAND0	2
#define	MCR_SLOT1	1
#define	MCR_SLOT0	0

/*	bit shifts to reach the various MCR fields.  A different MCR strategy
 *	can be set for each of these message TYPES.
 */
#define	MCR_WRITES		0
#define	MCR_READS		2
#define	MCR_AUX_WRITES		4
#define	MCR_AUX_READS		6
#define	MCR_LOCK_WRITES		8
#define	MCR_LOCK_READS		10
#define	MCR_AUX_LOCK_WRITES	12
#define	MCR_AUX_LOCK_READS	14

/*
 *	SIGA PROTOCOL TIMER CONFIGURATION REGISTER (POTC)
 *	(NB: MCR is the low 16 bits of POTC)
 */

#ifndef ASSEMBLER
struct potc_config
{
    /* timers */
    unsigned int reject               :  4;
    unsigned int prescale             :  4;
    unsigned int connection           :  8;
    /* strategies for particular cycle types */
    unsigned int aux_lock_read_strat  :	 2;
    unsigned int aux_lock_write_strat :	 2;
    unsigned int lock_read_strat      :	 2;
    unsigned int lock_write_strat     :	 2;
    unsigned int aux_read_strat	      :	 2;
    unsigned int aux_write_strat      :	 2;
    unsigned int read_strat	      :	 2;
    unsigned int write_strat	      :	 2;
};
#endif ASSEMBLER
    
#define	POTC_CNT_SHIFT	28	/* reject timer */
#define POTC_PRE_SHIFT	24	/* prescaler */
#define POTC_CON_SHIFT	16	/* connection timer */
#define	POTC_MCR_SHIFT	0	/* message classification register */

/* Create a value appropriate for being written to the POTC */
#define	POTC_VAL(potc_rej, potc_pre, potc_con, potc_mcr)		\
  				((potc_rej) << POTC_CNT_SHIFT | \
				 (potc_pre) << POTC_PRE_SHIFT | \
				 (potc_con) << POTC_CON_SHIFT | \
				 (potc_mcr) << POTC_MCR_SHIFT)

/*	maximums for the protocol timer config	*/
#define	POTC_CON_MAX	0xFE
#define	POTC_PRE_MAX	0xE
#define	POTC_REJ_MAX	0xE

/* defaults for the protocol timer config */
#define	MACH_POTC_CON_DEFAULT	0x80		/* = 128 usec */
#define	MACH_POTC_PRE_DEFAULT	POTC_PRE_MAX	/* = 2**(E+1) = 32K usec */
#define	MACH_POTC_REJ_DEFAULT	POTC_REJ_MAX	/* = 14 * 32K = ~0.5 sec */

/*
 *	SIGA PRIORITY TIMER CONFIGURATION REGISTER (PITC)
 */
#ifndef ASSEMBLER
struct pitc_config
{
    unsigned short slot;
    unsigned short mask;
};
#endif ASSEMBLER

#define	PITC_SLOT_SHIFT	16
#define PITC_MASK_SHIFT	0
#define PITC_MASK_64	0xf800

#define	PITC_MIN_SLOT	0x0000
#define	PITC_MAX_MASK	0xFFFF
#define	MACH_PITC_DEFAULT	(PITC_MIN_SLOT << PITC_SLOT_SHIFT | \
				 PITC_MAX_MASK << PITC_MASK_SHIFT)

/*
 *	SIGA TRANSMIT TIMER CONFIGURATION REGISTER (TTC).
 */

#ifndef ASSEMBLER
struct ttc_config
{
    unsigned int rand1_immed  : 1;
    unsigned int rand1_mask   : 5;
    unsigned int rand1_accon  : 2;
    unsigned int rand0_immed  : 1;
    unsigned int rand0_mask   : 5;
    unsigned int rand0_accon  : 2;
    unsigned int slot1_immed  : 1;
    unsigned int slot1_period : 2;
    unsigned int slot1_phase  : 5;
    unsigned int slot0_immed  : 1;
    unsigned int slot0_period : 2;
    unsigned int slot0_phase  : 5;
};
#endif ASSEMBLER

#define TTC_4_0US_PERIOD 0
#define TTC_2_0US_PERIOD 1
#define TTC_1_0US_PERIOD 2
#define TTC_0_5US_PERIOD 3

#define TTC_RAND1_SHIFT	24
#define TTC_RAND0_SHIFT	16
#define TTC_SLOT1_SHIFT	8
#define TTC_SLOT0_SHIFT	0

#define TTC_IMMED	0x80
#define TTC_4_0US_SLOT	0x00
#define TTC_2_0US_SLOT	0x20
#define TTC_1_0US_SLOT	0x40
#define TTC_0_5US_SLOT	0x60


/*
 *	Miscellaneous SIGA registers
 */
#define _rtc				(SIGA_A + SIGA_RTC)
#define _TONIA_CONFIG			(SIGA_A + SIGA_TONIAC)
#define _TONIA				(SIGA_A + SIGA_TONIA)

#ifndef ASSEMBLER
/*
 *	C VERSIONS OF DEFINITIIONS.
 */

#define IO_CPU_INT		(*((unsigned long *)_IO_CPU_INT))

#define IO_PROC_CFG     	(*((unsigned long *)_IO_PROC_CFG))
#define IO_LW_STEAL		(*((unsigned long *)_IO_LW_STEAL))
#define IO_MACHINE_CFG		(*((unsigned long *)_IO_MACHINE_CFG))
#define IO_NMI			(*((unsigned long *)_IO_NMI))
#define IO_INT_PEND		(*((unsigned long *)_IO_INT_PEND))

#define IO_CPU_LOCK_TMR		(*((unsigned long *)_IO_CPU_LOCK_TMR))
#define IO_AUG_REGISTER		(*((unsigned long *)_IO_AUG_REGISTER))
#define IO_AUG_MASK		(*((unsigned long *)_IO_AUG_MASK))
#define IO_AUG_BLOCK		((unsigned long *)_IO_AUG_BLOCK)    /* array */
#define IO_BERR_VEC		(*((unsigned long *)_IO_BERR_VEC))
#define IO_PCR_DSBLE_MASK	(*((unsigned long *)_IO_PCR_DSBLE_MASK))
#define IO_INT_MASK		(*((unsigned long *)_IO_INT_MASK))
#define INT_ENABLE_MASK_REG	(*((unsigned long *)_IO_INT_MASK))
#define IO_INT_SRC		(*((unsigned long *)_IO_INT_SRC))
#define INT_SOURCE_REG		(*((unsigned long *)_IO_INT_SRC))
#define IO_INT_DIS_TMR		(*((unsigned long *)_IO_INT_DIS_TMR))
#define IO_WRITE_ERR		(*((unsigned long *)_IO_WRITE_ERR))
#define IO_AUG_RD_AND_CLR	(*((unsigned long *)_IO_AUG_RD_AND_CLR)

#define VME_SLAVE_MAP		((unsigned long *)_VME_SLAVE_MAP)   /* array */
#define VME_MBUS_TIMER_REG	(*((unsigned long *)_VME_MBUS_TIMER_REG))
#define VME_SBUS_TIMER_REG	(*((unsigned long *)_VME_SBUS_TIMER_REG))
#define VME_CONFIG_REG		(*((unsigned long *)_VME_CONFIG_REG))
#define VME_ARBITER_TIMER_REG	(*((unsigned long *)_VME_ARBITER_TIMER_REG))
#define VME_INT_VECTOR_REG	(*((unsigned long *)_VME_INT_VECTOR_REG))
#define VME_INT_REQUEST_REG	(*((unsigned long *)_VME_INT_REQUEST_REG))

    /* VME SLAVE MAP definitions */
#define MASK_AD31to13		0xFFFFE000
#ifndef KERNEL
	/* This macro barely works.  Leave it around for simple-minded
	 * stand-alone programs.  Kernel should use SMRAM_FROM_ADDR().
	 */
#define ADDR_TO_ENTRY(x)   ((x) & MASK_AD31to13)
#endif KERNEL

    /* VME MASTER MAP definitions (loaded via the interleaver loader
     * registers defined with the rest of the siga registers.)
     */

/*
 * VME address modifier bits within master mapping register
 */
#define AM_SHORT_DEFAULT	0x2D  /* default short supervisory access */
#define AM_STANDARD_DEFAULT	0x3D  /* default std supervisory data access */
#define MMAP_AM_SHIFT		19    /* master map address modifier shift */
#define AM_SHORT		(AM_SHORT_DEFAULT << MMAP_AM_SHIFT)
#define AM_STANDARD		(AM_STANDARD_DEFAULT << MMAP_AM_SHIFT)

#define VME_ADDR_FIELD(x)	(((unsigned) (x)) >> 13)

	/* Index in master map of IACK entry */
#define	MMAP_IACK_ENTRY		0
	/* Offset in master map of IACK page */
#define	MMAP_IACK_OFFSET	0
	/* Virtual address of BATC IACK block */
#define VIRT_IACK_BATC_BLOCK	(MB_VIRT_START+(16*1024*1024)-CMMU_BATC_SIZE)
	/* Virtual address of IACK page */
#define VIRT_IACK_BLOCK		(VIRT_IACK_BATC_BLOCK+\
				 batc_offset(MMAP_IACK_OFFSET))

	/* Illegal physical address for Master window (for non-VME nodes) */
#define	MB_PHYS_NONE		0xFFFFFFFF

    /* SIGA definitions */
#define	SIGA_REG(siga,reg)	(*((unsigned *)((siga) + (reg))))
#define	SIGA_potc(siga)		SIGA_REG(siga, SIGA_POTC)
#define	SIGA_ttc(siga)		SIGA_REG(siga, SIGA_TTC)
#define	SIGA_pitc(siga)		SIGA_REG(siga, SIGA_PITC)

#define IAR			(*((unsigned long *)_IAR))
#define IDR			(*((unsigned long *)_IDR))
#define WRITE_MMAP_RAM		(*((unsigned long *)_WRITE_MMAP_RAM))
#define READ_MMAP_RAM		(*((unsigned long *)_READ_MMAP_RAM))
#define	CMRAM_READ		(*((unsigned long *)_CMRAM_READ))
#define	CMRAM_WRITE		(*((unsigned long *)_CMRAM_WRITE))

    /* RTC definitions */
#define rtc		(*(long *)_rtc)		/* full 32 bits */
#define rtc_high	(*(short *)_rtc)	/* only top 16 bits */
#define getrtclock() (*(long *)_rtc) /* function interface */

#define SECOND 1000000	/* also see 'tick' */
#define MILLISECOND (SECOND/1000)
#define MICROSECOND 1

/* WARNING: USE "LATER_THAN" TO COMPARE TWO TIMES, */
/* THE DIFFERENCE BETWEEN TWO TIMES IS A SIGNED NUMBER */
/* IF THIS SEEMS ODD, BE SURE YOU UNDERSTAND WHY - RDR */
#define LATER_THAN(a, b) ((b)-(a) & 0x80000000)/* true if a is later than b */

#endif ASSEMBLER

/*
 *  	Interleaver support.
 */

/* T_interleaved ram address manipulations */

#define T_INTRLV_HIGH_SHIFT	30
#define T_INTRLV_HIGH_MASK	(3 << T_INTRLV_HIGH_SHIFT)
#define T_INTRLV_MID_SHIFT	26
#define T_INTRLV_MID_MASK	(0xf << T_INTRLV_MID_SHIFT)
#define T_INTRLV_LOW_SHIFT	15
#define T_INTRLV_LOW_MASK	(0xFF << T_INTRLV_LOW_SHIFT)
#define T_INTRLV_MID_LOAD_SHIFT	28
#define T_INTRLV_DATA_SHIFT	15

/* Modulus ram address manipulations */

#define MODULUS_CLUMP_SHIFT	4
#define MODULUS_CLUMP_MASK	(0x1FF << MODULUS_CLUMP_SHIFT)
#define MODULUS_POOL_LOAD_SHIFT	13
#define MODULUS_OFFSET_SHIFT	23
#define MODULUS_OFFSET_MASK	(0x3F << MODULUS_OFFSET_SHIFT)
#define MODULUS_OFFSET_MASK_LOW	(0x7F << MODULUS_OFFSET_SHIFT)
#define MODULUS_OFFSET_SHIFT_HIGH 30
#define MODULUS_OFFSET_MASK_HIGH (0x3 << MODULUS_OFFSET_SHIFT_HIGH)
#define MODULUS_OFFSET_LOAD_SHIFT 2

/* General Interleaving */
#define INTRLV_CLUMP_SIZE	(1 << MODULUS_CLUMP_SHIFT)
#define POOL_DATA_SHIFT		12

/*
 *	MICROBOOT DEFINES.
 */

/*	micro boot request numbers	*/
#define	 MB_START	1	/* jsr to destination in mb_param */

/*	micro boot result numbers	*/
#define	 MB_ACCEPTED	1	/* request acccepted */
#define	 MB_IGNORED	2	/* request ignored (illegal) */

/*	address of the micro-boot data structure	*/
#define	 MB_DATA  	0x1700               

/*	The micro-boot data structure, itself	*/

#define	 MB_STATE 	(0)
#define	 MB_REQUEST	(MB_STATE+4)
#define	 MB_PARAM	(MB_STATE+8)
#define	 MB_RESULT	(MB_STATE+0xC)
#define  MB_RESERVED	(MB_STATE+0x14)
#define	 END_MB_DATA	(MB_STATE+0x30) 	

/* Miscellaneous MACH-specific definitions.  These belong elsewhere. */
#define _INTERPROCESSOR_INTERRUPT	0xFE002000
#define _SECBOOT_FLAG			0xFE00FFFC

/* Area for Communication between Bootstrap and Mach.
 * These belong elsewhere
 */
#define BOOT_PARMS	0x0000E000	/* last 8K of the 64K area available */
#define uaboot		0x0		/* Mach autoboot flags	*/
#define ubparm		0x4		/* Mach boot parameter	*/
#define ubhow		0x8		/* Mach boothowto	*/
#define UI_arg0		0xC		/* long user arg	*/
#define	_BOOTINFO_BASE	0x0000E020	/* Bootinfo for slaves  */
#define	mmu_reg		0x0		/* Shadow mmu area pointer */
#define	start_pc	0x4		/* PC to jump to at slave boot */

#ifndef ASSEMBLER
/*
 *	C VERSIONS OF DEFINITIIONS.
 */

/* MICROBOOT */
struct	mb_data	{
  unsigned int	mb_state;
  unsigned int	mb_request;
  unsigned int	mb_param;
  unsigned int	mb_result;
  unsigned int	mb_reserved[16];
};

#define SECBOOT_FLAG	((int *)_SECBOOT_FLAG)

#endif ASSEMBLER

#endif _B2VME_

