/* $Id: fpu.h,v 1.1 91/01/14 16:56:31 gibson Exp $ */
/*****************************************************************************
* Definitions for 88100 Floating Point Registers
******************************************************************************/

/* Floating-Point Exception Cause Register */
#define	FPECR_FIOV	0x00000080	/* floating-point integer overflow */
#define	FPECR_FUNIMP	0x00000040	/* floating-point unimplmented */
#define	FPECR_FPRV	0x00000020	/* floating-point privilege violation*/
#define	FPECR_FROP	0x00000010	/* floating-point reserved operand */
#define	FPECR_DVZ	0x00000008	/* floating-point divide-by-zero */
#define	FPECR_UNF	0x00000004	/* floating-point underflow */
#define	FPECR_FOVF	0x00000002	/* floating-point overflow */
#define	FPECR_FINX	0x00000001	/* floating-point inexact */

/* Floating-Point Status Register */
#define	FPSR_AFINV	0x00000010	/* accumulated invalid operation flag*/
#define	FPSR_AFDVZ	0x00000008	/* accumulated divide-by-zero flag */
#define	FPSR_AFUNF	0x00000004	/* accumulated underflow flag */
#define	FPSR_AFOVF	0x00000002	/* accumulated overflow flag */
#define	FPSR_AFINX	0x00000001	/* accumulated inexact flag */

/* Floating-Point Control Register */
#define	FPCR_RM_MASK	0x0000C000	/* mask of rounding mode bits */
#define	FPCR_RM_RN	0x00000000	/* round to nearest */
#define	FPCR_RM_RZ	0x00004000	/* round toward zero (truncate) */
#define	FPCR_RM_RM	0x00008000	/* round toward negative infinity */
#define	FPCR_RM_RP	0x0000C000	/* round toward positive infinity */
#define	FPCR_MBZ	0x00003E00	/* these bits must be zero */
#define	FPCR_EFINV	0x00000010	/* enable invalid operation handler */
#define	FPCR_EFDVZ	0x00000008	/* enable divide-by-zero handler */
#define	FPCR_EFUNF	0x00000004	/* enable underflow handler */
#define	FPCR_EFOVF	0x00000002	/* enable overflow handler */
#define	FPCR_EFINX	0x00000001	/* enable inexact handler */

/* Floating-Point Source 1/Source 2 Operand High Register */
#define	FPHS_SIGN	0x80000000	/* source operands sign */
#define	FPHS_EXPONENT	0x7FF00000	/* source operands exponent */
#define	FPHS_MANTISSA	0x000FFFFF	/* source ops mantissa high 20 bits */

/* Floating-Point Precise Operation Type Register */
#define	FPPT_OPERATION_MASK	0x0000FFE0	/* bits 15-5 of instruction */
#define	FPPT_OP1SIZE_MASK	0x00000600	/* size bits for operand 1 */
#define	FPPT_OP1SIZE_DOUBLE	0x00000400	/* op is double precision */
/*#define	FPPT_OP1SIZE_SINGLE	0x00000000*/
#define	FPPT_OP2SIZE_MASK	0x00000180	/* size bits for operand 2 */
#define	FPPT_OP2SIZE_DOUBLE	0x00000080	/* op2 is double precision */
/*#define	FPPT_OP2SIZE_SINGLE	0x00000000*/
#define	FPPT_DESTSIZE_MASK	0x00000060	/* size bits for destination */
#define	FPPT_DESTSIZE_DOUBLE	0x00000040	/* dest is double precision */
/*#define	FPPT_DESTSIZE_SINGLE	0x00000000*/
#define	FPPT_DEST		0x0000001F	/* destination register */

/* Floating-Point Imprecise Operation Type Register */
#define	FPIT_RESEXP	0xFFF00000	/* result exponent */
#define	FPIT_RESERVED	0x000F0000	/* reserved */
#define	FPIT_OPCODE	0x0000F800	/* 5-bit opcode (bits 15-11 of inst) */
#define	FPIT_DESTSIZ	0x00000400	/* destinatin size */
#define	FPIT_DESTDOUBLE	0x00000400	/* destination is double */
#define	FPIT_EFINV	0x00000200	/* enable invalid operation handler */
#define	FPIT_EFDVZ	0x00000100	/* enable divide-by-zero handler */
#define	FPIT_EFUNF	0x00000080	/* enable underflow handler */
#define	FPIT_EFOVF	0x00000040	/* enable overflow handler */
#define	FPIT_EFINX	0x00000020	/* enable inexact handler */
#define	FPIT_DEST	0x0000001F	/* destination register */

/* Floating-Point Result High Register */
#define	FPRH_SIGN	0x80000000	/* result sign */
#define	FPRH_RM_MASK	0x60000000	/* mask of rounding mode for result */
#define	FPRH_RM_RN	0x00000000	/* result rounded to nearest */
#define	FPRH_RM_RZ	0x20000000	/* result rounded toward zero */
#define	FPRH_RM_RM	0x40000000	/* result rounded toward negative inf*/
#define	FPRH_RM_RP	0x60000000	/* result rounded toward positive inf*/
#define	FPRH_GUARD	0x10000000	/* guard bit for result */
#define	FPRH_ROUND	0x08000000	/* round bit for result */
#define	FPRH_STICKY	0x04000000	/* sticky bit for result */	
#define	FPRH_ADDONE	0x02000000	/* add one */
#define	FPRH_MBZ	0x01E00000	/* these bits must be zero */
#define	FPRH_MANTISSA	0x001FFFFF   /*mantissa high 21 bits(incl hidden bit)*/

/* Floating-Point Result Low Register */
#define	FPRH_DPMANTISSA_LOW	0xFFFFFFFF	/* double prec: 32 bits */
#define	FPRH_SPMANTISSA_LOW	0xE0000000	/* single prec: 3 bits */
