/* $Id: locore_vec.s,v 1.75 91/05/28 13:03:09 gibson Exp $ */
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 */
 
/*
 * Vectorized variable declaration/initialization and exception handling code.
 */

/*
 * Device driver definition and feature includes.
 */
#include "eg.h"
#include "ex.h"
#include "exp.h"
#include "plx.h"
#include "dr.h"
#include "dt.h"
#include "vlp.h"
#include "tmt.h"
#include "smt.h"
#include "xy.h"
#include "sha.h"
#include "hps.h"
#include "imp.h"
#include "cpus.h"
#include "gist.h"
#include "ether.h"
#include "cs_kdb.h"
#include "bbn_reva_support.h"
#include "bbn_revbc_support.h"
#include "bbn_icc.h"
#include "bbn_sig_debug.h"
#include "../butterflydev/xyreg.h"
#include "../butterflydev/sscsi.h"

/*
 * Machine specific definition includes.
 */
#include "../machine/ioconf.h"
#include "../machine/psl.h"
#include "../machine/pte.h"
#include "../machine/pmap.h"
#include "../machine/mtpr.h"
#include "../machine/reg.h"
#include "../machine/b2vme.h"
#include "../machine/trap.h"
#include "../machine/useraug.h"

/*
 * General includes.
 */
#include "../net/netisr.h"

/*
 * Number of interrupt stack pages.
 */
NISP	=	1

/*
 * Vector macro definition.
 */ 
#define	SCBVEC(name)		NEWLINEXXX \
	global _X/**/name 	NEWLINEXXX \
_X/**/name

/*
 * Allocate a string in the data segment.
 */
#define	MSG(label,msg)	NEWLINEXXX \
	data		NEWLINEXXX \
label:	asciz 	msg	NEWLINEXXX \
	text

/* No-op strings, primarily intended for MC88100 chip, G.3 mask workaround */
#define	NOP_1			NEWLINEXXX \
	or	r0,r0,r0

#define	NOP_2			NEWLINEXXX \
	or	r0,r0,r0	NEWLINEXXX \
	or	r0,r0,r0

#define	NOP_3			NEWLINEXXX \
	or	r0,r0,r0	NEWLINEXXX \
	or	r0,r0,r0	NEWLINEXXX \
	or	r0,r0,r0

/*
 * Macro to call the panic routine with one argument.
 */
	global	_panic, _waittime
#define	PANIC(label)			NEWLINEXXX \
	or.u	r10,r0,hi16(_waittime)	NEWLINEXXX \
	st	r0,r10,lo16(_waittime)	NEWLINEXXX \
	or.u	r2,r0,hi16(label)	NEWLINEXXX \
	bsr.n	_panic			NEWLINEXXX \
	or	r2,r2,lo16(label)

; E.1 (AND UP) MASK WORKAROUND:
;	A no-op is required before an rte to prevent the 
;	destination  register of the instruction before the rte
;	from being trashed.
#define	rte_WITH_WORKAROUND		NEWLINEXXX \
	addu	r0,r0,r0		NEWLINEXXX \
	rte				/* rte workaround definition */

;
; Beginning of actual assembled stuff.
;
; The interrupt stack is org'ed as the first contents of
; the kernel data segment.  Be careful not to put other
; .data space definitions in scb.s or assym.s (which
; precede locore_vec.s in the 'cat' of locore_vec.c 
;
	data
	global	_intstack, _eintstack
_intstack:
_start_lock:			; squeeze this in here (R/W space)
	zero	NISP * NBPG
_eintstack:

	text
	global	_start_text
_start_text:
	zero   0x1000		; Finish off scb page
#ifndef PSECT_LOADER
	zero   0x2000		; get start of vectorized vbls to 0xC0014000
#endif PSECT_LOADER

;
; Here are the definitions of all vectorized kernel variables	
;
	global _start_vectorized, _end_vectorized
_start_vectorized:

;;;;;;;;;;;;;;;;; WARNING: START OF WORD ALIGNMENT RESTRICTION! ;;;;;;;;;;;;
; All variables declared within these bounds assume
; that they are at least longword aligned.
;

;;;;;;;;;;;;;;;;; WARNING: ORDERING RESTRICTIONS APPLY! ;;;;;;;;;;;;;;;;;;;;
; Notice: These two locations MUST be first in the vectorized page,
;         and in the stated order!  my_cpu_number MUST be third!
;         All of this is in support of the crash dump analyzer.
;
;         By the way: It's fine to use fossil_srp and fossil_log_node_map
;	  for DAS tracing games so long as they don't involve the
;	  crashdump writing code in butterflydev/[smt.c, scsi.c]
;	  These locations are overwritten when making a dump.

	global _fossil_srp, _fossil_log_node_map, _my_cpu_number
_fossil_srp:
	word	0		; set up by dumptape writer.

_fossil_log_node_map:
	word	0		; map pointer

_my_cpu_number:	
	word	0		; cached value of logical cpu number

	global _my_bytes_on_node
_my_bytes_on_node:
	word	0		; bytes of memory on this node

;;;;;;;;;;;;;;;;;;;;;; END OF ORDERING RESTRICTIONS ;;;;;;;;;;;;;;;;;;;;;;;;


	global  _my_boot_state	; info when a node fails to boot
_my_boot_state:
	word	0

	global  _my_kernel_map
_my_kernel_map:
	word	0

	global	_my_objoff_pp_ihash
_my_objoff_pp_ihash:
	zero	VM_IHASH_SIZE

	global _my_scanner_state
_my_scanner_state:
	word	0

	global _my_scanner_interval
_my_scanner_interval:
	word	0

	global	_my_pmaps
_my_pmaps:
 	word	0

        global  _my_pmap_to_scan
_my_pmap_to_scan:
        word    0
 
        global  _my_vaddr_in_pmap
_my_vaddr_in_pmap:
        word    0
 
	global	_my_pmaps_lock
_my_pmaps_lock:
	zero	SIMPLE_LOCK_SIZE	

	global _my_vm_page_lists
_my_vm_page_lists:
	word	0

	global _my_vm_intrlv_lists
_my_vm_intrlv_lists:
	word	0

	global _my_vm_intrlv_pages
_my_vm_intrlv_pages:
	word	0

	global _mb_phys_start
_mb_phys_start:
	word	0		; starting physical address of main bus

 	global  _phys_stack_page
_phys_stack_page:
 	word	0		; kernel stack cache
 
	global  _my_cpu_tlb_clock
_my_cpu_tlb_clock:
	word	0		; TLB shootdown

	global	_my_active_proc
_my_active_proc:
	word	0		; currently active proc pointer

	global	_my_idle_proc
_my_idle_proc:
	word	0		; idle process proc pointer

	global	_my_curproc_list
_my_curproc_list:
	word	0		; linked list of procs "owned" by this node


	global	_my_isp
_my_isp:	
	word	_eintstack	; this value is correct for the master,
				; the slaves install their own interrupt
				; stacks in slave_start_code

	global	_my_scratch_pte
_my_scratch_pte:
	zero	PTE_SIZE	; kernel VA of scratch PTE we can use

	global	_my_scratch_va
_my_scratch_va:	
	word	0		; the address the scratch PTE maps

	global	_my_phys_cpu_number
_my_phys_cpu_number:
	word	0xFFFFFFFF	; node's physical cpu number

	global	_my_active_cpus, _mach_cpus
; Actually the same cell
_mach_cpus:
_my_active_cpus:
	word	0		; cached copy of mach_cpus

	global	_hardpri
_hardpri:
	word	0		; current spl on this node
 
	global	_my_tick
_my_tick:
	word	0		; hardclock ticks
 
	global	_my_copy_mpte1
_my_copy_mpte1:
	zero	MMUPTE_SIZE	; kernel VA of PTE for copyphys() routines

	global	_my_copy_mpte2
_my_copy_mpte2:
	zero	MMUPTE_SIZE	; kernel VA of PTE for copyphys() routines

	global	_my_copy_va1
_my_copy_va1:	
	word	0		; address that the PTE maps to

	global	_my_copy_va2
_my_copy_va2:	
	word	0		; address that the PTE maps to

	global _my_vme_master_map_bitmap
_my_vme_master_map_bitmap:
	zero	BITMAP_HEADER_SIZE		; sizeof bitmap_data_t

	global	_my_buffer_headers
_my_buffer_headers:
	word	0

	global	_backpocket_bufpage, _backpocket_bufpage_busy

;
; Page for use by the buffer cache in emergencies
;
_backpocket_bufpage:
	word	0

_backpocket_bufpage_busy:
	word	0

	global _my_vme_slave_map_entries
_my_vme_slave_map_entries:
	word	VME_MAP_CHAIN_END	; head of slave mapper ram free list

	global _my_vme_map_alloc_lock
_my_vme_map_alloc_lock:	
	zero	SIMPLE_LOCK_SIZE	; mapper ram allocation lock

	global	_my_vme_vec_table	; VME vectored device handerl/arg table
_my_vme_vec_table:
	word	0

 	global _exceptlock
_exceptlock:
 	zero	SIMPLE_LOCK_SIZE	; signal exception lock
 
	global	_SIpending, _XSIpending
_SIpending:
	word	0			; pending software interrupts

_XSIpending:
	word	0			; externally caused sw interrupts

	global	_netisr
_netisr:
	word	0			; network software interrupt pending

	global	_netint_active
_netint_active:
	word	0			; network interrupt in progress flag

	global	_if_rpc_zone
_if_rpc_zone:
	zero	4			; network interface RPC zone pointer

	global	_if_rpc_queue
_if_rpc_queue:
	zero	MPQUEUE_HEAD_SIZE	; network interface RPC queue head

#if NETHER > 0
	global	_arptab_cache
_arptab_cache:
	zero	4			; addr resolution protocol table cache
#endif

	global _sched_lock
_sched_lock:
	zero	SIMPLE_LOCK_SIZE	; scheduler lock

	global	_my_sched_lock
_my_sched_lock:
	word	0		; pointer to sched_lock

	global _next_softclock_time
_next_softclock_time:
	word	0		; ticks to next softclock

	global _next_schedcpu_time
_next_schedcpu_time:
	word	0		; ticks to next schedcpu


	; stuff for statistics and scheduling

	global _my_stat_count
_my_stat_count:
	word	0

	global _my_switchout		; time-averaged switches to master
_my_switchout:
	word	0

	global _my_switchout_acc
_my_switchout_acc:
	word	0
	
	global _my_involswtch		; time-averaged involuntary context sw
_my_involswtch:
	word	0

	global _my_involswtch_acc
_my_involswtch_acc:
	word	0

	global _my_busy			; time-averaged non-idle ticks
_my_busy:
	word	0

	global _my_busy_acc
_my_busy_acc:
	word	0

	global _my_idle			; time-averaged idle ticks
_my_idle:
	word	0

	global _my_idle_acc		
_my_idle_acc:
	word	0

	global _my_load			; per-node load average for scheduling
_my_load:
	word	0,0,0			; VECTOR, one load for each strategy

	global _my_zombcount
_my_zombcount:
	word 0

	global	_my_64bit_time
_my_64bit_time:
	word	0		; high order word of 64 bit RTC
	
	global	_previous_rtc_value
_previous_rtc_value:
	word	0		; low order word of 64 bit RTC

	global	_local_runq
_local_runq:
	zero	RQ_SIZE		; local run queue
	
	global	_Mach_rtc_synch
_Mach_rtc_synch:
	word	0		; master to slave RTC synch location

	global	_kpb_dropped_chars
_kpb_dropped_chars:
	word	0		; kernel print buffer chars dropped

	global	_current_kpb
_current_kpb:
	word	0		; location in kernel print buffer

	global	_my_loadavg
_my_loadavg:
	word	0,0,0		; load average vector

	global	_cp_time
_cp_time:
	word	0,0,0,0		; cpu time in CP_USER, CP_NICE, CP_SYS, CP_IDLE

	global	_dk_time
_dk_time:
	word	0,0,0,0		; time busy for disk controllers 0, 1, 2, 3

	global	_dk_busy
_dk_busy:
	word	0,0,0,0		; disk controller 0, 1, 2, 3 is busy

	global _my_arg_buf
_my_arg_buf:
	word	0		; virtual addr of this node's exec args buffer

	global _authdes_cache
_authdes_cache:
	word	0		; RPC authentication cache

	global _authdes_lru
_authdes_lru:
	word	0		; RPC authentication cache LRU entries

	global	_my_panicstr
_my_panicstr:
	word	0		; string with which this node panicked

	global	_my_global_panicstr
_my_global_panicstr:
	word	0		; global panicstr state when node panicked

	global	_my_default_sigsynch
_my_default_sigsynch:		; usr SEGV pagefault hndlng works automatically
	word	HSYNCHTYPE_DEFAULT


#if BBN_SIG_DEBUG
	global	_sig_debug
_sig_debug:			; mask for signal debugging system
	word	0
#endif BBN_SIG_DEBUG

; per node list of zombie pages to reclaim
	global	_vm_zombie_lock, _vm_zombie_queue

_vm_zombie_lock:
	zero	SIMPLE_LOCK_SIZE

_vm_zombie_queue:
	zero	8

#if GIST
;
; Kernel GIST variables
;
	global	_gist_pointer, _gist_buffer, _gist_limit
	global  _gist_level, _gist_trigpt, _gist_trigoffs
_gist_pointer:	word	0
_gist_buffer:	word	0
_gist_limit:	word	0
_gist_level:	word	0
_gist_trigpt:	word	0
_gist_trigoffs:	word	0
#endif GIST

	global	_callout_info
_callout_info:
	word	0		; vectorized callout variable

	global _my_pcb_cache
_my_pcb_cache:
	zero	PCB_SIZE	; process context block cache

	global	_current_msp
_current_msp:
	word	0		; current stack pointer (used with stack cache)

	global _nofault
_nofault:
	word	0		; peek/poke fault flag

 	global	_breadhit
_breadhit:
	word	0		; buffer cache hits

 	global	_breadmiss
_breadmiss:
	word	0		; buffer cache misses

 	global	_breadhitra
_breadhitra:
	word	0		; buffer cache read ahead hits

 	global	_breadmissra
_breadmissra:
	word	0		; buffer cache read ahead misses
 
	global	_mb_vec_zone
_mb_vec_zone:
	word	0		; vectorized main bus structure zone

	global	_home_cluster_type
_home_cluster_type:
	word	0		; copy of flags from this node's home cluster

	global	_pf_timer_reasons
_pf_timer_reasons:
	word	0		; Count of reasons to do network timeouts

	global	_my_if_count
_my_if_count:
	word	0		; Number of network interfaces on node


#if CS_KDB
;
; Some kdb variables that are used by the stack backtrace code.
	global	_kdbcurpcb, _kdbcurmap, _kdbcurpid
_kdbcurpcb:
	word	0
_kdbcurmap:
	word	0
_kdbcurpid:
	word	0
#endif


; Here are the variable declarations for the vectorized device
; drivers. Remember that these MUST be longword aligned.
; Also, these declarations MUST match the C-code's understanding
; of the world, so be careful. So much for compiler support.

#if NXY > 0
;
; xylogics (XY) disk driver declarations
;
	global	_iopb
_iopb:
	zero	IOPB450_SIZE * IOPB_CNT

	global	_xycinfo, _xydinfo
_xycinfo:
	zero	4 * NXYC
_xydinfo:
	zero	4 * NXY

  	global  _drvtype
_drvtype:
	zero	4 * NXYC

  	global  _disk_map
_disk_map:
	zero	4 * NXY

  	global  _disk_map_pa
_disk_map_pa:
	zero	4 * NXY

  	global  _disk_map_vmeaddr
_disk_map_vmeaddr:
	zero	4 * NXY

	global	_xycopypa
_xycopypa:
	zero	4 * NCOPYBUF

	global	_xycopy_vmeaddr
_xycopy_vmeaddr:
	zero    4 * NCOPYBUF

	global	_copybufwanted
_copybufwanted:
	zero	8

	global	_xycopyinuse
_xycopyinuse:
	word	0

	global	_copybuflock
_copybuflock:
	zero	SIMPLE_LOCK_SIZE	; XY copy buffer lock

	global	_iopbwanted
_iopbwanted:
	zero	8

	global	_iopblock
_iopblock:
	zero	SIMPLE_LOCK_SIZE	; XY iopb lock

	global _raw_iopb, _raw_buf
_raw_iopb:
	zero	IOPB450_SIZE * NXY

_raw_buf:
	zero	BUF_SIZE * NXY

	global	_free_hd
_free_hd:
	zero	IOPBQ_SIZE

	global	_rdy_hd
_rdy_hd:
	zero	IOPBQ_SIZE * NXYC

	global	_act_hd
_act_hd:
	zero	IOPBQ_SIZE * NXYC

	global	_ioctl_buf
_ioctl_buf:
	zero	IOCTL_BUF_SIZE

	global	_queue
_queue:
	zero	XYQUEUEHEAD_SIZE * NXYC

	global	_xy_disks_zone
_xy_disks_zone:
	word	0			; XY disk label buffer zone

	global	_xy_disks
_xy_disks:
	zero	4 * NXY			; XY disk label buffer pointers

	global	_unitinfo
_unitinfo:
	zero	NXY * UNITINFO_SIZE

	global	_xy_par
_xy_par:
	zero	NXY * NPARTITIONS * PART_INFO_SIZE

	global	_translationzone
_translationzone:
	word	0

	global _translationlock
_translationlock:
	zero	SIMPLE_LOCK_SIZE	; XY translation buffer allocation lock

#endif NXY


;
; Pointer to kernel memory allocator buckets
;
	global	_kmalloc_buckets
_kmalloc_buckets:
	word	0

;
; Pointer to chunk of address space to free
;
	global	_kmalloc_freechunk
_kmalloc_freechunk:
	word	0

;
; Interprocess Communications Channel Manager (ICCM) declarations
;
	global	_my_icc_ringbuffer
_my_icc_ringbuffer:
	word	0			; Virtual address of local ring buffer

	global	_my_icc_ringbuffer_offset
_my_icc_ringbuffer_offset:
	word	0			; Physical offset of local ring buffer

;
; Pointers to locking functions for network data structures. These
; are set to noops if network code does not run in parallel (i.e. if
; the if_server is active).
	global	_simple_lock_IFNP, _simple_unlock_IFNP

_simple_lock_IFNP:
	word	0

_simple_unlock_IFNP:
	word	0

#include "inet.h"
#if INET > 0
;
; Local cache of last used in_pcb
;
	global	_tcp_pcb_cache, _udp_pcb_cache

_tcp_pcb_cache:
	zero	4

_udp_pcb_cache:
	zero	4

	global	_ipintrq, _ipq

;
; Local copy of tcb and udb.
; inpcb's are hashed onto different queues based on the
; local port number.
;
	global	_tcb, _udb
_tcb:
	zero	INPCB_SIZE

_udb:	
	zero	INPCB_SIZE

;
; Local IP input queue
_ipintrq:
	zero	IFQUEUE_SIZE

;
; Local IP reassembly queue
_ipq:
	zero	IPQ_SIZE

;
; Local variables used by ip source routing
	global	_ip_srcrt, _ip_nhops

_ip_srcrt:
	word	0
_ip_nhops:
	word	0

#endif INET

#include "nfsclient.h"
#if NFSCLIENT
;
; Table of cached client structures
;
	global	_chtable

_chtable:
	zero	CHTAB_SIZE
#endif NFSCLIENT

;
; Interphase Jaguar SCSI host adaptor
;
#if NSHA > 0
	global	_shactlr, _shadevice

_shactlr:
	zero	4 * NSHA
_shadevice:
	zero	4 * NSHA

;
; Sha driver (Synchronous SCSI Host Adapter)
;
	global  _my_shas, _my_nshas, _my_malloc_page, _my_malloc_next
_my_shas:
	word	0
_my_nshas:
	word	0
_my_malloc_page:
	word	0
_my_malloc_next:
	word	0

	global _my_copy_buffers, _my_sscsi_requests
	global _my_raw_buf_zone, _my_scsi_device_map
_my_copy_buffers:
	zero	MPQUEUE_HEAD_SIZE
_my_sscsi_requests:
	word	0
_my_raw_buf_zone:
	word	0
_my_scsi_device_map: 
	zero	DEVICES_PER_BUS * BUSSES_PER_ADAPTER * NSHA * 4

;
; Sd driver (Northern Telecom SCSI disk)
;
	global _my_sds
_my_sds:
	word	0

#endif NSHA

;
; excelan ethernet controller
;
#if NEX > 0
	global	_ex_softc, _exctlr, _exdevice
_ex_softc:
	zero	4 * NEX
_exctlr:
	zero	4 * NEX
_exdevice:
	zero	4 * NEX
#endif NEX

;
; CMC EXP-135 ethernet controller
;
#if NEXP > 0
	global	_expctlr, _expdevice

_expctlr:
	zero	4 * NEXP

_expdevice:
	zero	4 * NEXP
#endif 

;
; Interphase eagle ethernet controller
;
#if NEG > 0
	global	_eg_softc, _egctlr, _egdevice
_eg_softc:
	zero	4 * NEG
_egctlr:
	zero	4 * NEG
_egdevice:
	zero	4 * NEG
#endif NEG

;
; parallax 1280 series graphics controller
;
#if NPLX > 0
	global	_plx_chan, _plx_softc, _plxctlr, _plxdevice, _plxdriver
_plx_chan:
	zero	4 * NPLX
_plx_softc:
	zero	4 * NPLX
_plxctlr:
	zero	4 * NPLX
_plxdevice:
	zero	4 * NPLX
#endif NPLX

;
; ikon 10089 buffered VMEbus dr11-w emulator
;
#if NDR > 0
	global	_dr_chan, _dr_softc, _drctlr, _drdevice, _drdriver
_dr_chan:
	zero	4 * NDR
_dr_softc:
	zero	4 * NDR
_drctlr:
	zero	4 * NDR
_drdevice:
	zero	4 * NDR
#endif NDR

;
; data translations dt1401 VMEbus 12-bit A/D interface
;
#if NDT > 0
	global	_dt_chan, _dt_softc, _dtctlr, _dtdevice, _dtdriver
_dt_chan:
	zero	4 * NDT
_dt_softc:
	zero	4 * NDT
_dtctlr:
	zero	4 * NDT
_dtdevice:
	zero	4 * NDT
#endif NDT

;
; VMEbus loopback driver
;
#if NVLP > 0
	global	_vlp_softc, _vlpctlr, _vlpdevice
_vlp_softc:
	zero	4 * NVLP
_vlpctlr:
	zero	4 * NVLP
_vlpdevice:
	zero	4 * NVLP
#endif NVLP

;
; Systech HPS (high-performance serial line) controller
;
#if NHPS > 0
	global	_htyctlr, _h_tty, _h_hty
_htyctlr:			; driver internal
	zero	4 * NHPS
_h_tty:				; driver internal
	zero	4 * NHPS
_h_hty:				; driver internal
	zero	4 * NHPS

	global	_hpsctlr, _hpsdev, _hpsctl
_hpsctlr:			; driver internal 
	zero	4 * NHPS
_hpsdev:			; Unix interface struct
	zero	4 * NHPS
_hpsctl:			; Unix interface struct
	zero	4 * NHPS
#endif NHPS

; The following set of 4 byte flags must be aligned to fit in the
; 'ast' longword
; Note that these flags are all related to the current process
; even though they are stored globally
	global	_ast, _new_signal, _runrun, _do_profile
_ast:
_new_signal:
	byte	0		; new signal for current process
				; protected by proc_lock
_runrun:
	byte	0		; higher priority process ready to run
				; protected by my_sched_lock
_do_profile:
	byte	0		; profiling needed for current proc.
				; only set locally

	byte	0		; available flag
; The following should really be in a read only segment.  Here for convenience.
	global _hz
_hz:
	word	0
;
; Access to variables declared before this point assume longword alignment.
;;;;;;;;;;;;;;;;; END OF LONGWORD ALIGNMENT RESTRICTION! ;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;; START OF HALF WORD ALIGNMENT REGION ;;;;;;;;;;;;;;;;;;;;;;;

global	_intercept_bank, _usr_segtable_bank, _super_segtable_bank
_intercept_bank:
	half INTERCEPT_CMR_INDEX_SMALL
_usr_segtable_bank:
	half USR_SEGTABLE_INDEX_SMALL
_super_segtable_bank:
	half SUPER_SEGTABLE_INDEX_SMALL

;;;;;;;;;;;;;;;;; END OF HALF WORD ALIGNMENT REGION ;;;;;;;;;;;;;;;;;;;;;;;;;
; This flag used to mean that it was safe to call mcount. This
; was needed because the kernel data segment isn't mapped ab initio.
; Interestingly enough, this keeps the slaves from profiling
; unless this variable is patched on a per-node basis.
	global	_safe_to_profile
_safe_to_profile:
	byte	0

; The eventual goal is to make safe_for_svtop go away, since
; it no longer describes the function of the variable.
; Its current meaning is:
;     a) the node has scheduled the idle thread
;     b) the node has done the corresponding initialization
;        of its vectorized page variables
;     c) the node is scheduling threads.
	global	_my_node_past_idle
	global	_safe_for_svtop
_my_node_past_idle:
_safe_for_svtop:
	byte	0		; set after point in time when svtop is ok

	global _should_exit
_should_exit:
	byte	0		; halt synchronization flag

	global	_curpri
_curpri:
	byte	0		; priority (p_pri) of currently running process

	global	_switch_timeout
_switch_timeout:
	byte	0		; block transfer timeout

	global  _my_node_has_panicked
_my_node_has_panicked:
	byte	0		; per-node panic flag

	global	_my_node_eeprom_signature
_my_node_eeprom_signature:
	byte	0		; TCS node signature information

	global	_my_node_has_cmapping_ram
_my_node_has_cmapping_ram:
	byte	0		; node has CPU mapping ram flag

	global	_my_node_bypasses_page_walks
_my_node_bypasses_page_walks:
	byte	0		; node autobypasses page-level portion of walk
				; compare _my_pagewalk_autobypassed.
	
	global	_my_node_has_code_mmu2
_my_node_has_code_mmu2:
	byte	0		; node probed and found second code mmu

 	global	_stack_cache_used
_stack_cache_used:
	byte	0		; stack cache is in use flag

	global _cpu_idle
_cpu_idle:
	byte	0		; CPU is running idle process flag

	global _paging_stats_disabled
_paging_stats_disabled:
	byte	0		; if 1, paging statistics gathering is disabled

	global  _my_slave_map_needed
_my_slave_map_needed:
	byte    0               ; process needs slave_map registers

	global _my_copy_buffers_needed
_my_copy_buffers_needed:
	byte	0		; SCSI Host Adapter needs copy buffers flag

	global 	_my_sha_has_sg_bug
_my_sha_has_sg_bug:
	byte 0			; SHA  less than REV E has scatter/gather bug

	global	_vm_cache_flush_syscall
_vm_cache_flush_syscall:
	byte	0		; someone is in VM cache flush system call

#if BBN_REVA_SUPPORT
	global	_node_use_augmentations
_node_use_augmentations:
	byte	0		; flag: augmentation capability is there

	global	_node_use_remote_IO_refs
_node_use_remote_IO_refs:
	byte	0		; flag:  can reference remote VME over switch
#endif BBN_REVA_SUPPORT

#if BBN_REVBC_SUPPORT
	global	_my_pagewalk_autobypassed
_my_pagewalk_autobypassed:	; ALL nodes autobypass page walks. (No Rev B)
	byte 0
#endif BBN_REVBC_SUPPORT

#if NXY > 0
	global _xy_sts
_xy_sts:
	zero  NXY * NPARTITIONS	; XY partition status table
#endif NXY

;
; Pad to 1 page. When this overflows, you'll know it!
; At that point, add another page here AND IN PMAP.C
; Check the number of pages needed;
; you may be stuck changing f9 to f8 or worse...
;
_end_vectorized:
	zero	NBPG - (_end_vectorized - _start_vectorized)

;
; TC2000 exception handling
;
; XF macro converts word offsets from reg.h to byte offsets
;
#define XF(q)	XF_/**/q*4

	text

;
; Main interrupt service routine.
;
; See machine/psl.h for Interrupt Level definitions.
; See machine/conf.c for IsrcPending, and IsrcPendingVME.
; See BUILD/ioconf.c for IntTable and VecTable
; See machine/ioconf.h for the format of my_vme_vec_table
#define KERNEL_STACK	0xFBFFD000

SCBVEC(interrupt):				; Trap   1 -- interrupt
	stcr	r1, cr18
	bsr	shadow				; save shadow registers
	bsr	fpipe				; drain floating point pipe
	or	r10, r0, T_INTERRUPT
	bsr.n	saveregs			; save user registers
	st	r10, r25, XF(type)		; fpipe left frame ptr in r25
	bsr.n	_starttimenow			; get time stamp, if safe
	addu	r2,r25,XF(time)
	;
	; switch to interrupt stack if we're not already there
	;
	or	r24,r31,r31			; save stack pointer
	xor.u	r10,r31,hi16(KERNEL_STACK_VADDR); are we on kernel stack ?
	and	r10,r10,0			; clear low halfword
	bcnd	ne0,r10,1$
	or.u	r10,r0,hi16(_my_isp)		; switch to interrupt stack
	ld	r31,r10,lo16(_my_isp)
1$:
	;
	; At this point we are set up to handle the interrupt(s).
	; The variable to register assinment is as follows:
	;	r25 exception frame pointer
	;	r24 original stack pointer
	;
	; Call for C coded interrupt handler found in machine/conf.c.
	;
#include "bbn_assembler_ihandler.h"
#if BBN_ASSEMBLER_IHANDLER
	;
	; Code is a bit convoluted due to attempts to avoid pipeline
	; stalls, and to fill delayed branch slots...
	;
	; If all interrupts are blocked, then we are done, else
	; read the interrupt source register to see what is pending.
	;	r23 source
	;	r14 hi16(_hardpri)
	;	r15 old_pri
	;
	or.u	r14, r0, hi16(_hardpri)		;
	ld	r15, r14, lo16(_hardpri)	; r15 = old_pri = hardpri
	or.u	r4, r0, hi16(_INT_SOURCE_REG)	;
	cmp	r3, r15, IL_NMI			; if (old_pri >= IL_NMI)
	bb1.n	hs, r3, cleanup			;   return
	ld	r23, r4, lo16(_INT_SOURCE_REG)	;

	;
	; Cons up a bit vector of the bits coresponding to the Interrupt
	; Level(s) which are currently pending and are not blocked.
	;	r22 ipending
	;
	or.u	r4, r0, hi16(_IsrcPending)	;
	or	r4, r4, lo16(_IsrcPending)	; r5 = &IsrcPending[0]
	extu	r3, r23, 4<4>			; r3 = (source >> 4) & 0x0f
	ld	r22, r4[r3]			; ipending = IsrcPending[r3]

	;
	; If there is a VMEbus interrupt, then perform an Interrupt
	; ACKnowledge to quench the interrupt and to get the VMEbus vector.
	;	r20 vme_vector
	;
	bb0.n	0, r23, checkipi		; if (source & ISRC_VME) {
	extu	r3, r23, 3<1>			;   r3 = vme_level =
						;    (source & ISRC_VME_VEC)>>1
	or.u	r4, r0, hi16(VIRT_IACK_BLOCK)	;
	or	r4, r4, lo16(VIRT_IACK_BLOCK)	;   r4 = &VIRT_IACK_BLOCK[0]
	ld.hu	r20, r4[r3]			;   vme_vector =
						;    VIRT_IACK_BLOCK[vme_level]
	or.u	r4, r0, hi16(_IsrcPendingVME)	;
	or	r4, r4, lo16(_IsrcPendingVME)	;   r4 = &IsrcPendingVME[0]
	ld	r5, r4[r3]			;   ipending |=
	mask	r20, r20, 255			;   vme_vector &= 0xff;
	or	r22, r22, r5			;    IsrcPendingVME[vme_level]
						; }

checkipi:
	;
	; If there is a maskable InterProcessor Interrupt, then cons up
	; a bit vector of the bits coresponding to the Interrupt Level(s)
	; of the Software Interrupt(s) which are currently pending and are
	; not blocked.
	; To allow remote processors to generate softints, while keeping
	; SIpending a "local only" variable there is a variable "XSIpending"
	; (external SI pending) which is set when a remote processor
	; wishes to cause an SI. This value is or'd with the local SIPending
	; whenever an IPI is detected.
	;
	;	r18 hi16(_SIpending)
	bb0.n	5, r23, doint			; if (source & ISRC_IPI) {

	or.u	r18, r0, hi16(_SIpending)
	or.u	r2, r0, hi16(_XSIpending)	; shadow = virt_Atomic_fetch32(
						;       &XSIpending)
	or	r17, r0, r0			; r17 = 0
	bsr.n	_cnintrcheck			;   (poll TCS & console)
	xmem	r17, r2, lo16(_XSIpending)	; r17 = XSIpending
						; XSIpending = 0
#if BBN_ICC
	bsr	_icc_check			;   (check ICC)
#endif
	ld	r6, r18, lo16(_SIpending)	; fetch local bits
	or	r3, r0, 2			;   r3 = 0x02
	mak	r4, r3, r15			;   r4 = mask = 0x2 << old_pri
	subu	r5, r4, 1			;   r5 = mask = mask - 1
	or	r6, r17, r6			;   r6 = SIpending|XSIpending
	and.c	r7, r6, r5			;   r7 = SIpending & ~mask
	or	r22, r22, r7			;   ipending|=(SIpending&~mask)
	and	r6, r6, r5			;   SIpending &= mask
	st	r6, r18, lo16(_SIpending)	;   ...
						; }

doint:
	bcnd.n	eq0, r22, resetpri		; if (ipending == 0)
						;   goto resetpri
	or.u	r16, r0, hi16(_IntTable)	;
	or	r16, r16, lo16(_IntTable)	; r16 = &IntTable[0]
	;
	; While there are interrupts to service,
	; call the associated interrupt handlers.
	;	r17 first_pri
	;	r16 &IntTable[0]
	;	r14 hi16(_hardpri)
	;
	ff1	r17, r22			; first_pri = ff1(ipending)
	or	r2, r0, r17			;
whileint:					; while (ipending) {
	bsr.n	_SetEnables			;   hardpri = first_pri
	st	r17, r14, lo16(_hardpri)	;   SetEnables(first_pri)
; Register usage:
; When calling handler
; r2 = xframep for 
; r3 = vector for VME interrupt, else garbage
; r4 = handler specific argument
; r5 = ifunc
; my_vme_vec_table is indexed by the vector number and is
; an array of double words.
;	word1 - handler arg
;	word2 - handler routine pointer
;
	bsr.n	_enable				;  ifunc = IntTable[first_pri]
	ld	r5, r16[r17]			;   enable()
	or	r2, r0, r25			; first arg = xframep
	or.u	r4, r0, hi16(_my_vme_vec_table)		;
	bcnd.n	ne0, r5, notvec			;   if (ifunc == VME_VEC)
	ld	r4, r4, lo16(_my_vme_vec_table) ; r4 points at local table
	or	r3, r20, r20			; r3 == vector
	ld.d	r4, r4[r20] 			; r4,r5 =  my_vme_vec_table[vme_vector]
notvec: jsr.n	r5				;   (*ifunc)(xframep, ...)
	clr	r22, r22, r17			;   disable()
	bsr.n	_disable			;   ipending &= ~(1<<first_pri)
	ff1	r17, r22			;   first_pri = ff1(ipending)
	bcnd.n	ne0, r22, whileint		;
	or	r2, r0, r17			; }

resetpri:
	;
	;Set the interrupt enable masks to be consistent
	; with the old priority level and reset hardpri.
	;
	bsr.n	_SetEnables			; SetEnables(old_pri)
	or	r2, r0, r15			;
	st	r15, r14, lo16(_hardpri)	; hardpri = old priority

	; Check whether any software interrupts can handled now.
	; This will occur when a handler posted a new software
	; interrupt.
	;	r18 hi16(_SIpending)
	;	r15 old hardpri
	;	r14 hi16(_hardpri)
	;
	; interrupts are disabled
	cmp	r8, r15, IL_SI_HIGH		; if (old_pri < IL_SI_HIGH) {
	bb1.n	hs, r8, cleanup			;
	ld	r6, r18, lo16(_SIpending)	;   r6 = SIpending
	or	r5, r0, 2			;   r5 = 0x02
	mak	r5, r5, r15			;   mask = 0x02 << old_pri
	subu	r5, r5, 1			;   r5 = mask = mask - 1
	and.c	r3, r6, r5			;   r3 = SIpending & ~mask
	bcnd	eq0, r3, cleanup		;   if (SIpending & ~mask)

	bsr.n	_enable				;   enable interrupts
	or	r2, r15, r0			;   r2 = old hardpri
	bsr.n	_HandleSI			; Handle sw interrupts
	or	r3, r0, r25			;   r3 = xframep
	bsr	_disable			; disable interrupts again
	; }

cleanup:
#else BBN_ASSEMBLER_IHANDLER
	; bsr.n	_ihandler			; ihandler(xframep)
	; or	r2, r25, r0
#endif BBN_ASSEMBLER_IHANDLER

	;
	; We are done handling the interrupt, so restore the stack, etc.
	; The register expectations at this point are:
	;	r25 exception frame pointer
	;	r24 original stack pointer
	;
	
	or	r31,r24,r24			; restore original stack ptr

	or	r2, r25, r0			; exception frame pointer
	bsr.n	_ex_cleanup			; deal with pipes
	or	r3, r31, r0			; FP frame(s), if any

	ld	r9, r25, XF(tpsr)		; supervisor/user mode?
	bb1.n	PSR_mode, r9, 2$
;
; We came from user mode.
; If any of the flags in 'ast' are set then process them now

	or.u	r3, r0, hi16(_ast)
	ld	r3, r3, lo16(_ast)
	bcnd.n	eq0, r3, 2$			; set?
	or.u	r3, r0, hi16(U_AR0)		; (u.u_ar0 = regs) in case
	st	r25, r3, lo16(U_AR0)		; ACALLpsig takes a signal
	bsr.n	_astreset			; astreset(xframep)
	or	r2, r25, r0
2$:
	br	exreturn

;
; Code access fault
;
SCBVEC(codaccess):		; Trap   2 -- code access fault
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_CODACCESS

;
; Data access fault -- bus errors and other things
;
SCBVEC(dataccess):		; Trap   3 -- data access fault
	stcr	r1, cr18
#if AUG_DEBUG	
	/* Clear augmentation register so we can reference memory */
	stcr	r10,cr19
	or.u	r10,r0,hi16(_IO_AUG_RD_AND_CLR)
	ld	r10,r10,lo16(_IO_AUG_RD_AND_CLR)
	ldcr	r10,cr19
#endif AUG_DEBUG
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_DATACCESS

;
; Misaligned access
;
SCBVEC(misaligned):		; Trap   4 -- misaligned access
	stcr	r1, cr18
	bsr	shadow		; save shadow registers

	;
	; get the instruction and see what was
	; happening we may need to clear ssbr bits
	;
	ldcr	r10,sxip	; instruction address
	clr	r10,r10,2<0>

	ldcr	r1,tpsr		; Get the instr correctly from user/super space
	bb1	PSR_mode,r1,10$

	NOP_1			; G.3 workaround
	ld.usr	r10,r10,r0	; instruction (user mode)
	NOP_3			; G.3 workaround
	br	11$
10$:
	ld	r10,r10,r0	; instruction (supervisor mode)

		; r10 contains the instruction that misaligned
11$:
	bcnd	lt0,r10,1$	; triadic addressing mode ?

	br.n	2$		; immediate offset addressing
	extu	r1,r10,4<26>	; P and TY fields

1$:				; triadic addressing mode
	extu	r1,r10,4<10>	; P and TY fields
2$:
	bb1	3,r1,9$		; store instruction ? (ssbr ok)
	extu	r10,r10,5<21>	; destination register
	xor	r1,r1,4
	bcnd	eq0,r1,5$	; ld.d ?

	br.n	6$
	or	r10,r10,0x20	; single word -- clear 1 ssbr bit
5$:
	or	r10,r10,0x40	; double word -- clear 2 ssbr bits
6$:
	ldcr	r1,ssbr
	clr	r1,r1,r10
	stcr	r1,ssbr
9$:
	br.n	trapcommon
	or	r10,r0,T_MISALIGNED

;
; Unimplemented opcode
;
SCBVEC(opcode):			; Trap   5 -- unimplemented opcode
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_OPCODE

;
; Privilege violation
;
SCBVEC(privilege):		; Trap   6 -- privilege violation
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_PRIVILEGE

;
; Bounds check trap
;
SCBVEC(bounds):			; Trap   7 -- bounds check trap
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_BOUNDS

;
; Error exception -- always fatal
;
;	Use the interrupt stack to handle this.
;	NB: we come here in the event of kernel stack overflow,
;	    so put special code in to handle that case.
;
SCBVEC(error):			; Trap  10 -- error exception
	stcr	r1, cr18

	/* Clear augmentation register so we can reference memory */
	stcr	r10,cr19
	or.u	r10,r0,hi16(_IO_AUG_RD_AND_CLR)
	ld	r10,r10,lo16(_IO_AUG_RD_AND_CLR)
	ldcr	r10,cr19

	stcr	r31,cr17		; save old sp until after shadow
	or.u	r31,r0,hi16(_my_isp)	; run on interrupt stack
	ld	r31,r31,lo16(_my_isp)

	bsr	shadow
	xcr	r10,r10,cr17	
	st	r10,r31,XF(dma2)	; cannibalize this field
	or	r10,r0,T_ERROR		; it is an error exception
	st	r10,r31,XF(type)
	ldcr	r10,cr17
	;
	; Forget about handling fpe's...
	;
	stcr	r0,ssbr		;...just use big hammer to get through fpipe
	bsr	fpipe
	bsr	saveregs
	;
	; test for kernel stack overflow (stack pointer between
	; 0xFBFFC000 and 0xFBFFD000)
	;
	ld	r10,r25,XF(dma2)	; retrieve stack pointer

	or.u	r9,r0,hi16(0xFBFFC000)
	or	r9,r9,lo16(0xFBFFC000)
	cmp	r8,r9,r10		; if r10 < FBFFC000, not overflow
	bb1	hs,r8,generic_error

	or.u	r9,r0,hi16(0xFBFFD000)	; r9 <- base of kernel stack
	or	r9,r9,lo16(0xFBFFD000)
	cmp	r8,r9,r10
	bb1	ls,r8,generic_error	; if r10 > FBFFD000, not overflow

stackoflow_error:
	PANIC(stackoflow)

generic_error:
	PANIC(errmsg)

	MSG(stackoflow,"kernel stack overflow")
	MSG(errmsg,"error exception")

;
; Integer divide error
;
SCBVEC(divide):			; Trap   8 -- integer divide error
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	bsr	fpregs		; save fcr0 - fcr5
	br.n	trapcommon
	or	r10,r0,T_DIVIDE

;
; (truncate: floating point --> ) Integer overflow
;
SCBVEC(overflow):		; Trap   9 -- integer overflow
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	bsr	fpregs		; save fcr0 - fcr5
	br.n	trapcommon
	or	r10,r0,T_OVERFLOW

;
; Floating Point Precise Exception
;
SCBVEC(fprecise):		; Trap 114 -- floating precise exception
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	bsr	fpregs		; save fcr0 - fcr5
	br.n	trapcommon
	or	r10,r0,T_FPRECISE
;
; Floating Point Imprecise Exception
;
;	nasty -- could be draining the pipe
;
SCBVEC(fimprecise):		; Trap 115 -- floating imprecise exception
	stcr	r1, cr18
	ldcr	r1,tpsr
	bb0	PSR_mode,r1,1$	; did we come from user mode ?
	;
	; came from supervisor mode. . . was it from fpdrain ?
	;
	stcr	r10,cr19

	ldcr	r10,sxip		; get sxip
	or.u	r1,r0,hi16(fpdrainx+20)	; did we come from fpdrain ?
	or	r1,r1,lo16(fpdrainx+20)
	cmp	r1,r10,r1
	bb1	hi,r1,5$	; assumes nothing before fpdrain uses FP !!

				; fpdrain got one
	stcr	r31,cr20	; save user's stack pointer
	bsr.n	fpframe		; save FP imprecise frame
	ldcr	r31,cr17	; restore stack pointer

	br	fpipe1		; restart pipe

5$:				; kernel FP imprecise exception
	ldcr	r10,cr19
1$:				; didn't come from fpdrain
	bsr	shadow		; save shadow registers
	or	r10,r0,T_FIMPRECISE
	st	r10,r31,XF(type)
	bsr	fpframe		; save FP imprecise trap info
	bsr	fpfake		; kludge fpipe call !
	br	tcom1		; and jump into trapcommon
	;
	; Kludge alert -- save r1 in right place and jump into fpipe
	; because fpframe changes r31.
fpfake:
	br.n	fpipe1
	st	r1,r31,XF(r1)+XF(FPSIZE)	; past FP Frame

;
; System Call trap
;
SCBVEC(syscall):		; Trap 128 -- system call trap
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	or	r10,r0,T_SYSCALL
	bsr.n	fpipe		; drain floating point pipe
	st	r10,r31,XF(type)
	bsr	saveregs	; save user registers

	bsr.n	_starttimenow	; get time stamp, if safe
	addu	r2,r25,XF(time)

	bsr	_enable		; enable interrupts
	or	r2,r25,r0	; exception frame pointer
	bsr.n	_syscall	; process system call
	or	r3,r31,r0	; FP frame(s), if any
	br	exreturn	; return from exception

;
; Breakpoint -- adb or kdb
;
SCBVEC(udb_bkpt):		; Trap 129 -- breakpoint
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_UDB_BKPT

;
; Panic breakpoint -- 135
;
SCBVEC(panic_bkpt):		; Trap 135 -- panic breakpoint
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_PANIC_BKPT

;
; Trap timing breakpoint
;
SCBVEC(fasttrap):
	rte_WITH_WORKAROUND
	
;
; Read BFLY2 microsecond realtime clock
;
SCBVEC(getusecclock):
	or.u	r2,r0,hi16(_rtc)
	ld	r2,r2,lo16(_rtc)		; r5 gets rtc now
	rte_WITH_WORKAROUND

;
; Read BFLY2 64-bit clock (microseconds). This consists
; of the rtc in the low 32 bits, and a wrap counter in
; the upper 32 bits that is maintained in software. This
; is a per-node synchronized clock. Return 64 bits in r2/r3.
;
SCBVEC(get64bitclock):
	or.u	r6,r0,hi16(_previous_rtc_value)
	ld	r5,r6,lo16(_previous_rtc_value)	; r5 gets rtc at last intr

	or.u	r7,r0,hi16(_rtc)
	ld	r3,r7,lo16(_rtc)		; r3 gets rtc now

	or.u	r8,r0,hi16(_my_64bit_time)
	ld	r2,r8,lo16(_my_64bit_time)  	; r2 get high half at last intr

	; r3 is now.  r5 is then.  We subtract r3 from r5.
	; In normal case this will give us NEGATIVE of ticks since last intr.
	; NB: A borrow will be required, so the carry bit will be set to 0
	; In wrapping case, r5 (then) will be more than r3 (now).
	; We will get POSITIVE number of ticks since last intr.
	; THE CARRY BIT WILL GET SET.
	subu.co	r5,r5,r3

	; Add any wrap into r2 (upper half of clock)
	addu.ci	r2,r2,r0
	rte_WITH_WORKAROUND

;
; Convert virtual to physical address.
; r2 contains virtual address to be translated
; r2 gets physaddr, r3 gets mmu bits, r4 gets physnode
;

SCBVEC(getphysaddr):
	stcr	r1, cr18
	xcr	r31,r31,cr17			; set up supervisor stack

	subu	r31,r31,ALIGNED_MMU_INFO_SIZE+8	; allocate result + save space
	st	r11,r31,ALIGNED_MMU_INFO_SIZE	; save r11 for caller

	or	r3,r31,0			; pass pointer as second arg
	bsr.n	_uvtop_full
	or	r4,r0,r0

	ld.d	r2,r31,0			; get return values...
	ld.d	r4,r31,8

	ld	r11,r31,ALIGNED_MMU_INFO_SIZE
	addu	r31,r31,ALIGNED_MMU_INFO_SIZE+8	; pop stack
	xcr	r31,r31,cr17			; restore user stack ptr
	ldcr	r1,cr18				; and r1
	rte_WITH_WORKAROUND

;
; USER ACCESS TO CMMU 88200 COMMANDS via fasttraps
;
SCBVEC(vm_cache_flush):
	stcr	r1, cr18
	xcr	r31,r31,cr17		; set up supervisor stack
	bsr	_vm_cache_flush
	xcr	r31,r31,cr17		; restore user stack ptr
	ldcr	r1,cr18			; and r1
	rte_WITH_WORKAROUND

;
; Not yet implemented
;
SCBVEC(vm_codecache_flush):
	rte_WITH_WORKAROUND

;
; Not yet implemented
;
SCBVEC(mmu_line_flush):
	rte_WITH_WORKAROUND

;
; Not yet implemented
;
SCBVEC(code_mmu_line_flush):
	rte_WITH_WORKAROUND
;
; Not yet implemented
;
SCBVEC(btransfer):
	rte_WITH_WORKAROUND

#define	ALIGNED_PADDR_INFO_SIZE	16
SCBVEC(getpaddrinfo):
	stcr	r1, cr18
	xcr	r31,r31,cr17			; set up supervisor stack

	subu	r31,r31,ALIGNED_PADDR_INFO_SIZE+8 ; allocate result + save space
	st	r11,r31,ALIGNED_PADDR_INFO_SIZE	; save r11 for caller

	or	r5,r31,0			; pass pointer as second arg
	bsr	_paddr_getinfo

	ld.d	r2,r31,0			; get return values...
	ld.d	r4,r31,8

	ld	r11,r31,ALIGNED_PADDR_INFO_SIZE
	addu	r31,r31,ALIGNED_PADDR_INFO_SIZE+8 ; pop stack
	xcr	r31,r31,cr17			; restore user stack ptr
	ldcr	r1,cr18				; and r1
	rte_WITH_WORKAROUND

;
; USERMODE ACCESS TO ATOMIC OPERATIONS IMPLEMENTED VIA AUGMENTATION REGISTER
;

SCBVEC(usr_atomic_call):
	rte_WITH_WORKAROUND

	; Same args/results as _aug_xmem_16 (except special args)
SCBVEC(usr_atomic_xmem_16):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,1<0>

	bcnd	eq0,r13,1$	; error
	bsr	_aug_xmem_16    ; Do 16-bit "xmem" operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; XMEM_16 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_XMEM_16	; secondary info for trap handler
	rte_WITH_WORKAROUND

	; Same args/results as _aug_cta_8 (except special args)
SCBVEC(usr_atomic_cta_8):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	or	r12,r0,r0	; can't be misaligned

	bcnd	eq0,r13,1$	; error
	bsr	_aug_cta_8	; Do our 32-bit "clear-then-add" operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; CTA_8 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_CTA_8	; secondary info for trap handler

	; Same args/results as _aug_cta_16 (except special args)
SCBVEC(usr_atomic_cta_16):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,1<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_cta_16     ; Do our 32-bit "clear-then-add" operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; CTA_16 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_CTA_16	; secondary info for trap handler

	; Same args/results as _aug_cta_32 (except special args)
SCBVEC(usr_atomic_cta_32):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,2<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_cta_32     ; Do our 32-bit "clear-then-add" operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; CTA_32 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_CTA_32	; secondary info for trap handler

	; Same args/results as _aug_cas_32 (except special args)
SCBVEC(usr_atomic_cas_32):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,2<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_cas_32     ; Do our 32-bit "compare-and-swap" operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; CAS_32 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_CAS_32	; secondary info for trap handler

#if 0
> 	$$$ NOT CURRENTLY SUPPORTED OR BELIEVED WORKING $$$
> 
> 	; Same args/results as _aug_cas2_32 (except special args)
> SCBVEC(usr_atomic_cas2_32):
> 	stcr	r1, cr18
> 
> 	extu	r12,r2,2<0>	; alignment bits
> 	bsr.n	_useraug_check2	; Check r2,r3
> 	extu	r13,r3,2<0>
> 	bcnd	eq0,r13,1$	; error
> 
> 	; r12 set to hi16(address of augmentation register)
> 	; r13 set to bit for locking in aug register
> 	bsr	_aug_cas2_32     ; Do our 32-bit "compare-and-swap" operation
> 	ldcr	r1,cr18		; restore r1
> 	rte_WITH_WORKAROUND
> 
> 	; CAS2_32 failed.  Load up subcode and head for trap
> 1$:
> 	br.n	useraug_error_common
> 	or	r11,r0,USERAUG_CAS2_32	; secondary info for trap handler
#endif 0

SCBVEC(usr_atomic_CAS2_32):
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff0_and_set_8 (except special args)
SCBVEC(usr_atomic_ff0_and_set_8):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	or	r12,r0,r0	; can't be misaligned

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff0_and_set_8     ; Do our 8-bit ff0 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF0_AND_SET_8 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF0_AND_SET_8	; secondary info to trap handlr
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff0_and_set_16 (except special args)
SCBVEC(usr_atomic_ff0_and_set_16):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,1<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff0_and_set_16     ; Do our 16-bit ff0 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF0_AND_SET_16 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF0_AND_SET_16	; secondary info to trap handlr
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff0_and_set_32 (except special args)
SCBVEC(usr_atomic_ff0_and_set_32):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,2<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff0_and_set_32     ; Do our 32-bit ff0 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF0_AND_SET_32 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF0_AND_SET_32	; secondary info to trap handlr
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff1_and_clear_8 (except special args)
SCBVEC(usr_atomic_ff1_and_clear_8):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	or	r12,r0,r0	; can't be misaligned

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff1_and_clear_8     ; Do our 8-bit ff1 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF1_AND_CLEAR_8 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF1_AND_CLEAR_8	; secondary info to trap handlr
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff1_and_clear_16 (except special args)
SCBVEC(usr_atomic_ff1_and_clear_16):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,1<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff1_and_clear_16     ; Do our 16-bit ff1 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF1_AND_CLEAR_16 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF1_AND_CLEAR_16	; secondary info to trap handlr
	rte_WITH_WORKAROUND

	; Same args/results as _aug_ff1_and_clear_32 (except special args)
SCBVEC(usr_atomic_ff1_and_clear_32):
	stcr	r1, cr18

	; r12 set to hi16(address of augmentation register)
	; r13 set to bit for locking in aug register
	bsr.n	_useraug_check
	extu	r12,r2,2<0>	; alignment bits

	bcnd	eq0,r13,1$	; error
	bsr	_aug_ff1_and_clear_32     ; Do our 32-bit ff1 operation
	ldcr	r1,cr18		; restore r1
	rte_WITH_WORKAROUND

	; FF1_AND_CLEAR_32 failed.  Load up subcode and head for trap
1$:
	br.n	useraug_error_common
	or	r11,r0,USERAUG_FF1_AND_CLEAR_32	; secondary info to trap handlr
	rte_WITH_WORKAROUND

;
; Reserved trap
;
SCBVEC(reserved):		; Reserved traps all come here
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	br.n	trapcommon
	or	r10,r0,T_RESERVED

	global trapcommon
trapcommon:			; TRAPCOMMON
	st	r10,r31,XF(type)
	bsr	fpipe		; drain floating point pipe
tcom1:
	bsr	saveregs	; save user registers

	bsr.n	_starttimenow	; get time stamp, if safe
	addu	r2,r25,XF(time)	; r25 is frame pointer now

	bsr	_enable		; enable interrupts
	or	r2,r25,0	; address of exception frame
	or	r3,r31,0	; address of FP frame(s), if any
	subu	r31, r31, 8	; make fp, return address frame ...
	ld	r4, r25, XF(sxip)
	clr	r4, r4, 2<0>	; clear IP_V|IP_E
	st	r4, r31, 4
	st	r30, r31, 0
	bsr.n	_trap		; process
	addu	r30, r31, 0

	br	exreturn	; return from this exception ...

;
; Trap10 used by user-mode sleep_and_unlock, user-mode wakeup
;
;	r2 = function code 
;		0 => user-mode sleep_and_unlock
;		1 => user-mode wakeup
;	r3 = wait-channel (both)
;	r4 = &lock (sleep_and_unlock only)
;	r5 = timeout (sleep_and_unlock only)
;
SCBVEC(trap10):
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	ldcr	r10,tpsr	; did we come from supervisor mode ?
	bb1	PSR_mode,r10,t10panic
	bsr	fpipe		; drain floating point pipe
	bsr	saveregs	; save user registers

	bsr.n	_starttimenow	; get time stamp, if safe
	addu	r2,r25,XF(time)	; r25 is frame pointer now

	bsr	_enable		; enable interrupts

	or	r2,r25,r0	; exception frame pointer
	bsr.n	_ex_cleanup	; deal with pipes
	or	r3,r31,r0	; FP frame(s), if any

	or.u	r10, r0, hi16(_u+300)	; (u.u_ap) place in u block for user arguments
	ld	r10, r10, lo16(_u+300)	; expected by user_sleep_and_unlock and user_wakeup ...

	ld	r2, r25, XF(r3)
	st	r2, r10, 0		; store argument 1
	ld	r3, r25, XF(r4)
	st	r3, r10, 4		; store argument 2
	ld	r4, r25, XF(r5)
	st	r4, r10, 8		; store argument 3

	ld	r2, r25, XF(r2)		; check for function ...
	bcnd	ne0, r2, calluser_wakeup
	bsr	_user_sleep_and_unlock
	br	trap10end

calluser_wakeup:
	bsr	_user_wakeup
trap10end:
	or.u	r10, r0, hi16(_u+384)	; u.u_r.r_val1 ...
	ld	r10, r10, lo16(_u+384)	; get return code
	bcnd	ge0, r10, NotAnError
	or.u	r11, r0, hi16(_u+392)	; (u.u_error) u.u_r.r_val1 == -1, return u.u_error in users errno ...
	ld.b	r11, r11, lo16(_u+392)	;	and implement skip return to indicate error
	st	r11, r25, XF(r2)	; store for placement in errno
	ld	r11, r25, XF(snip)	; skip return for both snip and sfip ...
	addu	r11, r11, 4
	st	r11, r25, XF(snip)
	ld	r11, r25, XF(sfip)
	addu	r11, r11, 4
	br.n	exreturn
	st	r11, r25, XF(sfip)
NotAnError:
	br.n	exreturn
	st	r10, r25, XF(r2)
t10panic:
	bsr	saveregs
	PANIC(t10msg)
	MSG(t10msg,"Trap 10 in supervisor mode")

;
; Trap11 -- Interface to user gist logging
;
SCBVEC(trap11):
#if GIST
	stcr	r1, cr18
	bsr	shadow		; save shadow registers
	ldcr	r10,tpsr	; did we come from supervisor mode ?
	bb1	PSR_mode,r10,t11panic
	bsr	fpipe		; drain floating point pipe
	bsr	saveregs	; save user registers

	bsr.n	_starttimenow	; get time stamp, if safe
	addu	r2,r25,XF(time)	; r25 is frame pointer now

	bsr	_enable		; enable interrupts

	or	r2,r25,r0	; exception frame pointer
	bsr.n	_ex_cleanup	; deal with pipes
	or	r3,r31,r0	; FP frame(s), if any

	ld	r2, r25, XF(r2)	; gather the three arguments ...
	ld	r3, r25, XF(r3)
	ld	r4, r25, XF(r4)
	bsr	_user_elog
	br	exreturn	; return to the user
t11panic:
	bsr	saveregs
	PANIC(t11msg)
	MSG(t11msg,"Trap 11 in supervisor mode")
#else GIST
	rte_WITH_WORKAROUND
#endif GIST


SCBVEC(DivByZeroTrap):			; workaround for Divide-By-Zero errata
	stcr	r1, cr18
	ldcr	r1, sxip
	addu	r1, r1, 4
	stcr	r1, sxip		; skip tcnd instruction
	ldcr	r1, snip
	addu	r1, r1, 4
	stcr	r1, snip		; increment by one instruction
	ldcr	r1, sfip
	addu	r1, r1, 4
	stcr	r1, sfip		; increment by one instruction
	br.n	_Xdivide
	ldcr	r1, cr18		; go to real divide-by-zero vector ...

;
; fpframe -- create floating point imprecise frame
;
fpframe:
	subu	r31,r31,XF(FPSIZE)	; make room
	or.c	r10,r0,r0		; -1 identifies FP frame
	st	r10,r31,0

	fldcr	r10,cr0
	st	r10,r31,XF(fpimpcr)	; fpecr
	fldcr	r10,cr6
	st	r10,r31,XF(fprh)
	fldcr	r10,cr7
	st	r10,r31,XF(fprl)
	fldcr	r10,cr8
	st	r10,r31,XF(fpit)
	;
	; for double precision results, ssbr bit is set for second register.
	; first (or only) bit is cleared when exception is recognized by H/W.
	;
	bb0	10,r10,1$		; single precison ?

	xcr	r1,r1,ssbr		; get ssbr, save r1
	extu	r10,r10,5<0>		; get register specifier
	or	r10,r10,0x40		; clear 2 bit wide field
	clr	r1,r1,r10		; . . . it's easier !
	xcr	r1,r1,ssbr		; update ssbr, retrieve r1
1$:
	jmp	r1

;
; fpipe -- drain floating point pipeline
;
;	need to rte to trap instruction
;	which forces pipe to drain
;
	global	fpipe			; for vmunix.map
	global	fpdrain
fpipe:
	st	r1,r31,XF(r1)		; save return address
	ldcr	r10,ssbr		; save shadow scoreboard 
	st	r10,r31,XF(ssbr)
fpipe1:
	ldcr	r10,psr			; get psr
	clr	r10,r10,1<PSR_sfrz>	; enable exceptions
	clr	r10,r10,1<PSR_sfd1>	; enable floating point unit
	stcr	r10,tpsr		; set tpsr for rte

	stcr	r0,snip			; clear snip (invalid --> nop)
	or.u	r10,r0,hi16(fpdrain+2)	; setup for rte
	or	r10,r10,lo16(fpdrain+2)	; +2 makes fip valid
	stcr	r10,sfip		; valid sfip --> fpdrain

	stcr	r31,cr17		; save supervisor stack pointer

	ldcr	r1,cr18			; restore user's r1
	ldcr	r10,cr19		; . . . and r10
	ldcr	r31,cr20		; . . . and r31
	rte_WITH_WORKAROUND		; go let pipe drain
	or	r0,r0,r0		; so DAS can see

fpdrain:
	tb1	0,r0,0			; let pipe drain
fpdrainx:				; label for panic check

	;
	; FP pipe is drained now
	; Exceptions are enabled
	; Real scoreboard == ssbr == 0
	;
	stcr	r1,cr18
	stcr	r31,cr20	; get registers into right places
	stcr	r10,cr19

	ldcr	r31,cr17
	subu	r10,r31,XF(FPSIZE)
xframe:				; find exception frame
	addu	r10,r10,XF(FPSIZE)
	ld	r1,r10,0
	bcnd	lt0,r1,xframe	; -1 identifies FP frame

	st	r25,r10,XF(r25)
	or	r25,r10,0	; r25 is exception frame pointer

	ld	r1,r25,XF(r1)	; return address
	jmp	r1

;
; shadow -- create stack frame on supervisor stack and save shadow registers
;
; assumes:
;	r1 is stored in cr18
;	supervisor stack pointer is in cr17 ... if we just came from user mode
;
;	frees r10 for temp use
;
	global	shadow			; for vmunix.map
shadow:
	stcr	r10,cr19		; free a working register
	ldcr	r10,tpsr		; get shadow psr
	bb1.n	PSR_mode,r10,super	; did we come from supervisor mode ?
	stcr	r31,cr20		; (save current stack pointer)

	ldcr	r31,cr17		; if from user, get super stack pointer
super:
	subu	r31,r31,XF(SIZE)	; make space for exception frame

	ldcr	r10,cr8			; must capture dmt0 BEFORE first st.
	bb0.n	0,r10,1$		; -- At least until Rev. D5 chips --
	st	r10,r31,XF(dmt0)	; branch if not valid

	xcr	r1,r1,ssbr		; save r1, get ssbr

	bb1	1,r10,4$		; store ?
	extu	r10,r10,5<7>		; get register specifier
	or	r10,r10,0x20		; 1 bit wide field
	clr	r1,r1,r10		; clear ssbr bit
4$:
	ldcr	r10,cr9
	st	r10,r31,XF(dmd0)
	ldcr	r10,cr10
	st	r10,r31,XF(dma0)

	ldcr	r10,cr11
	bb0.n	0,r10,3$		; branch if not valid
	st	r10,r31,XF(dmt1)

	bb1	1,r10,33$		; store ?
	extu	r10,r10,5<7>		; get register specifier
	or	r10,r10,0x20		; 1 bit wide field
	clr	r1,r1,r10		; clear ssbr bit
33$:
	ldcr	r10,cr12
	st	r10,r31,XF(dmd1)
	ldcr	r10,cr13
	st	r10,r31,XF(dma1)
3$:
	ldcr	r10,cr14
	bb0.n	0,r10,2$		; branch if not valid
	st	r10,r31,XF(dmt2)

	bb1	1,r10,22$		; store ?
	bb1.n	13,r10,24$		; ld.d ?
	extu	r10,r10,5<7>		; get register specifier

	br.n	23$			; ld
	or	r10,r10,0x20		; 1 bit wide field

24$:					; ld.d
	or	r10,r10,0x40		; 2 bit wide field
23$:
	clr	r1,r1,r10		; clear ssbr bit(s)
22$:
	ldcr	r10,cr15
	st	r10,r31,XF(dmd2)
	ldcr	r10,cr16
	st	r10,r31,XF(dma2)
2$:					; restore r1, update ssbr
	xcr	r1,r1,ssbr		; bit 0 comes back clear from ssbr
					; but that's ok for r1
1$:
	st	r31,r31,XF(recovery)	; store stack recovery pointer

	st	r0,r31,0		; clear frame type
	ldcr	r10,tpsr		; save shadow registers
	st	r10,r31,XF(tpsr)
	ldcr	r10,sxip
	st	r10,r31,XF(sxip)

	ldcr	r10, snip
	bb1	1, r10, SNIPok		; check for valid SNIP
	ldcr	r10, sfip		; not valid snip, use sfip as snip
	st	r10, r31, XF(snip)
	addu	r10, r10, 4		; and recalculate the sfip ...
	st	r10, r31, XF(sfip)
	br	IPScopied
SNIPok:					; valid snip, sfip ...
	st	r10,r31,XF(snip)
	ldcr	r10,sfip
	st	r10,r31,XF(sfip)
IPScopied:

	or.u	r10,r0,hi16(DMMU_lsr)	; save MMU status registers
	ld	r10,r10,lo16(DMMU_lsr)
	st	r10,r31,XF(dlsr)
	or.u	r10,r0,hi16(IMMU_lsr)
	ld	r10,r10,lo16(IMMU_lsr)
	st	r10,r31,XF(ilsr)
	or.u	r10,r0,hi16(IMMU2_lsr)
	ld	r10,r10,lo16(IMMU2_lsr)
	jmp.n	r1			; done
	st	r10,r31,XF(ilsr2)

;
; fpregs -- save FP precise registers (fcr0 - fcr5)
;
fpregs:
	fldcr	r10,cr0
	st	r10,r31,XF(fpecr)
	fldcr	r10,cr1
	st	r10,r31,XF(fphs1)
	fldcr	r10,cr2
	st	r10,r31,XF(fpls1)
	fldcr	r10,cr3
	st	r10,r31,XF(fphs2)
	fldcr	r10,cr4
	st	r10,r31,XF(fpls2)
	fldcr	r10,cr5
	st	r10,r31,XF(fppt)

	xcr	r1,r1,ssbr		; get ssbr, save r1
	bb1.n	5,r10,1$		; double precison ?
	extu	r10,r10,5<0>		; get register specifier

	br.n	2$			; single precision
	or	r10,r10,0x20		; 1 bit wide field

1$:					; double precision
	or	r10,r10,0x40		; 2 bit wide field
2$:
	clr	r1,r1,r10		; clear ssbr bit(s)
	xcr	r1,r1,ssbr		; update ssbr, retrieve r1

	jmp	r1

;
; saveregs -- save general registers
;
	global	saveregs		; for vmunix.map
saveregs:
	ldcr	r10,cr18		; retrieve r1
	st	r0, r25, XF(r0)
	st	r10,r25,XF(r1)
	st	r2,r25,XF(r2)
	st	r3,r25,XF(r3)
	st	r4,r25,XF(r4)
	st	r5,r25,XF(r5)
	st	r6,r25,XF(r6)
	st	r7,r25,XF(r7)
	st	r8,r25,XF(r8)
	st	r9,r25,XF(r9)
	ldcr	r10,cr19		; retrieve r10
	st	r10,r25,XF(r10)
	st	r11,r25,XF(r11)
	st	r12,r25,XF(r12)
	st	r13,r25,XF(r13)
	st	r14,r25,XF(r14)
	st	r15,r25,XF(r15)
	st	r16,r25,XF(r16)
	st	r17,r25,XF(r17)
	st	r18,r25,XF(r18)
	st	r19,r25,XF(r19)
	st	r20,r25,XF(r20)
	st	r21,r25,XF(r21)
	st	r22,r25,XF(r22)
	st	r23,r25,XF(r23)
	st	r24,r25,XF(r24)
					; already have r25
	st	r26,r25,XF(r26)
	st	r27,r25,XF(r27)
	st	r28,r25,XF(r28)
	st	r29,r25,XF(r29)
	st	r30,r25,XF(r30)
	ldcr	r10,cr20		; retrieve r31
	st	r10,r25,XF(r31)

	fldcr	r10,cr62		; floating status register
	st	r10,r25,XF(fpsr)
	fldcr	r10,cr63		; floating control register
	st	r10,r25,XF(fpcr)

	jmp	r1

;
; exreturn -- restore registers and return from exception
;
	global	exreturn		; for vmunix.map
exreturn:
	ldcr	r10, psr		; get psr
	set	r10, r10, 1<PSR_ind>	; disable interrupts
	stcr	r10, psr

	ld	r10,r25,XF(tpsr)	; forget profiling if not user mode
	bb1	PSR_mode,r10,1$

	or.u	r2, r0, hi16(_do_profile)
	ld.bu	r2, r2, lo16(_do_profile)
	bcnd	eq0, r2, 1$		; no ticks pending
	bsr.n	_check_profiling_ticks	; Account for profiling ticks
	or	r2, r25, r0
1$:
	ld	r10,r25,XF(fpsr)
	fstcr	r10,cr62		; floating status register
	ld	r10,r25,XF(fpcr)
	fstcr	r10,cr63		; floating control register

	ld	r1,r25,XF(r1)		; general purpose registers
	ld	r2,r25,XF(r2)
	ld	r3,r25,XF(r3)
	ld	r4,r25,XF(r4)
	ld	r5,r25,XF(r5)
	ld	r6,r25,XF(r6)
	ld	r7,r25,XF(r7)
	ld	r8,r25,XF(r8)
	ld	r9,r25,XF(r9)
					; skip r10 for now
	ld	r11,r25,XF(r11)
	ld	r12,r25,XF(r12)
	ld	r13,r25,XF(r13)
	ld	r14,r25,XF(r14)
	ld	r15,r25,XF(r15)
	ld	r16,r25,XF(r16)
	ld	r17,r25,XF(r17)
	ld	r18,r25,XF(r18)
	ld	r19,r25,XF(r19)
	ld	r20,r25,XF(r20)
	ld	r21,r25,XF(r21)
	ld	r22,r25,XF(r22)
	ld	r23,r25,XF(r23)
	ld	r24,r25,XF(r24)
					; skip r25 for now
	ld	r26,r25,XF(r26)
	ld	r27,r25,XF(r27)
	ld	r28,r25,XF(r28)
	ld	r29,r25,XF(r29)
	ld	r30,r25,XF(r30)
	;
	; restore shadow registers
	;
	tb1	0,r0,0			; let pipe drain before
					;   disabling shadowing.
	ldcr	r10,psr			; get psr
	set	r10,r10,1<PSR_sfrz>	; and shadowing
	stcr	r10,psr

	ld	r10,r25,XF(snip)
	stcr	r10,snip
	ld	r10,r25,XF(sfip)	; sfip
	stcr	r10,sfip

	ld	r10,r25,XF(tpsr)
	stcr	r10,tpsr

	ld	r10,r25,XF(recovery)	; recovery frame pointer
	addu	r10,r10,XF(SIZE)	; de-allocate exception frame
	stcr	r10,cr17

	ld	r10,r25,XF(r10)		; r10 at last
	ld	r31,r25,XF(r31)		; and user stack pointer
	ld	r25,r25,XF(r25)		; and r25

	stcr	r0,ssbr			; clear ssbr
	rte_WITH_WORKAROUND

#include "bbn_assembler_handlesi.h"
#if BBN_ASSEMBLER_HANDLESI
#include "il_kdb.h"
;
; HandleSI - software interrupt handler.
;
; This is called whenever the interrupt priority drops low enough to
; handle a pending software interrupt, it's called from splx() 
; and from the tail end of the interrupt handler.
;
; This duplicates some of the code in the interrupt handler, but
; doesn't bother with maintaining the IPI enable
;
; This routine is only called when the current interrupt level is
; < IL_SI_HIGH
	global	_HandleSI
; Called
; r2 - current hardpri
; r3 - exception frame pointer
;
; Returns
_HandleSI:
	subu	r31, r31, 16		; Reserve space for 4 long words
	st	r1, r31, 12		; Make a normal stack frame
	st	r30, r31, 8
	addu	r30, r31, 8

	st.d	r14, r31, 0		; Save two registers
	or	r15, r2, r0		; Save the hardpri value
	or	r14, r3, r0		; and the exception frame

	.stabs	":MPr2,4;r3,4;;",196,0,0,_HandleSI
	.stabs	":MSr2,r15,4;r3,r14,4;r14,-8,4;r15,-4,4;;",196,0,0,LL105
LL105:

; r15 - new level
; r14 - exception frame
; Optimized for one time through the loop, thus we don't cache any
; temporary values.
	or	r4, r0, 2		; r4 = 0x02
HandleSILoop:
	mak	r4, r4, r15		; r4 = 0x02 << level
	subu	r4, r4, 1		; r4 = mask = mask - 1
	bsr.n	_disable			; Disable
	or.u	r8, r0, hi16(_SIpending)	; r8 = hi16(_SIpending)
	ld	r6, r8, lo16(_SIpending)	; r6 = SIPending
	and.c	r3, r6, r4		; r3 = SIpending & ~mask
	bcnd.n	eq0, r3, HandleSIDone	; Done if no more pending > level
	or.u	r7, r0, hi16(_hardpri)	; r7 = hi16(_hardpri)

;
; Special case: a KDB interrupt needs a real exception frame. In
; the case that we are called from splx() we will have a fake one
; distinguished by xf == 0. In that case we want to defer
; processing until a real interrupt
;
	bb0.n	IL_SI_KDB, r6, 1f
	ff1	r3, r6			; r3 = si = findfirstset(SIpending)

; here iff a KDB interrupt is pending
	bcnd	eq0, r14, HandleSIDone	; Called from splx - defer

1:	or.u	r2, r0, hi16(_IntTable)
	or	r2, r2, lo16(_IntTable)
	ld	r4, r2[r3]		; r1 = ifunc = IntTable[si]
	clr	r6, r6, r3		; SIpending &= ~(1<<si)
	st	r6, r8, lo16(_SIpending)	; SIpending = ...
	bsr.n	_enable			; reenable
	st	r3, r7, lo16(_hardpri)	; hardpri = si
	jsr.n	r4			; ifunc(xframep)
	or	r2, r14, r0		; r2 = exception frame
	br.n	HandleSILoop
	or	r4, r0, 2		; r4 = 0x02

HandleSIDone:
; r7 - hi16(_hardpri)
; r15 - final hardpri
; interrupts disabled
	bsr.n	_enable
	st	r15, r7, lo16(_hardpri)

; Restore registers
	ld	r1, r31, 12
	ld.d	r14, r31, 0
	ld	r30, r31, 8
	jmp.n	r1			; and return
	addu	r31, r31, 16

#endif BBN_ASSEMBLER_HANDLESI

;
; Disable interrupts
; preserves argument registers
	global	_disable
_disable:
	ldcr	r10,psr			; disable interrupts
	set	r10,r10,1<PSR_ind>
	jmp.n	r1
	stcr	r10,psr

;
; Enable interrupts
; preserves argument registers
	global	_enable
_enable:
	ldcr	r10,psr			; reenable interrupts
	clr	r10,r10,1<PSR_ind>
	jmp.n	r1
	stcr	r10,psr
;
; Hardclock interrupt handler wrapper
;
	global	_hardclock_interface
_hardclock_interface:
	subu	r31, r31, 16
	st	r1, r31, 12
	st	r30, r31, 8
	addu	r30, r31, 8
	st	r14, r31, 0
	or	r14, r2, 0
	.stabs	":MPr2,4;;",196,0,0,_hardclock_interface
	.stabs	":MSr2,r14,4;r14,-8,4;;",196,0,0,LL101
LL101:
	ld	r2, r14, XF(sxip)	; arg 1 - exception pc
	ld	r3, r14, XF(tpsr)	; arg 2 - psr
; XXX: for hardclock delay histogram. Get caller caller pc -
; the exception pc is usually splx from supervisor mode
;	
	bb0.n	PSR_mode,r10,1$	; skip if from user mode
; From supervisor mode - trace back the fp
	ld	r4, r14, XF(r30)	; old frame pointer

; Have to validate whether stack is one of the kernel stacks
	xor.u	r5,r4,hi16(KERNEL_STACK_VADDR); are we on kernel stack ?
	and	r5,r5,0			; clear low halfword
	bcnd.n	eq0,r5,2$			; result is 0 if on sup stk

	or.u	r5,r0,hi16(_my_isp)		; switch to interrupt stack
	ld	r5,r5,lo16(_my_isp)
	xor	r5, r5, r4
	and	r5, r5,0
	bcnd	ne0, r5, 1$			; not on istack either
2$:
	ld	r4, r4, 4	; extract caller pc from old frame
1$:
; XXX: end of hardclock histogram code
	bsr.n	_hardclock
	clr	r2, r2, 2<0>

;	.ef
	ld	r1, r31, 12
	ld	r14, r31, 0
	ld	r30, r31, 8
	jmp.n	r1
	addu	r31, r31, 16

;
; Softclock interrupt handler wrapper
;
; If this is called from splx() then the exception frame
; pointer will be 0
	global	_softclock_interface
_softclock_interface:
	subu	r31, r31, 16
	st	r1, r31, 12
	st	r30, r31, 8
	addu	r30, r31, 8
	st	r14, r31, 0
	or	r14, r2, r0

	bcnd.n	eq0, r14, 1f
	or.u	r3, r0, 0x8000		; assume psr supervisor mode (1b31)

	.stabs	":MPr2,4;;",196,0,0,_softclock_interface
	.stabs	":MSr2,r14,4;r14,-8,4;;",196,0,0,LL100
LL100:
	ld	r2, r14, XF(sxip)
	ld	r3, r14, XF(tpsr)
1:	bsr.n	_softclock
	clr	r2, r2, 2<0>
;	.ef
	ld	r1, r31, 12
	ld	r30, r31, 8
	ld	r14, r31, 0
	jmp.n	r1
	addu	r31, r31, 16
;
; Kernel DeBugger interrupt handler
;
#if CS_KDB
	global	_kdb_interface
_kdb_interface:
	subu	r31, r31, 16
	st	r1, r31, 12
	st	r30, r31, 8
	addu	r30, r31, 8
	st	r14, r31, 0
	or	r14, r2, r0
	.stabs	":MPr2,4;;",196,0,0,_kdb_interface
	.stabs	":MSr2,r14,4;r14,-8,4;;",196,0,0,LL102
LL102:
	or	r10, r0, T_KDB_ENTRY
	bsr.n	_kdb_trap
	st	r10, r14, XF(type)
;	.ef
	ld	r1, r31, 12
	ld	r30, r31, 8
	ld	r14, r31, 0
	jmp.n	r1
	addu	r31, r31, 16
#endif CS_KDB

;
; Return true if we are currently executing on the kernel stack.
;
	global	_on_kernel_stack
_on_kernel_stack:
	or	r2,r0,0				; clear return value
	xor.u	r10,r31,hi16(KERNEL_STACK)	; are we on kernel stack ?
	and	r10,r10,0			; clear low halfword
	bcnd	ne0,r10,1$
	or	r2,r0,1				; return TRUE
1$:	jmp	r1
