/* $Header: /nfs/europa/u1/BFMACH.local/mk/bfly2/RCS/tcs.h,v 1.18 90/11/10 16:37:06 tappan Exp $ */
/* tcs.h  -  taken from uboot.h, which was mostly taken from debug.h */
				                                               
#include	"../bfly2/card_types.h"		/* EEPROM signatures */

	/* Setting this bit in signature indicates card is disabled */
#define	ACARD_DISABLE	0x80


/*	TCS definitions		*/

#define	RING_SIZE		128  /* ?? */
#define	MAX_NODES		512
#define	SERIAL_NUMBER_LEN	16
#define MAX_TCS_ARGS		127  /* maximum number of char args */

/* TCS Service codes */
#define SVC_NONE		0x00	/* NOP - quiescent condition */
#define SVC_SET_COMP_STATE	0x10
#define SVC_GET_COMP_STATE	0x11
#define SVC_SHUTDOWN		0x20	/* request TCS to shutdown */
#define SVC_REBOOT		0x21	/* request TCS to reboot */
#define SVC_REREAD_CONFIG	0x22
#define SVC_GET_BOOTSTRING	0x23
#define SVC_SET_BOOTSTRING	0x24
#define SVC_GET_BOOTVAR		0x25	/* request TCS to devulge var value */
#define SVC_SET_BOOTVAR		0x26	/* request TCS to reset var value */
#define SVC_GET_CALENDAR	0x27
#define SVC_SET_CALENDAR	0x28
#define SVC_EXIT		0x29	/* less drastic halt */
#define SVC_WARM_BOOT_NODE	0x2a	/* just reload and start uboot */
#define SVC_COLD_BOOT_NODE	0x2b	/* full reset and clear then uboot */
#define SVC_DOS_OPENR		0x40	/* DOS open file for read */
#define SVC_DOS_OPENW		0x41	/* DOS open file for write */
#define SVC_DOS_OPENA		0x42	/* DOS open file for append */
#define SVC_DOS_SEEKR		0x43	/* DOS seek file read */
#define SVC_DOS_SEEKW		0x44	/* DOS seek file write */
#define SVC_DOS_READ		0x45	/* DOS read file */
#define SVC_DOS_WRITE		0x46	/* DOS write file */
#define SVC_DOS_CLOSER		0x47	/* DOS close file read */
#define SVC_DOS_CLOSEW		0x48	/* DOS close file write */
#define SVC_DOS_DELETE		0x49	/* DOS delete file */
#define SVC_DOS_MKDIR		0x4a	/* DOS make directory */
#define SVC_DOS_RMDIR		0x4b	/* DOS remove directory */
#define SVC_DOS_GETCWD		0x4c	/* DOS get working directory */
#define SVC_TCS_RD_ACTION	0x80	/* read TCS slave action register */
#define SVC_TCS_WR_ACTION	0x81	/* write TCS slave action register */
#define SVC_TCS_RD_EEPROM	0x82	/* read TCS slave eeprom */
#define SVC_TCS_WR_EEPROM	0x83	/* write TCS slave eeprom */
#define SVC_TCS_RD_GATEARRAY	0x84	/* read TCS slave gate array reg. */
#define SVC_TCS_WR_GATEARRAY	0x85	/* write TCS slave gate array reg. */
#define SVC_TCS_RD_HWARE	0x86	/* read TCS slave hardware register */
#define SVC_TCS_WR_HWARE	0x87	/* write TCS slave hardware register */
#define SVC_TBUS_RD		0x88	/* perform a TCS TBUS read request */
#define SVC_TBUS_WR		0x89	/* perform a TCS TBUS write request */

/* component states */
#define	SVC_CS_SYSTEM		1	/* system (normal for nX nodes) */
#define	SVC_CS_AUX		2	/* auxilary (normal for bare nodes) */
#define	SVC_CS_ISOLATE		3	/* isolated (for diagnostics) */
#define	SVC_CS_OFF		4	/* off or disabled */

/* BOOT var arguements */
#define BV_PANIC	0	/* number of times through boot */
#define BV_PANIC_LIM	1	/* limit of panics before we shutdown */
#define BV_AUTO		2	/* 0 if not autoboot */
#define BV_BITS		3	/* various boothowto flags */

/* boothowto flags - used in BV_BITS: see reboot.h */

/* bits for to_tcs_bits field.  Note that Halt, idle and run
 * are mutually exclusive states
 */
#define TO_TCS_CONOUT_INT_EN    1
#define TO_TCS_KEYBIN_INT_EN    2
#define TO_TCS_SRVREQ_INT_EN    4

#define FROM_TCS_SRV_DONE       1
#define FROM_TCS_SRV_ERR	2

#define TO_TCS_HALT_STATE       0x0
#define TO_TCS_IDLE_STATE       0x40
#define TO_TCS_RUN_STATE        0x80

/* bit defs for boot_bits field */
#define BOOT_SINGLE_USER        0x01
#define BOOT_MFG_DIAG           0x02
#define BOOT_FIELD_DIAG         0x04
#define BOOT_UNATTENDED_DIAG    0x08
#define BOOT_LOOP_ON_ERR_DIAG   0x10
#define BOOT_HALT_ON_ERR_DIAG   0x20

#define BOOT_TCS_INATTENTIVE	0x80000000

/* bit defs for halt_bits field */
/* used only by board level diagnostics */
#define HALT_FOUND_CPU          0x01
#define HALT_PASSED_CPU         0x02
#define HALT_FOUND_XY0          0x04
#define HALT_PASSED_XY0         0x08
#define HALT_FOUND_MT0          0x10
#define HALT_PASSED_MT1         0x20
 /* etc */

/* node types */
#define TYPE_EMPTY_SLOT     -1
#define TYPE_4MEG_B2VME     1
#define TYPE_16MEG_B2VME    2

/* component clases */
#define TC_CLK		0	/* clock card */
#define TC_SSA		1	/* sender switch A */
#define TC_SRA		2	/* receiver switch A */
#define TC_SSB		3	/* sender switch B */
#define TC_SRB		4	/* receiver switch B */
#define TC_FP		5	/* any type of processor card */



/*	address of the tcs data structure	*/
#define	TCS		0x00001800

/*	the tcs data structure	*/
#ifdef	ASSEMBLER

#define TO_TCS_CMD		(0)
#define TO_TCS_BITS		(TO_TCS_CMD+0x1)
#define CONS_RING_OUTPUT_OFFSET	(TO_TCS_CMD+0x2)
#define CONS_RING_INPUT_OFFSET	(TO_TCS_CMD+0x3)
#define FROM_TCS_CMD		(TO_TCS_CMD+0x4)
#define FROM_TCS_BITS		(TO_TCS_CMD+0x5)
#define KEY_RING_OUTPUT_OFFSET	(TO_TCS_CMD+0x6)
#define KEY_RING_INPUT_OFFSET	(TO_TCS_CMD+0x7)
#define BOOT_BITS		(TO_TCS_CMD+0x8)
#define HALT_BITS		(TO_TCS_CMD+0xC)
#define TO_TCS_ARGS		(TO_TCS_CMD+0x10)
#define FROM_TCS_RESPONSE	(TO_TCS_CMD+0x14)
#define CONS_OUT_RING_BUF	(TO_TCS_CMD+0x18)
#define KEY_OUT_RING_BUF	(TO_TCS_CMD+0x18+RING_SIZE)
#define TERMINAL_TYPE		(TO_TCS_CMD+0x18+2*RING_SIZE)
#define END_TO_TCS_CMD		(TO_TCS_CMD+0x18+2*RING_SIZE+1)  

/*	watch the alignment! 	*/
#define TCS_CONFIG		(TO_TCS_CMD+0x18+2*RING_SIZE+1+3)  
#define NODE_TYPE		(TCS_CONFIG)
#define SERIAL_NUMBER		(TCS_CONFIG+MAX_NODES)
#define MY_NODE_NUM		(TCS_CONFIG+MAX_NODES+SERIAL_NUMBER_LEN)
#define SWITCH_COLUMNS		(TCS_CONFIG+MAX_NODES+SERIAL_NUMBER_LEN+2)
#define END_TCS_CONFIG		(TCS_CONFIG+MAX_NODES+SERIAL_NUMBER_LEN+4)

#else
  /* C definitions on next page	*/

/*	TCS structure definitions, continued */

struct ring_header_struct {
    char ring_output_offset;
    char ring_input_offset;
};

struct tcs_io_struct {
  /* first long word -- console output, service requests */
    char to_tcs_cmd;        /* zero means 'nothing to do' */
    char to_tcs_bits;       /* defined above ... TO_TCS bits */
    struct ring_header_struct cons_out_ring;
  /* second long word -- keyboard input, service responses */
    char from_tcs_cmd;
    char from_tcs_bits;     /* indicates tcs service request complete */
    struct ring_header_struct keyboard_ring;

    unsigned long boot_bits;     /* defined above ... BOOT* bits */
    unsigned long halt_bits;     /* defined above */
 
    char *to_tcs_args;          /* see text */
    char *from_tcs_response;

    char cons_out_ring_buf[RING_SIZE];
    char keyboard_ring_buf[RING_SIZE];

    char terminal_type;         /* console type - vt320, vt100, hardcopy */
};


/*
  TCS System Configuration structure
*/

struct tcs_config_struct {
    char node_type[MAX_NODES];              /* what's in each function slot */
    char serial_number[SERIAL_NUMBER_LEN];  /* local node's serial number */
    short my_node_num;                      /* local node's node-number */
    short switch_columns;                   /* 1, 2 or 3 */
};


struct tcs_struct {
    struct tcs_io_struct io;            /* console i/o, run/halt info */
    struct tcs_config_struct config;    /* system config info */
};

#ifdef KERNEL
#include	"../butterfly/pmap.h"
	/* Address of TCS stuff in 0xFE (virtual) space */
#define	tcs	((struct tcs_struct *) (PHYS_MAP_VADDR | TCS))
#define	tcs_io	(&tcs->io)

extern char	tcs_initialized;	/* MACH initialization of tcs struct */

/* Characteristics of node that can be deduced from
 * node's eeprom signature.
 */
struct tcs_node_descriptor {
	int	node_eeprom_signature;
	int	node_memory_size;
	int	node_vme_mastermapper_window;
	int	node_speed;
	int	node_code_cmmus;
	int	node_data_cmmus;
	int	node_has_cmapping_ram;
	int	node_bypasses_page_walks;
	int	node_is_prototype;
	int	node_16bit_sbus_timer;
	int	node_future;		/* spare for testing */
};

#define	VME_WINDOW_NONE	(-1)

/* routine declarations for exported routines */
long		tcs_get_time();
int		tcs_set_time();
void		tcs_shutdown();
void		tcs_reboot();
long		tcs_get_comp_state();
void		tcs_set_comp_state();
int		tcs_cold_boot_node();
int		tcs_check_intr();
#endif KERNEL

#endif

