/* $Header: pmap.doc.h,v 1.4 89/03/08 21:56:18 gibson Exp $ */
/*
 * pmap documentation
 */
/*
 *-Our page size is 8K (13 bits)
 * Our translation trees have two levels:
 *	The upper level ("page table map") contains 256 elements
 *  (8 bits of address), each mapping 16 Meg.
 *	Thus, the upper byte in a virtual address selects one of
 *  these elements, which points at a second level entry ("page map").
 *	Each page map contains 2K elements (11 bits of address).
 *  (Note that, since each element is 4 bytes, a page map takes one page.)
 *
 *-Each process has its own page table map.  Each process has its own page
 * maps for process-specific memory (user address space and u_area).
 * All processes on a node SHARE page maps for the kernel on that
 * node.
 *
 * User Address Space:		0x00000000	to	0xBFFFFFFF
 * Kernel Address Space:	0xC0000000	to	0xFFFFFFFF
 *
 *-Since each process has its own page table map mapping 16M per entry,
 * 16M is a convenient size region to take for various purposes.
 *
 *-The kernel address space has 64 of these 16M units, some of which (in
 * the (0xC0 and up range) are used for kernel text and data.  The following
 * kernel entries in the page table map are currently allocated.
 *
 * C0:		Kernel text
 * C1:		Kernel data
 *
 * FF:		PNC functions, swapped and unswapped multibus memory, etc.
 * FE:		Physical memory on the local node.
 * FD:		Map our page table map one level down.
 *		This is a neat trick that requires a little thought.
 *		    We make a copy of the page table map, using entries
 *		  appropriate for a page map.  We point the FD entry of
 *		  the page table map at this copy.  By doing this, we
 *		  make the page TABLE map (through the copy) double as
 *		  a page map.
 *		    Since the page table map points at page tables,
 *		  that means our page maps double as part of our address
 *		  space.  Thus, by writting to virtual address
 *		  0xFD000000, we modify the page table entry for page 0
 *		  of our process.  Using virtual addresses to modify the
 *		  page tables is much more convenient than walking the page
 *		  tree switching back and forth between physical and virtual
 *		  address.
 *		    Since the page table map requires only 2048 bytes
 *		  (1/4th of a page), we make the copy in the remaining
 *		  6K, immediately after the page table map, thereby
 *		  saving space.
 * FC:		This is used as a scratch version of the FD entry for
 *		modifying the page tables of OTHER processes.  We stick
 *		whatever is in that process's FD entry into our FC
 *		entry.
 * FB:		The process's kernel stack and u_area are in the top TWO
 *		pages of this 16M.  (u_area at 0xFBFFC000.  stack growing
 *		down from 0xFBFFFFFF.)
 * FA:		Scratch entry used by the master to map the various slaves'
 *		physical memory while they are being bootstrapped.
 * F9:		Maps the various processors' "vectorized pages".
 *		  - The kernel"data segment" (in the a.out(5) sense of the
 *		    word) is shared by all the kernels.  Many variables
 *		    are "vectorized" by turning them into arrays with
 *		    one entry per processor (e.g. node_list[NCPUS]) and
 *		    referencing a particular processor's version of the
 *		    variable via a processor number
 *		    (e.g. node_list[cpu_number()]).  Not only is
 *		    this a bit awkward, but, on the Butterfly, it loses
 *		    because these accesses are remote for N-1 of the
 *		    processors, and all processors beat on the same
 *		    memory (where the array is), possibly producing
 *		    contention.  Particularly in the case where most of the
 *		    accesses are to a processor's own copy of the variable,
 *		    we'd like that variable to be local.
 *
 *		    This effect is achieved via the "vectorized-variable
 *		    page" at virtual address 0xCxxxxxxx.  Each process gets
 *		    its own local physical memory for that virtual page,
 *		    and thus its own copy of the data in that page.  (The
 *		    exact implementation is to use the last page of the
 *		    kernel text segment, since each processor gets its
 *		    own copy of kernel text, and make the page writable.)
 *
 *		  - But by making it so easy to access our vectorized
 *		    variables, we've made it impossible to access another
 *		    processor's, since we don't have any virtual address
 *		    that map to it.  That's where F9 space comes in.
 *		    The vectorized page from each node is mapped in
 *		    at location 0xF9000000+(logical_node_number*PAGE_SIZE).
 *		    So node 1 has the same thing at 0xF9002000 as in
 *		    its vectorized page.  Any node can tweak the master's
 *		    vectorized variables at 0xF9000000.  If we had 256
 *		    nodes, 256*8K would fill 4 of our 16M.
 * F8:		Scratch entry used for translating virtual to physical
 *		addresses through a trick similar to the FD/FC approach.
 *		The F8 entry points at a page table.  We walk the page tree
 *		by stuffing different entries in the first entry of that
 *		page table.
 */


#ifdef BFLY2

/* CACHING ALGORITHMS */

	The Butterfly II, through the MC 88200, supports data caching.
The hardware has no support for automatic maintenance of cache
consistency, however, neccessitating software cache-consistency
algorithms.

	Software cache-consistency algorithms are rather fragile, and
these are no exception.  Carefully documenting the Mach-2000 approach
should make maintaining and modifying these algorithms easier and
safer.

Special cases:

	u_area, for several reasons (e.g. table call, 3 mappings)
	debuggers?
	table call in general?

Performance of hardware cache flushing?
