/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/drreg.h,v 1.4 90/03/05 09:00:34 tappan Exp $ */

#ifndef	_DRREG_H
#define	_DRREG_H

/*
 * File: drreg.h
 * Definitions for the Ikon 10089 buffered VMEbus DR11-W emulator.
 *
 * September 2, 1989 Thomas W. Uban
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced
 * Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights
 * to Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

#ifdef	KERNEL
/*
 * The Ikon DR11-W VME board lives in the standard I/O space
 * and has the following device registers...
 */
struct drdevice {		/* write/read (nop == no operation) */
    u_short dr_csr;		/* control/status register */
    u_short dr_data;		/* data out/in register */
    u_char dr_mod;		/* modifier register */
    u_char dr_vec;		/* vector register */
    u_short dr_pulse;		/* pulse/nop register */
    u_short dr_res1;		/* reserved register */
    u_short dr_res2;		/* reserved register */
    u_short dr_res3;		/* reserved register */
    u_short dr_res4;		/* reserved register */
    u_short dr_res5;		/* reserved register */
    u_short dr_dma_addrl;	/* DMA address low/nop register WR */
    u_short dr_dma_rangel;	/* DMA range low register */
    u_short dr_dma_addrl_rd;	/* nop/DMA address low register RD */
    u_short dr_res6;		/* reserved register */
    u_short dr_dma_addrh;	/* DMA address high/nop register WR */
    u_short dr_dma_rangeh;	/* DMA range high register */
    u_short dr_dma_addrh_rd;	/* nop/DMA address high register RD */
};

/*
 * Control register definitions (write only).
 */
#define	CTL_RDMA	0x8000	/* reset DMA EOR flag, INT, and BERR */
#define	CTL_RATN	0x4000	/* reset attention flag and INT */
#define	CTL_RPER	0x2000	/* reset parity error flag */
#define	CTL_MCLR	0x1000	/* master clear and pulse INIT H */
#define	CTL_CYCL	0x0100	/* force DMA cycle if enabled */
#define	CTL_TERM	0x0080	/* terminate DMA and set REDY */
#define	CTL_IENB	0x0040	/* enable/disable interrupts */
#define	CTL_SDIR	0x0010	/* software direction 1=in, 0=out */
#define	CTL_FCN3	0x0008	/* FCT3 H signal */
#define	CTL_FCN2	0x0004	/* FCT2 H signal */
#define	CTL_FCN1	0x0002	/* FCT1 H signal */
#define	CTL_GO		0x0001	/* pulse GO H signal and enable DMA */
#define	CTL_BITS	0x005e	/* non-pulsed control bit mask */

/*
 * Status register definitions (read only).
 */
#define	STS_DMAF	0x8000	/* DMA End-Of-Range flag */
#define	STS_ATTF	0x4000	/* attention flag */
#define	STS_ATTN	0x2000	/* state of ATTENTION H input */
#define	STS_PERR	0x1000	/* parity error flag */
#define	STS_STTA	0x0800	/* state of STATUS A input */
#define	STS_STTB	0x0400	/* state of STATUS B input */
#define	STS_STTC	0x0200	/* state of STATUS C input */
#define	STS_REDY	0x0080	/* interface ready indication */
#define	STS_IENB	0x0040	/* interrupt enable latch */
#define	STS_BERR	0x0020	/* bus error flag */
#define	STS_SDIR	0x0010	/* state of SDIR latch */
#define	STS_FCN3	0x0008	/* state of FCN3 latch */
#define	STS_FCN2	0x0004	/* state of FCN2 latch */
#define	STS_FCN1	0x0002	/* state of FCN1 latch */
#define	STS_DFLG	0x0001	/* device flag */

/*
 * Pulse register definitions (write only).
 */
#define	PUL_RDMA    CTL_RDMA	/* reset DMA EOR, INT, and BERR */
#define	PUL_RATN    CTL_RATN	/* reset attention flag and INT */
#define	PUL_RPER    CTL_RPER	/* reset parity error flag */
#define	PUL_MCLR    CTL_MCLR	/* master clear and pulse INIT H */
#define	PUL_CYCL    CTL_CYCL	/* force DMA cycle if enabled */
#define	PUL_TERM    CTL_TERM	/* terminate DMA and set REDY */
#define	PUL_SMSK    0x0040	/* set IENB */
#define	PUL_RMSK    0x0020	/* reset IENB */
#define	PUL_FCN2    CTL_FCN2	/* pulse ACLO FCT2 H signal */
#define	PUL_GO	    CTL_GO	/* pulse GO H signal and enable DMA */

/*
 * DR11W software status per interface.
 */
struct	dr_softc {
    short	xs_flags;	  /* flag information */
    short	xs_dmacnt;	  /* # of DMA blocks already xfer'd */
    int		xs_lnode;	  /* logical node where device lives */
    struct buf	*xs_rawbp;	  /* raw buffer pointer */
    caddr_t	xs_rawbuf;	  /* virtual addr of raw data buffer */
    u_long	xs_rawbuf_busaddr;/* bus address of raw data buffer */
    struct	mb_device *xs_md; /* pointer to device information */
    struct	drdevice *xs_dr;  /* vectorized drdevice pointer */
    int		xs_intlevel;	/* Interrupt priority level for device */
    simple_lock_data_t xs_lock;	  /* multiprocessor lock */
};

/*
 * xs_flags definitions.
 */
#define	DR_ALIVE	0x0001	/* device is alive */
#define	DR_OPEN		0x0002	/* someone has the device opened */
#define	DR_WAIT		0x0004	/* wait for reader on write */
#define	DR_WAITING	0x0008	/* someone is waiting for reader */

/*
 * Device delay definitions.
 */
#define	DR_ONE_SECOND	1000	/* DELAY one second value */
#define	DR_WAIT_SECONDS	10	/* wait up to ten seconds */

/*
 * General definitions.
 */
#define spldr() setpl(xs->xs_intlevel)
#define	DR_MOD		0x39	  /* std-non-priv-data addr modifier */
#define	DR_PRIO		(PZERO+1) /* interruptible sleep priority */
#define	DR_MINPHYS	4096	  /* maximum raw physio size */

#endif	KERNEL

/*
 * Ioctl packet command definitions.
 */
#define	DR_IOCTL_RESET	0	/* reset */
#define	DR_IOCTL_WAIT	1	/* wait for reader on write */
#define	DR_IOCTL_NOWAIT	2	/* don't wait ... (default) */

#endif	_DRREG_H
