/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/dtreg.h,v 1.2 90/03/05 09:01:50 tappan Exp $ */

#ifndef	_DTREG_H
#define	_DTREG_H

/*
 * File: dtreg.h
 * Definitions for the Data Translations DT1401 VMEbus 12-bit A/D Interface.
 *
 * October 15, 1989 Thomas W. Uban
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
 * Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

#ifdef	KERNEL
/*
 * The DT1401 board lives in the standard I/O space
 * and has the following device registers...
 */
struct dtdevice {
    u_short dt_csr;			/* control/status register (RW) */
    u_short dt_chan;			/* channel register (RW) */
    u_char dt_res0;			/* reserved */
    u_char dt_pclk;			/* pacer clock register (RW) */
    u_short dt_addata;			/* A/D data register (RO) */
    u_short dt_dadata0;			/* D/A 0 data register (WO) */
    u_short dt_dadata1;			/* D/A 1 data register (WO) */
    u_short dt_dio;			/* digital I/O data register (RW) */
    u_short dt_res1;			/* reserved */
};

/*
 * A/D control/status register definitions.
 */
#define	CSR_ERR			0x8000	/* error */
#define	CSR_ERR_INT_ENA		0x4000	/* error interrupt enable */
#define	CSR_ERR_CLR		0x2000	/* error clear */
#define	CSR_INIT_CLR		0x1000	/* initialize (clear) */
#define	CSR_SCAN_ENA		0x0400	/* scan enable */
#define	CSR_DO_1		0x0200	/* data out control, bit 1 */
#define	CSR_DO_0		0x0100	/* data out control, bit 0 */
#define	CSR_DONE		0x0080	/* A/D done */
#define	CSR_DONE_INT_ENA	0x0040	/* A/D done interrupt enable */
#define	CSR_EOS			0x0020	/* end of scan */
#define	CSR_EOS_INT_ENA		0x0010	/* end of scan interrupt enable */
#define	CSR_HARD_TRG_ENA	0x0008	/* hardware trigger enable */
#define	CSR_INC_ENA		0x0004	/* increment enable */
#define	CSR_TO			0x0002	/* timeout */
#define	CSR_START		0x0001	/* start */
#define	CSR_BITS		0x775d	/* writeable bits in the CSR */

/*
 * A/D channel register definitions.
 */
#define	CHAN_G_1		0x8000	/* gain control, bit 1 */
#define	CHAN_G_0		0x4000	/* gain control, bit 0 */
#define	CHAN_LC_4		0x1000	/* last channel, bit 4 */
#define	CHAN_LC_3		0x0800	/* last channel, bit 3 */
#define	CHAN_LC_2		0x0400	/* last channel, bit 2 */
#define	CHAN_LC_1		0x0200	/* last channel, bit 1 */
#define	CHAN_LC_0		0x0100	/* last channel, bit 0 */
#define	CHAN_SC_4		0x0010	/* start channel, bit 4 */
#define	CHAN_SC_3		0x0008	/* start channel, bit 3 */
#define	CHAN_SC_2		0x0004	/* start channel, bit 2 */
#define	CHAN_SC_1		0x0002	/* start channel, bit 1 */
#define	CHAN_SC_0		0x0001	/* start channel, bit 0 */

#define	GAIN_MASK		0xc000
#define	X_TO_GAIN(x)		(((x) << 14) & GAIN_MASK)
#define	LC_MASK			0x1f00
#define	X_TO_LC(x)		(((x) << 8) & LC_MASK)
#define	SHADOW_LC		((xs->xs_chan_shadow & LC_MASK) >> 8)
#define	SC_MASK			0x001f
#define	X_TO_SC(x)		((x) & SC_MASK)
#define	SHADOW_SC		(xs->xs_chan_shadow & SC_MASK)

/*
 * Pacer clock register definitions.
 */
#define	PCLK_CSE		0x0080	/* clock select enable */
#define	PCLK_RES		0x0040	/* reset */
#define	PCLK_CTL_5		0x0020	/* clock frequenct set, bit 5 */
#define	PCLK_CTL_4		0x0010	/* clock frequenct set, bit 4 */
#define	PCLK_CTL_3		0x0008	/* clock frequenct set, bit 3 */
#define	PCLK_CTL_2		0x0004	/* clock frequenct set, bit 2 */
#define	PCLK_CTL_1		0x0002	/* clock frequenct set, bit 1 */
#define	PCLK_CTL_0		0x0001	/* clock frequenct set, bit 0 */

#define	FREQ_MASK		0x003f
#define	X_TO_PCLK_FREQ(x)	((x) & FREQ_MASK)

#define	DT_CHANNELS		32

/*
 * DT1401 software status per interface.
 */
struct	dt_softc {
    short	xs_flags;		/* flag information */
    int		xs_lnode;		/* logical node where device lives */
    u_char	xs_pclk_shadow;		/* pacer clock register shadow */
    u_short	xs_chan_shadow;		/* channel register shadow */
    u_short	xs_current_chan;	/* channel currently being scanned */
    struct	mb_device *xs_md;	/* pointer to device information */
    struct	dtdevice *xs_dt;	/* vectorized drdevice pointer */
    simple_lock_data_t xs_lock;		/* multiprocessor lock */
    int		xs_intlevel;		/* device interrupt level */
    u_short	xs_errors;		/* mapped A/D conversion error count */
    u_short	xs_frame;		/* mapped A/D conversion scan count */
    u_short	xs_data[DT_CHANNELS];	/* mapped A/D conversion results */
};

/*
 * xs_flags definitions.
 */
#define	DT_ALIVE	0x0001		/* device is alive */
#define	DT_OPEN		0x0002		/* someone has the device opened */
#define	DT_INUSE	0x0004		/* someone is using the device */
#define	DT_WANTED	0x0008		/* someone wants to use the device */
#define	DT_SLEEPING	0x0010		/* sleeping until scan is complete */

/*
 * General definitions.
 */
#define	spldt()		setpl(xs->xs_intlevel)
#define	BUS_ADDR(x)	physoffset(x)
#define	DT_PRIO		(PZERO + 8)	/* interruptable sleep priority */
#define	SPIN_COUNT	400		/* spin loop iteration count ~100us */
#define	SPIN_TIMEOUT	50000		/* spin loop timeout count */

#endif	KERNEL

/*
 * All user interactions with the DT1401 are through the
 * ioctl interface. There is one ioctl (DTCMD) which takes
 * a dt_ioctl structure as its argument. The command which
 * the user places into the dt_ioctl structure is then
 * passed to the device by the driver. Users are restricted
 * to user mode commands.
 */
struct dt_ioctl {
    u_short cmd;	/* command */
    u_long arg0;	/* command argument 0 */
    u_long arg1;	/* command argument 1 */
    u_long arg2;	/* command argument 2 */
    u_long arg3;	/* command argument 3 */
};

/*
 * Ioctl packet command definitions.
 */
#define	DT_IOCTL_RESET		0	/* reset -- no args */
#define	DT_IOCTL_SET_GAIN	1	/* set gain to arg0 */
#define	DT_IOCTL_SET_LAST_CHAN	2	/* set last scan channel to arg0 */
#define	DT_IOCTL_SET_START_CHAN	3	/* set start scan channel to arg0 */
#define	DT_IOCTL_SET_PCLK_CSE	4	/* set pacer clk select enable */
#define	DT_IOCTL_RESET_PCLK_CSE	5	/* reset pacer clk select enable */
#define	DT_IOCTL_ENABLE_PCLK	6	/* enable pacer clk output */
#define	DT_IOCTL_DISABLE_PCLK	7	/* disable pacer clk output */
#define	DT_IOCTL_SET_PCLK_FREQ	8	/* set pacer clk frequency to arg0 */
#define	DT_IOCTL_READ_AD_DATA	9	/* read A/D data register into *arg0 */
#define	DT_IOCTL_MAPIN		10	/* ret addr of mapped buf in *arg0 */
#define	DT_IOCTL_SCAN		11	/* start A/D conversion scan */
#define	DT_IOCTL_SLEEP		12	/* sleep until next scan complete */

/*
 * The DT_IOCTL_MAPIN and DT_IOCTL_SCAN pair map a kernel data buffer
 * into the process address space. The data buffer consists of the
 * following:
 *	u_short errors;		|* A/D conversion error count *|
 *	u_short frame;		|* A/D conversion scan count *|
 *	u_short chan_data[32];	|* A/D conversion data results *|
 *
 * After scanning is started, chan_data is updated automatically at the
 * frequency specified by DR_IOCTL_SET_PCLK_FREQ for channels 0 through
 * the last channel specified by DR_IOCTL_SET_LAST_CHAN.
 *
 * The DT_IOCTL_SLEEP cmd puts the process to sleep until the next scan
 * completes.
 */

/*
 * Pacer clock frequency definitions.
 */
					/* freq divisor, selected freq */
#define	PCLK_600kHz		0x00	/* 1           , 600kHz */
#define	PCLK_60kHz		0x04	/* 10          , 60kHz */
#define	PCLK_6kHz		0x24	/* 100         , 6kHz */
#define	PCLK_600Hz		0x14	/* 1000        , 600Hz */
#define	PCLK_60Hz		0x34	/* 10,000      , 60Hz */
#define	PCLK_6Hz		0x0c	/* 100,000     , 6Hz */
#define	PCLK_x6Hz		0x2c	/* 1,000,000   , .6Hz */
#define	PCLK_x06Hz		0x1c	/* 10,000,000  , .06Hz */
#define	PCLK_x006Hz		0x3c	/* 100,000,000 , .006Hz */

#define	PCLK_300kHz		0x02	/* 2           , 300kHz */
#define	PCLK_30kHz		0x22	/* 20          , 30kHz */
#define	PCLK_3kHz		0x12	/* 200         , 3kHz */
#define	PCLK_300Hz		0x32	/* 2000        , 300Hz */
#define	PCLK_30Hz		0x0a	/* 20,000      , 30Hz */
#define	PCLK_3Hz		0x2a	/* 200,000     , 3Hz */
#define	PCLK_x3Hz		0x1a	/* 2,000,000   , .3Hz */
#define	PCLK_x03Hz		0x3a	/* 20,000,000  , .03Hz */

#define	PCLK_200kHz		0x06	/* 3           , 200kHz */
#define	PCLK_20kHz		0x26	/* 30          , 20kHz */
#define	PCLK_2kHz		0x16	/* 300         , 2kHz */
#define	PCLK_200Hz		0x36	/* 3000        , 200Hz */
#define	PCLK_20Hz		0x0e	/* 30,000      , 20Hz */
#define	PCLK_2Hz		0x2e	/* 300,000     , 2Hz */
#define	PCLK_x2Hz		0x1e	/* 3,000,000   , .2Hz */
#define	PCLK_x02Hz		0x3e	/* 30,000,000  , .02Hz */

#define	PCLK_150kHz		0x01	/* 4           , 150kHz */
#define	PCLK_15kHz		0x21	/* 40          , 15kHz */
#define	PCLK_1x5kHz		0x11	/* 400         , 1.5kHz */
#define	PCLK_150Hz		0x31	/* 4000        , 150Hz */
#define	PCLK_15Hz		0x09	/* 40,000      , 15Hz */
#define	PCLK_1x5Hz		0x29	/* 400,000     , 1.5Hz */
#define	PCLK_x15Hz		0x19	/* 4,000,000   , .15Hz */
#define	PCLK_x015Hz		0x39	/* 40,000,000  , .015Hz */

#define	PCLK_120kHz		0x05	/* 5           , 120kHz */
#define	PCLK_12kHz		0x25	/* 50          , 12kHz */
#define	PCLK_1x2kHz		0x15	/* 500         , 1.2kHz */
#define	PCLK_120Hz		0x35	/* 5000        , 120Hz */
#define	PCLK_12Hz		0x0d	/* 50,000      , 12Hz */
#define	PCLK_1x2Hz		0x2d	/* 500,000     , 1.2Hz */
#define	PCLK_x12Hz		0x1d	/* 5,000,000   , .12Hz */
#define	PCLK_x012Hz		0x3d	/* 50,000,000  , .012Hz */

#define	PCLK_100kHz		0x03	/* 6           , 100kHz */
#define	PCLK_10kHz		0x23	/* 60          , 10kHz */
#define	PCLK_1kHz		0x13	/* 600         , 1kHz */
#define	PCLK_100Hz		0x33	/* 6000        , 100Hz */
#define	PCLK_10Hz		0x0b	/* 60,000      , 10Hz */
#define	PCLK_1Hz		0x2b	/* 600,000     , 1Hz */
#define	PCLK_x1Hz		0x1b	/* 6,000,000   , .1Hz */
#define	PCLK_x01Hz		0x3b	/* 60,000,000  , .01Hz */

#define	PCLK_50kHz		0x07	/* 12          , 50kHz */
#define	PCLK_5kHz		0x27	/* 120         , 5kHz */
#define	PCLK_500Hz		0x17	/* 1200        , 500Hz */
#define	PCLK_50Hz		0x37	/* 12,000      , 50Hz */
#define	PCLK_5Hz		0x0f	/* 120,000     , 5Hz */
#define	PCLK_x5Hz		0x2f	/* 1,200,000   , .5Hz */
#define	PCLK_x05Hz		0x1f	/* 12,000,000  , .05Hz */
#define	PCLK_x005Hz		0x3f	/* 120,000,000 , .005Hz */

#endif	_DTREG_H
