/* $Header: /nfs/columbia/u1/BFMACH.local/mk.gibson/butterflydev/RCS/hty.h,v 1.22 91/02/13 12:30:06 gibson Exp $ */
/*****************************************************************/
/* hty.c							*/
/*****************************************************************/
/* PART NUMBER: 90-070048-8					*/
/*****************************************************************/
/*  Copyright (c) 1986 by Systech Corporation			*/
/*								*/
/*  CONFIDENTIAL.  THIS SOFTWARE IS LICENSED BY AND IS THE	*/
/*  CONFIDENTIAL AND PROPRIETARY PROPERTY OF SYSTECH COMPUTER	*/
/*  CORPORATION AND MAY NOT, WITHOUT THE PRIOR WRITTEN CONSENT	*/
/*  OF SYSTECH COMPUTER CORPORATION, BE REPRODUCED, USED OR	*/
/*  DISCLOSED EXCEPT IN ACCORDANCE WITH A SYSTECH COMPUTER	*/
/*  CORPORATION SOFTWARE LICENSE AGREEMENT.			*/
/*****************************************************************/
/*								*/
/* MODIFICATION HISTORY:					*/
/* Rel.	--Date--	Who	Description			*/
/* ----	--------	---	-----------			*/
/* ddr	mm/dd/yy	xxx	Initial release			*/
/*	03/16/87	CHB	Set priority level for sun3    */
/*	03/18/87	CHB	Added variable for new mb mapping  */
/*	03/27/87	chb	Using stored config value for priority level */
/*	05/06/87	chb	Add declaration of sleep queue pointers */
/*	05/26/87	chb	Modify for different size configs*/
/*	05/29/87	chb	Sleep queue pointers moved to htyc		*/
/*	08/18/87	chb	Add a total channel count 	*/
/*	11/10/87	DGA	Added support for 38.4K baud in the 	*/
/*				"hty_speeds" data structure.	*/
/*	07/27/88	LCW	Added groupd cmds to ioctl.c        	*/
/*				See ioctl.c for more info.  Unlike the  */
/*				systemv tty driver, there is not group.c*/
/*	07/29/88	LCW	Implement big write buffer scheme.  Also*/
/*				modify READBUF define so that it uses   */
/*				all the data space available.           */
/*	08/22/88	LCW	Attempt to implement reset of the hps   */
/*				while in multi-user mode. add 1 def.    */
/*	11/01/88	LCW	Changed referneces of NHPSCTRL to NRHP  */
/*				which is in the file rhp.h.  rhp.h is 	*/
/*				created from the 'config' program when 	*/
/*				configuring the kernel.		*/
/*	11/02/88	WLW	Updated copyright statement.	*/
/*				COSMETIC			*/
/*	01/26/89	DGA	Corrected copyright statement.	*/
/*				COSMETIC			*/

/*****************************************************************************
* GP1000 documentation (jcg): This driver is different from a standard 4.3
* terminal device driver in one important respect.  
* The input/output editing normally done on characters in
* rawq and outq is done for us in the cluster controller.  That's a big
* selling point for the product -- taking load off the main computer.
* 
* Unfortunately, we still want to glue our driver into the device-independent
* part of 4.3 terminal manipulation: the tty driver/line discipline stuff.
* That provides a lot of handy functionality, e.g. job control, as well
* as some stuff we don't want, e.g. input/output editing.
*
* This driver's approach is to bag clists, do all our own buffering, and
* call the line discipline routines in such a way that they are "faked out",
* and do only the parts we want.  That's because they must interface
* with an externally provided tty.c (from Sun).
*
* We have a slightly easier job.  We can go modify the tty driver with an
* extra flag meaning "don't do anything with clists: let the device do
* its own buffering".
*
* Of course, we might get into trouble with line disciplines other than
* OLD and NEW, so we'll still, e.g.,  call the linesw write routine with an
* argument of 0 bytes to fake it out.  That way, we won't be any worse than
* the unmodified SYSTECH driver.  If we don't work with those new line
* disciplines, we'll go fix it.
******************************************************************************/

/*****************************************************************/
/*     NOTES ON CURRENT STATE OF SOFTWARE 8 May 1987		*/
/*     The list of uncompleted features given below		*/
/* is not necessarily complete.  				*/
/*								*/
/*  (1)  Various unimplemented entry points:			*/
/*       (a) stop - This entry point is called by the ioctl     */
/*           tty subroutine.  Its purpose is to stop output	*/
/*           during an ioctl flush call. This is not required	*/
/*           with our product since we don't use the clist data	*/
/*           structures.					*/
/*		
/*	     
/*       (b) reset 						*/
/*       (c) select - a version of the select entry point is 	*/
/*			included, but is incomplete and not tested */
/*  (2) fcntl calls: see FCINT(5), FCNTL(2), TTY(2)and OPEN(2)	*/
/*      Also look at all the ioctl call constants in ioctl.h	*/
/*      which have the letter "f" associated with them.		*/
/*      There are several constants: FIOCLEX, FIONCLEX,		*/
/*      FIONREAD, FIONBIO, FIONASYNC, FIOSETOWN, FIOGETOWN.	*/
/*      The only one now working is FIONREAD, that's		*/
/*      because it's well documented in the TTY driver.		*/
/*      FIONBIO and FIONASYNC are mentioned in almost all the	*/
/*      the references listed above.  There is code in		*/
/*      the read.c file to support them. 			*/
/*								*/
/*  (3) Job access control: see TTY(4), vfork(2)		*/
/*      We used the linesw subroutines for ioctl,		*/
/*      open, close and write operations.			*/
/*         When a write operation was to be performed, we called*/
/*      the linesw routine write code with a residual I/O count	*/
/*      of 0 (zero).  This forced the ttwrite to do all the	*/
/*      signal processing without actually doing the write.	*/
/*      ( This would usually call the start routine declared in	*/
/*      the tty data structure t_oproc.)			*/
/*         All of this works fine. Upon investigating the signal*/
/*		processing required for doing a read, we found that 	*/
/*		we couldn't fool				*/
/*      it like we had fooled the ttwrite routine. One could try*/
/*      to call the linesw read routine with carrier off and the*/
/*      state set to non-blocking I/O. This might fool it into 	*/
/*      doing all the read signal processing then returning	*/
/*      without attempting to do read processing.		*/
/*****************************************************************/

#include	"hps.h"

#include "../h/param.h"
#include "../h/conf.h"
#include "../h/errno.h"
#include "../h/ioctl.h"
#include "../h/signal.h"
#include "../h/user.h"
#include "../h/tty.h"
#include "../h/file.h"
#include "../h/buf.h"
#include "../h/uio.h"
#include "../h/proc.h"
#include "../h/callout.h"

#include "../machine/psl.h"
#include "../butterflydev/mbvar.h"

#include "../butterflydev/hps_include/hps_debug.h"
#include "../butterflydev/hps_include/htytty.h"
#include "../butterflydev/hps_include/hps.types.h"
#include "../butterflydev/hps_include/hps.iocb.h"
#include "../butterflydev/hps_include/hps.ioctl.h"
#include "../butterflydev/hps_include/hps.if.h"
#include "../butterflydev/hps_include/hps.onb.h"
#include "../butterflydev/hps_include/hps.net.h"
#include "../butterflydev/hps_include/hps.asynch.h"
#include "../butterflydev/hps_include/hpsdrconf.h"
#include "../butterflydev/hps_include/hpsctlr.h"
#include "../butterflydev/hps_include/htyctlr.h"

/*
 *	COMMON SERVICE ROUTINES 
 */

int hty_intr();
int hty_reset();
int hty_start();

/*
 *	DRIVER CONSTANTS
 */

	/* if told to by configuration file, look at modem signals */ 
#define USE_MODEM_SIGS	1

/* Wractive flag values:
 *	The only tricky part of the write protocol is that a request to
 *	send a small number of bytes is delayed in the hope that other
 *	write requests will be made, allowing us to compact multiple
 *	requests into a single IOCB.  When other requests come in or
 *	a timer goes off, the data is sent to the controller.
 */

#define WRBUF_EMPTY	0x00	/* Write buffer is empty */
#define WRBUF_FILLING	0x01	/* Filling up buffer (buf owned by top-half process, with interrupt enabled) */
#define WRBUF_PARTFULL	0x02	/* Buffer filled partway, not owned, not sent */
#define WRBUF_ONBOARD	0x04	/* Buffer sent to board */

	/* Filling (owned by process) or on board.  Must wait to fill */
#define WRBUF_BUSY_MASK	(WRBUF_FILLING|WRBUF_ONBOARD)

/* Rdactive flag values */

#define	RD_NONE		0x00	/* No read's pending */
#define RD_BUFONBOARD	0x01	/* MultRead request sent to board */
#define RD_PENDING	0x02	/* Reissue MultRead when current one finished*/

/* Sleeps flag bits.  */
#define BUF_AVAIL	0x01
#define MOSIG		0x02

/* flags bits */
#define WAITING		0x0001
#define	HTC_QUICK_INUSE	0x0002

/* Constants for controlling input flushing */

#define FLSHTRP		0x01
#define FLSHCMD		0x02

/* Constants for allocating shared memory */

#ifdef HPS_NEWTCS

#define WRITEBUF 	MW_BUFSIZE
#define READBUF 	(sizeof(IOBUF)-sizeof(A_IOCB))

#else HPS_NEWTCS

/* pre Rev 3A TCS: use 512, instead of big read/write buffers */
#define	WRITEBUF	512
#define	READBUF		512

#endif HPS_NEWTCS

/*
 *	DRIVER MACROS
 */

/* The following macros set, clear, or test bits in a byte array */
/* The first argument is the channel #, 2nd is the byte array    */
/* b_ar_index is determined by what swapping is required between
 * main processor and Systech card.
 */
#define   BYTE_MASK(ch)	(1 << (ch))
#define   hpsio_set_bit(ch, b_ar) (b_ar[b_ar_index(ch)] |= BYTE_MASK((ch)&0x7))
#define   hpsio_clr_bit(ch, b_ar) (b_ar[b_ar_index(ch)] &=~BYTE_MASK((ch)&0x7))
#define   hpsio_test_bit(ch, b_ar) (b_ar[b_ar_index(ch)] & BYTE_MASK((ch)&0x7))

/* This macro returns a block of bits corresponding to 8 channels */
#define	  hpsio_read_block8(ch, b_ar) (b_ar[b_ar_index(ch)])

#define   splHTY	spltty

/* 
 * EXTERNAL VARIABLES
 */

/* The following are defined in the hty_config file */
extern int hps_debug;
extern int nhps;		/* maximum number of adaptors */
extern int numextra;		/* number of IOCB buffers for ioctl's, etc. */
extern short  *min2hpsp[];	/* translate HPS/min dev to channel */

/* The following is defined in hps.c */
extern struct hpsctlr *hpsctlr[];/* HPS (raw) control structures */

/* The following are defined in hty_config.c */
extern struct htyctlr *htyctlr[];	/* hty control structures */
extern struct tty *h_tty[];	/* tty structures - one per channel */
extern struct hty *h_hty[];	/* Systech-specific "struct tty"-equivalents */

		
/* Find the address of the indicated [ht]ty by walking the vec page */
#define	h_tty_addr(lnode,adp,adp_min)	\
	(&(*((struct tty **)vvrmtptr(&h_tty[(adp)],(lnode))))[(adp_min)])
#define	h_hty_addr(lnode, adp, adp_min)	\
	(&(*((struct hty **)vvrmtptr(&h_hty[(adp)],(lnode))))[(adp_min)])


/* hps_get() gets a buffer from the raw driver's pool.  
 * The hty driver gets them in init and never returns them.
 */
IOBUF	*hps_get();

/* hps_alloc() allocates memory local to the hps adaptor */
caddr_t	hps_alloc();

/***********************/
/*   GLOBAL DATA AREA  */
/***********************/ 

extern char hty_speeds[];

IOBUF	*get_iocb();

#define	MIN_HPSADPSHIFT		8
#define	MIN_HPSHWRDSHIFT	7
#define	MIN_HPSMINSHIFT		0

#define	MIN_HPSADPMASK		0x07	/* 3 bits */
#define	MIN_HPSHWRDMASK		0x01	/* 1 bits */
#define	MIN_HPSMINMASK		0x7F	/* 7 bits */

#define	hpsadaptor(minor)	((minor >> MIN_HPSADPSHIFT) & MIN_HPSADPMASK)
#define	hpsminor(minor)		((minor >> MIN_HPSMINSHIFT) & MIN_HPSMINMASK)
#define	hpshwired(minor)	((minor >> MIN_HPSHWRDSHIFT) & MIN_HPSHWRDMASK)

#define	hpsmakemin(adp,adpmin)	(((adp) << MIN_HPSADPSHIFT) | ((adpmin) << MIN_HPSMINSHIFT))

