/* $Header: /nfs/gemini/u1/BFMACH.local/mk.gibson/butterflydev/RCS/hty_config.c,v 1.11 90/04/03 12:21:37 gibson Exp $ */
/*****************************************************************/
/* hty_config.c							*/
/*****************************************************************/
/* PART NUMBER: 90-070048-8					*/
/*****************************************************************/
/*  Copyright (c) 1986 by Systech Corporation			*/
/*								*/
/*  CONFIDENTIAL.  THIS SOFTWARE IS LICENSED BY AND IS THE	*/
/*  CONFIDENTIAL AND PROPRIETARY PROPERTY OF SYSTECH COMPUTER	*/
/*  CORPORATION AND MAY NOT, WITHOUT THE PRIOR WRITTEN CONSENT	*/
/*  OF SYSTECH COMPUTER CORPORATION, BE REPRODUCED, USED OR	*/
/*  DISCLOSED EXCEPT IN ACCORDANCE WITH A SYSTECH COMPUTER	*/
/*  CORPORATION SOFTWARE LICENSE AGREEMENT.			*/
/*****************************************************************/
/*								*/
/* MODIFICATION HISTORY:					*/
/* Rel.	--Date--	Who	Description			*/
/* ----	--------	---	-----------			*/
/* ddr	mm/dd/yy	xxx	Initial release			*/
/*								*/
/*****************************************************************/
/*   This file contains configuration information for the SUN	*/
/* hty driver.  This file is compiled separately and linked	*/
/* with the driver object and the kernel.  The reason for this	*/
/* is to simplify the changes required to reconfigure the	*/
/* driver and to make it unnecessary to distribute driver	*/
/* source.							*/
/*****************************************************************/

#include	"../butterflydev/hty.h"

/* Global variables used by the hty driver */
int	numextra = NUMEXTRA;

#ifndef VECTORIZED
#define	VECTORIZED	extern
#endif VECTORIZED

/* HTY control structures, one for each HPS host adapter */
VECTORIZED struct htyctlr *htyctlr[NHPS];

/* pointers to tty/hty structures (one per channel for each host adaptor) */
VECTORIZED struct tty *h_tty[NHPS];
VECTORIZED struct hty *h_hty[NHPS];

/*****************************************************************************
* Mapping from dev_t to hardware:
*
*	The dev_t for an hty device consists of major,iobus, and minor.  The
* minor is broken up into 1 unused bit, 3 bits of adaptor, and 8 bits of 
* "adaptor-minor".  The adaptor/adaptor-minor are used to index into the
* per-adaptor tty/hty arrays to get the tty/hty for that device.
*
*	The adaptor/adaptor-minor is mapped via min2hpsp/hps2minp to
* a Systech "channel number" for that adaptor.  The channel number is
* used when giving commands to the Systech.  At configuration time,
* the AIOC_SETC (set configuation) command is used to establish a
* correspondence between channel number and cluster controller/port.
*
*	The result is that, on a system call, the minor gives us the adaptor
* and (via min2hpsp) the channel.  At interrupt time, the adaptor and
* channel can be used (via hps2minp) to get the adaptor minor and hence
* the tty/hty structures.
*
*	The mapping done by min2hpsp/hps2minp is trivial at the moment.
* Channel number always equals adaptor-minor (the low 8 bits of the 
* Unix minor).  The AIOC_SETC command, directed by /etc/hps.conf, enables
* a system administrator to set up correspondence between dev_t and 
* cluster-controller/port.
******************************************************************************/

/* The following two-dimensional table maps a Unix minor
 * (adaptor/adaptor-minor) to a Systech channel number.  At the moment,
 * the channel number always equals the adaptor minor, and all entries in
 * min2hpsp point to min2hps_default.
 */
short	*min2hpsp[NHPS];

/* min2hps_default does the default mapping: adaptor-minor=channel */
short min2hps_default[NHTYLINES] =
{
	00, 01, 02, 03, 04, 05, 06, 07,
	08, 09, 10, 11, 12, 13, 14, 15,
	16, 17, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39,
	40, 41, 42, 43, 44, 45, 46, 47,
	48, 49, 50, 51, 52, 53, 54, 55,
	56, 57, 58, 59, 60, 61, 62, 63,
};

/* 	The following default table maps an HPS channel number to the
 * adaptor-minor portion of the Unix minor device number.  Since, at the
 * moment, the channel always equals the adaptor-minor, all htyctlr structs
 * point at this default table.
 * 	The hps2min table must implement the reverse mapping of the
 * min2hps table.
 */

short hps2min_default[NHTYLINES] =
{
	00, 01, 02, 03, 04, 05, 06, 07,
	08, 09, 10, 11, 12, 13, 14, 15,
	16, 17, 18, 19, 20, 21, 22, 23,
	24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39,
	40, 41, 42, 43, 44, 45, 46, 47,
	48, 49, 50, 51, 52, 53, 54, 55,
	56, 57, 58, 59, 60, 61, 62, 63
};

/* Map indices to constants representing different baud rates. */
char hty_speeds[] = {
	0, 0, A_75, A_110, A_134, A_150, 0, A_300,
	A_600, A_1200, 0, A_2400, A_4800, A_9600, A_192K,
#if HPS_NEWTCS
	/* Newer cluster controllers support 38400 */
	A_384K,
#else HPS_NEWTCS
	/* Older (BFLY1) cluster controllers do not support 38400 */
	0
#endif HPS_NEWTCS
};
