/* $Header: /nfs/columbia/u1/BFMACH.local/mk.2.9.6/butterflydev/RCS/if_exreg.h,v 1.28 91/03/19 07:46:54 tappan Exp $ */

#ifndef	_IF_EXREG_H
#define	_IF_EXREG_H

/*
 * File: if_exreg.h
 * Definitions for the EXOS 301/302 Inteligent Ethernet Controllers
 *
 * April 11, 1989 Thomas W. Uban
 *
 * (Parts barrowed from previous drivers from Excelan, BB&N, etc.)
 *
 * Copyright 1989 by BBN Advanced Computers Inc.
 * ALL RIGHTS RESERVED
 *
 * No part of this material may be reproduced, stored in a retrieval
 * system, transmitted or used in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise, without the prior
 * written permission of, or express license from BBN Advanced Computers Inc.
 *
 * RESTRICTED RIGHTS LEGEND
 *
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
 * Technical Data and Computer Software clause at DFARS 52.227-7013.
 *
 * BBN Advanced Computers Inc.
 * 10 Fawcett Street
 * Cambridge, Ma. 02238
 * 617-873-6000
 */

/*
 * Device spin/delay definitions.
 */
#define	SPIN_COUNT	300		/* spin loop iteration count */
#define	ONE_SECOND	1000		/* DELAY one second value */
#define	WAIT_SECONDS	10		/* wait up to ten seconds */
#define HANG_DELAY      1		/* DELAY between port B checks */
#define HANG_TIMEOUT (500/HANG_DELAY)	/* 500ms in delay ticks*/

/*
 * The VME (302) board lives in the standard I/O space
 * and has the following device registers...
 */
struct exdevice302 {
    u_char pad0;
    u_char ex_portA;	/* read for status1, write to reset */
    u_char pad1;
    u_char ex_portB;	/* read for status0, write command word */
    u_char pad2;
    u_char ex_portC;	/* write (only) VME address modifier */
};

/*
 * The Multibus (301) board has the following device registers...
 */
struct exdevice301 {
    u_char ex_portA;	/* read to reset, write to ack interrupt */
    u_char ex_portB;	/* read for status, write command word */
};

/*
 * device register operations
 */
#define	EX_INTACK	1		/* acknowledge EXOS interrupt */
#define	EX_RESET	1		/* reset EXOS */
#define	EX_INTERRUPT	1		/* interrupt EXOS */
#define	EX_TESTOK	0x01		/* fatal error when 0 */
#define	EX_INTPENDING	0x02		/* EXOS has interrupted when 1 (301) */
#define	EX_BUSY		0x08		/* EXOS is ready when 0 */

/*
 * Run the 302 in Master A32 and Slave A24.
 */
#define	EX_ADDRMOD_STD	0x39		/* std-non-priv-data-access */
#define	EX_ADDRMOD_EXT	0x09		/* ext-non-priv-data-access */

/*
 * The generic device structure definition.
 */
struct exdevice {
    union {
	struct exdevice301 ex301;
	struct exdevice302 ex302;
    } u;
};

/*
 * Wait until port B is ready to accept input, and then place val into it.
 * Use the correct device port offsets based on the type of controller.
 */

#define	EX_PORTB(xs, val) { \
    register u_char *portBp = xs->xs_portB; \
    register int loops; \
    for(loops = 0; (*portBp & EX_BUSY) != 0 && loops < HANG_TIMEOUT; ++loops) \
	DELAY(HANG_DELAY); \
    if (loops >= HANG_TIMEOUT) \
	printf("%s: port B hung\n", xs->xs_if.if_ifname); \
    *portBp = (u_char)(val); \
}

#define EX_POKE(xs) { \
    if (xs->xs_poke_needed) \
	EX_PORTB(xs, EX_INTERRUPT) \
    xs->xs_poke_needed = 0; \
}

#define EX_LOCK(xs) simple_lock(&(xs)->xs_lock)
#define EX_UNLOCK(xs) simple_unlock(&(xs)->xs_lock)

/*
 * Wait for EXOS to return a message. In the case of the 301,
 * acklnowledge the interrupt. Use the correct device port
 * offsets based on the type of controller.
 */
#define	EX_WAIT(xs, mp) { \
    if ((xs)->xs_flags & EX_302) { \
	while (((mp)->mh_status & EXMQ_OWNER) == EXMQ_EXOS) { \
	    int x; \
	    for (x = 0; x < SPIN_COUNT; x++) \
		continue; \
	    continue; \
	} \
    } else { \
	u_char *portAp; \
	register u_char *portBp; \
	portAp = &((xs)->xs_ex->u.ex301.ex_portA); \
	portBp = &((xs)->xs_ex->u.ex301.ex_portB); \
	while ((*portBp & EX_INTPENDING) == 0) \
	    DELAY(ONE_SECOND); \
	*portAp = EX_INTACK; \
    } \
}

#ifdef BFLY1
#define READ_LONG(var) (var)
#define ASSIGN_LONG(var, value) ((var) = (value))
#define	BUS_ADDR(x)	MBI_ADDR(x)
#endif	BFLY2

#ifdef	BFLY2
/*
 * On the 88k, longword access must be aligned. These macros allow
 * non aligned access by breaking the longword into two shorts.
 */
#define READ_LONG(var) (((var)[0] << 16) | ((u_short)(var)[1]))
#define ASSIGN_LONG(var, value) { \
    (var)[0] = (value) >> 16; \
    (var)[1] = (short)(value); \
}
#define	BUS_ADDR(x)	physoffset(x)
#endif BFLY2

/*
 * Structure used for initialization only.
 * (some of the dummy entries are due to byte swapping)
 */
struct init_msg {
    short	im_reserved;		/* must be 1 */
    char	im_version[4];		/* version of the hardware */
    u_char	im_result;		/* completion code */
    u_char	im_mode;		/* set to link mode (0) */
    char	im_hdfo[2];		/* HOST data format option */
    char	im_junk[3];
    u_char	im_haddrmode;		/* HOST address mode */
    char	im_dummy2;
    u_char	im_mmsize;		/* memory map size (returned) */
    char	im_byteptn[4];		/* data order byte pattern */ 
    short	im_wordptn[2];		/* data order word pattern */
#ifdef	BFLY2
    short	im_longptn[2];		/* data order long pattern */
#else	BFLY2
    long	im_longptn;		/* data order long pattern */
#endif	BFLY2
    u_char	im_mmap[20];		/* (rest of) memory map (returned) */
    u_short	im_off;			/* movable block offset */
    u_short	im_seg;			/* movable block segment */
    u_char	im_nproc;		/* number of EXOS processes */
    u_char	im_nmb;			/* number of EXOS mailboxes */
    u_char	im_nslots;		/* number of address slots */
    u_char	im_nhosts;		/* number of hosts == 1 */
#ifdef	BFLY2
    short	im_h2xqaddr[2];		/* HOST->EXOS msg q address */
#else	BFLY2
    long	im_h2xqaddr;		/* HOST->EXOS msg q address */
#endif	BFLY2
    short	im_h2xoff;		/* offset from base of actual q */
    u_char	im_h2xtype;		/* interrupt type for h2x msg q */
    u_char	im_h2xvalue;		/* interrupt value for h2x msgs */
#ifdef	BFLY2
    short	im_h2xaddr[2];		/* interupt address for h2x msg q */
#else	BFLY2
    long	im_h2xaddr;		/* interupt address for h2x msg q */
#endif	BFLY2
#ifdef	BFLY2
    short	im_x2hqaddr[2];		/* EXOS->HOST msg q address */
#else	BFLY2
    long	im_x2hqaddr;		/* EXOS->HOST msg q address */
#endif	BFLY2
    short	im_x2hoff;		/* offset from base of actual q */
    u_char	im_x2htype;		/* interrupt type for x2h msg q */
    u_char	im_x2hvalue;		/* interupt value for x2h msgs */
    short	im_x2haddr[2];		/* interupt address for x2h msg q */
};

/*
 * im_mode values
 */
#define	EXOS_LINKMODE		0
#define	EXOS_HOSTLOAD		1
#define	EXOS_NETLOAD		2

/*
 * im_haddrmode values
 */
#define	EXOS_ABSOLUTE		3

/*
 * im_x2htype values
 */
#define	EXOS_NOINTR		0
#define	EXOS_IOMAPPED		1	/* 301 only */
#define	EXOS_MEMORYMAPPED	2
#define	EXOS_LEVEL		3	/* 301 only */
#define	EXOS_BUSVEC		4	/* 302 only */

typedef unsigned long	mbaddr_t;
#define	NFRAGS		8		/* up to 8 DMA fragments */

/*
 * Data block structure for transmit/receive.
 */
struct	exdata {
    u_char	nm_aslot;		/* address slot for this packet */
    u_char	n_frags;		/* number of data block fragments */
    struct	frag {			/* DMA fragment length/address pairs */
	u_short	frag_length;
#ifdef	BFLY2
	u_short	frag_address[2];
#else	BFLY2
	u_long	frag_address;
#endif	BFLY2
    } frag[NFRAGS];
};

struct net_mode {
    u_char nm_reqmask;			/* read/write mode request */
    u_char nm_optmask;			/* options */
    u_char nm_mode;			/* mode of controller */
};

/*
 * nm_optmask values
 */
#define	EX_OPT_RCVBADALGN	0x10	/* receive aligmnent error packets */
#define	EX_OPT_RCVBADCRC	0x20	/* receive bad CRC packets */
#define	EX_OPT_NETDISABLE	0x80	/* disable controller from Ethernet */

/*
 * nm_mode values
 */	
#define	EX_MODE_DISCONNECT	0x00	/* disconnect controller from Ether */
#define	EX_MODE_PERFECT		0x01	/* connect to net, perfect filter */
#define	EX_MODE_IMPERFECT	0x02	/* connect to net, imperfect filter */
#define	EX_MODE_PROMISCUOUS	0x03	/* connect to net, receive all pkts */

struct net_addrs {
    u_char nm_reqmask;			/* read/write address request */
    u_char nm_addrslot;			/* address slot */
    u_char nm_netaddr[6];		/* network address */
};

struct net_recv {
    u_char nm_reqmask;			/* request flag */
    u_char nm_addrslot;			/* address slot */
};

/*
 * nm_addrslot values (multicast are 1 .. 252)
 */
#define	EX_BROADCAST_SLOT	255
#define	EX_MULTICAST_SLOT	254
#define	EX_PHYSICAL_SLOT	253
#define	EX_NOMATCH_SLOT		  0

struct net_ststcs {
    u_char	nm_reqmask;		/* read/write statistics request */
    u_char	nm_statzero;		/* must be zero */
    u_short	nm_numobj;		/* # of statistics objects wanted */
    u_short	nm_objindex;		/* starting index for objects */
#ifdef	BFLY2
    u_short	nm_statbuf[2];		/* buffer address for statistics */
#else	BFLY2
#ifdef	BFLY1
    mbaddr_t	nm_statbuf;		/* buffer address for statistics */
#else	BFLY1
    caddr_t	nm_statbuf;		/* buffer address for statistics */
#endif	BFLY1
#endif	BFLY2
};

/*
 * Layout of a net message.  This is a combination of all the above
 * structures.  A message on the message queue (HOST->EXOS or EXOS->HOST)
 * has contents which are one of the objects described above. This
 * structure is char swapped with respect to nm_reply and nm_request.
 */
struct	net_msg {
    u_short	nm_reserved;		/* rsvd for EXOS usage or socket id */
    u_short	nm_userid[2];		/* sequence # to attach to msg */
                         /* We only use word 0 to simplify BFLY2 access*/
    u_char	nm_request;		/* command */
    u_char	nm_reply;		/* result */
    union {
	struct exdata	  _nm_dblk;	/* for transmit/receive */
	struct net_mode	  _nm_netmode;	/* for net_mode */
	struct net_addrs  _nm_addrs;	/* for net_addrs */
	struct net_recv	  _nm_recv;	/* for net_recv */
	struct net_ststcs _nm_ststcs;	/* for net_ststcs */
    } nm_u;
};

/*
 * net_msg structure element equivelent definitions
 */
#define	nm_soid		nm_reserved
#define	nm_dblk		nm_u._nm_dblk
#define	nm_netmode	nm_u._nm_netmode
#define	nm_addrs	nm_u._nm_addrs
#define	nm_recv		nm_u._nm_recv
#define	nm_ststcs	nm_u._nm_ststcs

/*
 * nm_request types
 */
#define	EX_TRANSMIT		0x0C	/* transmit a packet */
#define	EX_XMITECHO		0x0E	/* transmit with self-receive */
#define	EX_RECEIVE		0x0D	/* receive packets */
#define	EX_NET_MODE		0x08	/* read/write controller mode */
#define	EX_NET_ADDRS		0x09	/* read/write address slot */
#define	EX_NET_RECV		0x0A	/* read/alter receive status */
#define	EX_NET_STSTCS		0x0B	/* gather/set statistics */

/*
 * nm_reply codes
 */
#define	REPLY_OK		0x00	/* all is well */
#define EX_REPLY_SQL_ERR	0x08	/* IEEE 802.3 heartbeat missing */

/*
 * nm_reqmask bits
 */
#define	EX_RQ_WRITE		1
#define	EX_RQ_READ		2
#define	EX_SLOT_ENABLE		4	/* valid only for NET_RECV read */
#define	EX_SLOT_VALID		8	/* valid only for NET_ADDRS read */

/*
 * When talking to the EXOS firmware in link mode, all communication is
 * done through a  queue of messages.
 *	Format of a standard EXOS->HOST or HOST->EXOS message:
 *	- this is what is linked together into a queue which both the
 *	  HOST and EXOS manipulate while talking to each other
 *	- a message contains:
 *		- a header describing the state of the message and its size
 *		- an actual network message
 *		- a link for the HOST to use to maintain and follow the
 *		  message queue with
 */
struct	ex_msg {
#ifdef BFLY2
    u_short	mh_dummy;		/* pad so mh_netmsg is word aligned */
#endif BFLY2
    u_short	mh_link;		/* EXOS link address */
    u_char	mh_reserved;		/* not used; must be 0 */
    u_char	mh_status;		/* status of queue element */
    u_short	mh_length;		/* length of data in queue element */
    struct	net_msg mh_netmsg;	/* actual message */
    struct	ex_msg *mh_next;	/* HOST ptr to next msg */
};

/*
 * mh_status flags/values/masks
 */
#define	EXMQ_OWNER		0x01	/* ownership mask */
#define	EXMQ_HOST		0x00	/* HOST owns queue element */
#define	EXMQ_EXOS		0x01	/* EXOS owns queue element */
#define	EXMQ_DONE		0x02	/* EXOS is done with queue element */
#define	EXMQ_OVERFLOW		0x04	/* data area too big */

/*
 * To run this board, a static data area is allocated at boot time which
 * will contain the linked list of queues used by the EXOS.
 *
 * The msg_area structure is used to contain the working queues which
 * both the HOST and the EXOS use.  It is declared in this fashion so as
 * to allow it to be allocated at run time in the main-bus address space.
 *
 * DMA_BUFS must be less than 32 (the Excelan hardware queue limit)
 * Each causes allocation of ETHERMTU+sizeof(struct ether_header)+4
 * bytes. You want to avoid wasting space on that page.
 *
 */

#define	SEGNO(x)	(((u_long)(x) >> 4) & 0xF000)
#define	SEGADDR(x)	((u_long)(x) & ((u_long) 0xFFFF0000))

/*
 * 6 packets can hold one maximally sized NFS request.
 * post one receive beyond that for whatever.
 */
#define EX_NRECEIVE	8	/* Try to keep this many receives posted */
#define EX_MAXTRANSMIT  3	/* Maximum transmit buffers pending */
#define DMA_BUFS		(EX_NRECEIVE+EX_MAXTRANSMIT+2)
				/* one extra for receive restart */
				/* one extra for watchdog */
struct	msg_area {
    struct	ex_msg *ma_next;		/* next available msg */
    struct	ex_msg ma_msgs[DMA_BUFS];	/* HOST<->EXOS msgs */
    u_short	ma_link;			/* EXOS link to ma_msgs */
};

/*
 * Definition of pending DMA resources held by a message
 */
struct ifdma {
    u_long busaddr;		/* Address of allocated slave map ram */
    struct mbuf *mbuf;		/* mbuf which holds the message */
    u_char	inuse;		/* flag set when entry is in use */
    u_char	echo;		/* flag set to echo a transmitted packet */
};

/*
 * Ethernet software status per interface.
 *
 * Each interface is referenced by a network interface structure, xs_if,
 * which the routing code uses to locate the interface. This structure
 * contains the output queue for the interface, its address, ...
 */
struct	ex_softc {
    u_long	xs_busaddr;		/* bus address of xs memory */
    u_short	xs_nreceive;	/* # receives currently pending */
    u_short	xs_ntransmit;	/* # of transmits pending */
    struct	arpcom xs_ac;		/* Ethernet common part */
    struct	ifdma xs_ifdma[DMA_BUFS];/* DMA resources */
    struct	init_msg xs_im;		/* initialization message */
    u_char	xs_flags;		/* flag information */
    u_char	xs_poke_needed;		/* Set if the board needs to be poked*/
					/* cleared only under lock */
    short	xs_scaninterval;	/* interval of stat collection */
    struct	ex_stats xs_stats;	/* holds on-board statistics */
    struct	ex_stats xs_sum;	/* summation over time */
    char	xs_version[4];		/* version numbers from init */
    struct	msg_area *xs_h2xarea;	/* the request queue HOST->EXOS */
    struct	msg_area *xs_x2harea;	/* the reply queue EXOS->HOST */
    u_short	xs_intcount;		/* count of interrupts, for watchdog */
    u_short	xs_intwatch;		/* Last count */
    u_short	xs_stattimer;		/* countdown used to set stat  */
    u_short	xs_deathwatch;		/* countdown to controller hung */
    struct	mb_device *xs_md;	/* pointer to device information */
    struct	exdevice *xs_ex;	/* vectorized exdevice pointer */
    u_char	*xs_portB;		/* pointer to portB on the device */
    simple_lock_data_t xs_lock;		/* multiprocessor lock */
    int		xs_sendactive;		/* count of active transmitters */
					/* interlock with reset code */
};

/*
 * xs_softc structure element equivelent definitions
 */
#define	xs_if	xs_ac.ac_if		/* network-visible interface */
#define	xs_addr	xs_ac.ac_enaddr		/* hardware Ethernet address */

/*
 * xs_flags values
 */
#define	EX_STATPENDING		0x01	/* stat cmd pending */
#define	EX_ALIVE		0x02	/* board is there */
#define	EX_RUNNING		0x04	/* board is online */
#define	EX_SETADDR		0x08	/* physaddr has been changed */
#define	EX_302			0x10	/* Excelan 302 controller if set */
#define EX_RESETTING		0x20	/* Controller reset in progress */

/*
 * general definitions
 */
#define	BBN_SQL_ERR_IGNORE	1	/* remove SQL err on 802.3 nets */
#define	EXWATCHINTERVAL		1500	/* watch stats interval (seconds) */
#define	EXDEATHINTERVALS	2	/* scanintervals for death threshold */

#endif	_IF_EXREG_H
