/* $Header: /nfs/europa/u1/BFMACH.local/mk.gibson.RP/butterflydev/RCS/mbreg.h,v 1.20 90/05/01 12:26:08 gibson Exp $ */

/* mbreg.h:
 * Constants for
 *	(On BFLY1): Multibus AND BMA (butterfly multibus adaptor).
 * 	(On BFLY2): "Main" Bus (VMEbus) AND Butterfly 2 VMEbus interface
 *		(on B2VME function card).
 */


/*
 * MBI_ADDR will convert a butterfly virtual address to a physical address
 * on the BFLY1.  On BFLY2, we use a more complicated allocation scheme,
 * thus the simple translation of MBI_ADDR() is not appropriate.
 */
#ifdef BFLY1
/*
 * On BFLY1, there is no need for further conversion to Multibus address since
 * Butterfly physical address is mapped 1:1 into MB address space.
 */
# define MBI_ADDR(x) svtop(x)

#ifndef VME130
/*
 * On BFLY1, if Multibus address is offset by MBI_SWAP_OFFSET,
 * the device will see a swapped version of BFLY1 local physical memory.
 */
#define	MBI_SWAP_OFFSET	0x400000
#endif VME130

#endif BFLY1

#ifdef BFLY2

/* Base of virtual mapping for each node's vme slave mapping REGISTERS.
 * The registers for "lognode" occupy 8K (2K 4 byte registers) at
 * VEC_VME_SLAVE_MAP+(lognode*8K)
 */
#define	VEC_VME_SLAVE_MAP	((struct vme_slave_map_entry *)_VEC_VME_SLAVE_MAP)

/* Base of virtual mapping for machine wide vme master map PAGES.
 * The mapping for "vme_slot" (an 8K window on some node's VME bus)
 * occupies 8K of virtual space at VEC_VME_SLAVE_MAP+(vme_slot*8K)
 */
#define	VEC_VME_MASTER_MAP	_VEC_VME_MASTER_MAP

/* Note: MB_PHYS_START is now a per-node, vectorized quantity set in */
/* init_map.c for the master, and pmap_bootstrap_slave (pmap.c) for slaves */

#define MB_VIRT_START	_VEC_VME_MASTER_MAP /* virt addr of VME memory start */
#define MB_STD_OFFSET	0x00010000 /* offset of VMEbus standard I/O space */

/* Definitions for Butterfly memory, as seen from the VMEbus.  These are
 * useful for passing addresses to controllers, so they can DMA into/out of
 * Butterfly memory.  The slave mapping ram converts these "virtual" addresses
 * into Butterfly physical addresses.
 */
#define BFLY_VIRT_START 0x00400000 /* beginning of mapping Butterfly memory */

#endif BFLY2

/* Definitions pertaining to MB and relating routines.
 */
#ifdef BFLY1
#define MB_VEC_PERNODE	NXY
#endif BFLY1
#ifdef BFLY2
#define MB_VEC_PERNODE	32	/* maximum vectorized devices per node */
#endif BFLY2
#define	NBMAS		32	/* maximum useful bma id (for now) */
#define	BMAID_NULL	(-1)	/* uninitialized BMAID */

#ifdef BFLY1
/*
 * Format of the Misc register:
 *	Bits
 *	15..8		Jumpers
 *	7..4		Unused
 *	3		nopipe
 *	2		not nlock
 *	1		emap
 *	0		embi
 */
# define	MISC		0xFFF7D026
/*
 * values for the bits in the misc. register
 */
# define MB_ENBINT	1		/* let multibus interrupts int bfly */
# define MB_ENBMAP	2		/* enable mb access to bfly */
# define MB_NOLOCK	4
/*
 * and other macros for accessing it
 */
# define misc_Jumpers(x)	(((x) >> 8) & 0xFF)
# define misc_nopipe(x)		(((x) >> 3) & 0x01)
# define misc_not_nlock(x)	(((x) >> 2) & 0x01)
# define misc_emap(x)		(((x) >> 1) & 0x01)
# define misc_embi(x)		((x)        & 0x01)


/* Multibus mapping RAM register:
 *	multibus => bfly address mapping
 *	maps bits 16-23 of 0-23 so multibus devices can access bfly memory
 * There are 256 of these 8 bit registers.  The high order 2 bits in the
 * register define if/how the multibus accesses Butterfly memory.
 */
# define	MB_MAPPING	0xFFF7E000
# define	N_MBMAP		256
/*
 * values for the high order 2 bits in an address mapping register
 *	MB_SWAP		swap bytes in short
 *	MB_SS0		to get PNC special function
 *	MB_MAPNONE	mbaddr does not access Bfly
 */
# define	MB_NOSWAP	((unsigned char) 0)
# define	MB_SWAP		((unsigned char) 0x80)
# define	MB_SS0		((unsigned char) 0x40)
# define	MB_MAPNONE	((unsigned char) 0xc0)
/*
 * When X is mapped by a register with type MB_SS0, the Multibus should
 * do the equivalent of the following C code to perform a PNC special function:
 *
 *	MBADDR(phys addr of PNC function parameters) = PNC function code;
 *
 * Currently, the microcode supports only the post event function:
 */
# define	MB_POST		((unsigned char) 0)

/*
 * Definitions for types of multibus memory.  Used to convert multibus
 * addresses into virtual butterfly addresses to allow reads and writes
 * into multibus address space
 */

# define	MB_MEMSWAP	0	/* mb memory swapped */
# define	MB_IOSWAP	1	/* mb io swapped */
# define	MB_MEMNSW	2	/* mb memory not swapped */
# define	MB_IONSW	3	/* mb io not swapped */

# define	MB_IOMASK	0xFFFF
# define	MB_MEMMASK	0xFFFFFF
#ifdef VME130
#define     MB_SWIO     syntax botch
#define     MB_NSWIO    0xFFFF0000
#else VME130
# define	MB_SWIO		0xFFF40000	/* mb swapped IO */
# define	MB_NSWIO	0xFFE40000	/* mb not swapped IO */
#endif VME130
# define	MB_NSWMEM	0xFF000000	/* mb not swapped memory */
# define	MB_MAP		0xFF7FFFFF


/* Multibus vector RAM register:
 *	maps multibus interrupt 0-7 to butterfly interrupt 3 or 4
 *	and allows software control over interrupt processing hierarchy
 *	by treating the interrupt request lines as addressing bits for the ram
 */
# define	MB_VECTOR	0xFFF7F000
# define	N_MBVEC		256

/* each vector Ram register is 16 bits wide.  It is broken down as follows:
 *
 *	bits	function
 *	7	1 if a butterfly level 4 is to be generated
 *	6..4	which multibus interrupt to service, and index passed to bfly
 *	3	1 if a butterfly level 3 is to be generated
 *	2..0	which multibus interrupt to service, and index passed to bfly
 */

# define	VEC_LEVEL4	0x80
# define	VEC_LEVEL3	0x8


/*
 * and a Multibus address is:
 */
typedef long		MBADDR;
typedef unsigned short	MBIOADDR;

/*
 * BMA static information (to direct configure).
 */
struct bma {
	short	*bma_misc;		/* miscellaneous register address */
	caddr_t	bma_map;		/* mapping ram address */
	caddr_t	bma_vec;		/* vector ram address */
};
#endif BFLY1
