/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/mbvar.h,v 1.18 90/03/05 08:58:56 tappan Exp $ */


/*****************************************************************************
* Structuctures for describing bus setup.
*
* NB: "mb" (main bus) is the MAIN I/O BUS on the machine (e.g. Multibus).
*	It is so-called for historical reasons, since our first main bus was
*	the multibus.
*
******************************************************************************/
#include "../h/map.h"
#include "../machine/psl.h"

#ifndef LOCORE
struct mb_hd {
	int	mh_node;		/* physical node main bus attached to*/
	int	mh_bmaid;
	int	mh_alive;		/* is entry instantiated */
	struct map *mh_map;		/* dma info (not currently used) */
};


struct mb_ctlr {
	struct mb_driver *mm_driver;	/* ptr to driver struct */
	short	mm_ctlr;		/* index into driver cinfo tbl */
	short	mm_alive;		/* is it here? */
	caddr_t	mm_addr;		/* register address */
#ifdef	BFLY2
	int	mm_amod;		/* VMEbus address modifier */
#endif	BFLY2
	int	mm_intpri;		/* device interrupt priority */
	int	mm_mbnum;		/* main bus number */
	struct mb_hd *mm_hd;		/* pointer to header */
};

struct mb_device {
	struct mb_driver *md_driver;	/* ptr to driver struct */
	short	md_unit;		/* unit number */
	short	md_ctlr;		/* index into driver cinfo tbl */
					/* md_ctlr = -1 if no ctlr */
	short	md_slave;		/* slave on controller */
#if defined(BFLY2) && defined(KERNEL_GENASSYM)
	short	pad1;			/* for xcompilation/genassym.c */
#endif KERNEL_GENASSYM
	caddr_t	md_addr;		/* register address */
#ifdef	BFLY2
	int	md_amod;		/* VMEbus address modifier */
#endif	BFLY2
	short	md_alive;		/* is it here? */
#if defined(BFLY2) && defined(KERNEL_GENASSYM)
	short	pad2;			/* for xcompilation/genassym.c */
#endif KERNEL_GENASSYM
	int	md_intpri;		/* device interrupt priority */
	int	md_mbnum;		/* main bus number */
	u_short  md_vec1, md_vec2; /* VME interrupt vectors */
	int	md_flags;
	struct mb_device *md_device;
	struct mb_ctlr *md_mi;		/* pointer to its ctlr struct */
	struct mb_hd *md_hd;		/* pointer to head struct */
};

struct mb_driver {
	int	(*mi_probe)();		/* see if device is really there */
	int	(*mi_slave)();		/* see if slave is there */
	int	(*mi_attach)();		/* setup driver for a slave */
	int	mi_empty;		/* here to make sun drivers compat */
	int	mi_empty2;
	int	(*mi_intr)();		/* interrupt handler routine */
	int	mi_size;
	char	*mi_dname;		/* name of the device */
	struct mb_device **mi_dinfo;	/* backpointers to mbdinit table */
	char	*mi_cname;		/* name of the controller */
	struct mb_ctlr **mi_cinfo;
	int	mi_flags;
	int	mi_empty3;
};

/* Struct used by config to pass information about mapping between specific
 * bma's and a "multibus number".  This lets system configurer specify a
 * mapping between specific bma id's and specific devices.
 */
struct mb_adptr {
	int	ma_bmaid;		/* bma misc register id (jumpered) */
	int	ma_mbnum;		/* main bus number */
};
#endif LOCORE

#define IOPBMAPSIZE 64
#ifdef BFLY1
#define IOPBMEMSIZE 0x2000	/* should always be less than a page size */
#endif BFLY1
#ifdef BFLY2
#define IOPBMEMSIZE VM_PAGESIZE	/* should always be less than a page size */
#endif BFLY2

struct map *iopbmap;
vm_offset_t iopbmem;

#define MDR_DMA	0x1
#define MDR_BIODMA 0x2


#ifndef LOCORE
#ifdef KERNEL

/*
 * Character symbols in ioconf.c, used in place of integers to indicate
 *	wildcarding.  If '*' is too small, replace with 'A'.  If '?' is
 *	too small, more work is required since mb.c (or vme.c) is littered with
 *	'?'.
 */
#define	QUES	'?'
#define	ANY	'*'

/* routines called by autoconf.c */
void mbinitmap(), mbioinit(),  mbinit();
#ifdef BFLY1
void mbmeminit();
#endif BFLY1

/*
 * Main Bus related kernel variables
 */

extern struct mb_hd	mb_hd[];
extern int	nummb;			/* number of main buses */

/*
 * These initialize the tables specifying possible devices (generated in
 *	ioconf.c by config(8).
 */

extern struct mb_adptr	mbainit[];

#ifdef BFLY1
extern struct mb_ctlr	mbiocinit[];
extern struct mb_device	mbiodinit[];

extern struct mb_ctlr	mbmemcinit[];
extern struct mb_device	mbmemdinit[];
#endif  BFLY1

#ifdef	BFLY2
extern struct mb_ctlr	mbcinit[];
extern struct mb_device	mbdinit[];
#endif BFLY2


#ifdef BFLY1
extern	vm_offset_t	multibus_start[], multibus_end[];
#endif BFLY1 

#endif KERNEL
#endif LOCORE
