/* $Header: /nfs/columbia/u1/BFMACH.local/mk/butterflydev/RCS/sd.h,v 1.9 90/12/28 16:56:21 uban Exp $ */

#ifndef _SD_H
#define _SD_H
/*****************************************************************************
* File: sd.h - SCSI Disk Driver definitions.                                 *
*                                                                            *
* Started 12/8/88 by drb                                                     *
*                                                                            *
* [And now, a few words from the legal department.]                          *
*                                                                            *
* Copyright 1988 by BBN Advanced Computers Inc.                              *
* ALL RIGHTS RESERVED                                                        *
*                                                                            *
* No part of this material may be reproduced, stored in a retrieval          *
* system, transmitted or used in any form or by any means, electronic,       *
* mechanical, photocopying, recording, or otherwise, without the prior       *
* written permission of, or express license from BBN Advanced Computers Inc. *
*                                                                            *
* RESTRICTED RIGHTS LEGEND                                                   *
*                                                                            *
* Use, duplication, or disclosure by the Government is subject to            *
* restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to      *
* Technical Data and Computer Software clause at DFARS 52.227-7013.          *
*                                                                            *
* BBN Advanced Computers Inc.                                                *
* 10 Fawcett Street                                                          *
* Cambridge, Ma. 02238                                                       *
* 617-873-6000                                                               *
*****************************************************************************/

#ifdef KERNEL
struct sd_part
    {
    unsigned int offset;
    unsigned int size;  
    };

struct sd
    {
    int     ident;
    int     flags;
    int     in_progress;
    int low_water_mark ;
    unsigned nrequests;
    simple_lock_data_t lock;
    struct  sd_part partab[NPARTITIONS];
    unsigned transfer_count;
    char lbl_name[64];
    };

typedef struct sd SD;

/* flags */
#define NT_DRIVE_ALIVE    0x00000001
#define NT_DRIVE_THROTTLE 0x00000002

#define SCSI_COMMAND_READ  0x08
#define SCSI_COMMAND_WRITE 0x0A
#define SCSI_COMMAND_SENSE 0x03

#define SCSI_IDENT  0x12345678

#define NT_THROTTLE_LOWATER 5


/* Minor device number mappings:
   low-order 3 bits: Partition Number
         next 1 bit: Adapter Number
         next 1 bit: Bus Number
        high 3 bits: Disk Number (SCSI bus ID)      */

#define SD_PARTITION_SHIFT 0
#define SD_ADAPTER_SHIFT 3
#define SD_BUS_SHIFT 4
#define SD_DEVICE_SHIFT 5
#define SD_PARTITION_MASK 0x7
#define SD_ADAPTER_MASK 0x1
#define SD_BUS_MASK 0x1
#define SD_DEVICE_MASK 0x7

#define dev_to_partition(d) (((d)>>SD_PARTITION_SHIFT) & SD_PARTITION_MASK)
#define dev_to_adapter(d) (((d)>>SD_ADAPTER_SHIFT) & SD_ADAPTER_MASK)
#define dev_to_bus(d) (((d)>>SD_BUS_SHIFT) & SD_BUS_MASK)
#define dev_to_device(d) (((d)>>SD_DEVICE_SHIFT) & SD_DEVICE_MASK)

/* make the scsi disk minor device given the adapter,bus,device,partition */

#define sd_makeminor(a,b,c,d) ((((a)&SD_ADAPTER_MASK)<<SD_ADAPTER_SHIFT)|\
			       (((b)&SD_BUS_MASK)<<SD_BUS_SHIFT) | \
			       (((c)&SD_DEVICE_MASK)<<SD_DEVICE_SHIFT) |\
			       (((d)&SD_PARTITION_MASK)<<SD_PARTITION_SHIFT))

#define SECTOR_SIZE_IN_BYTES  512
#define SENSE_BYTE_COUNT 64  /* >= number of sense bytes expected */

#define RECOVERABLE_ERROR 1

/* Note: requires ../h/ioctl.h */
#endif
#endif _SD_H
