/* $Header: /nfs/europa/u1/BFMACH.local/mk/butterflydev/RCS/sscsi.h,v 1.9 90/03/05 09:02:46 tappan Exp $ */

#ifndef _SSCSI_H
#define _SSCSI_H

#define DEVICES_PER_BUS 7
#define BUSSES_PER_ADAPTER 2
#define ADAPTERS_PER_NODE   4

#ifndef ASSEMBLER

/*****************************************************************************
  File:     sscsi.h
  Contents: Definitions for VME Synchronous SCSI Bus-layer interface
  System:   MACH 1000
  
  Copyright 1988 by BBN Advanced Computers Inc.
  ALL RIGHTS RESERVED
  
  No part of this material may be reproduced, stored in a retrieval
  system, transmitted or used in any form or by any means, electronic,
  mechanical, photocopying, recording, or otherwise, without the prior
  written permission of, or express license from BBN Advanced Computers Inc.
  
  RESTRICTED RIGHTS LEGEND
  
  Use, duplication, or disclosure by the Government is subject to
  restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
  Technical Data and Computer Software clause at DFARS 52.227-7013.
  
  BBN Advanced Computers Inc.
  10 Fawcett Street
  Cambridge, Ma. 02238
  617-873-6000
  ****************************************************************************/

#ifdef KERNEL
#define SSCSI_MAXCMD	32
#define MAX_SG_PAGES 8

struct sscsi_request
{
    int	reqflags;		/* Request flags 			*/
    struct buf     *bp;		/* Buffer, may be NULL 		*/
    unsigned long sg_list_addr; /* Used for scatter/gather list address */
    int             sg_count ;    /* the number of sg list elements */
    unsigned        physaddr ;	/* Phys address, may be NULL*/
    unsigned        physaddr2 ;	/* Phys address, may be NULL*/
    unsigned sg_copybuf_list[MAX_SG_PAGES+1]; /* copybufs for scatter/gather */
    int		    lognode;	 /* Logical node w/ device   */
    int             localnode ;  /* node that the i/o started from */
    int             iopb_index ; /* index into HUS address space for iopb*/
    short	    busid;	 /* bus number on that node  */
    short	    deviceid;	 /* device number on that bus*/
    int           (*done_routine)(); /* Completion routine		*/
    simple_lock_data_t lock;	     /* Interlock for next datum */
    int statflags;	/* Status flags, eg DONE    */
    int cmdlen; 	/* Length of command        */
    unsigned char command [SSCSI_MAXCMD]; /* command                  */
    int transfer_count;         /* number of bytes xferred  */
    short device_retval;	/* SCSI status byte         */
    short bus_retval;		/* VME / host adapt status  */
    char sc ;                   /* sense code from a device */
    char asc ;                  /* additional sense code */
    unsigned blocknum;          /* block number for printf  */
    unsigned procp ;            /* used to do fast sleep and wakeup */
    
};

typedef struct sscsi_request *sscsi_request_t;
typedef struct sscsi_request SSCSI_REQUEST;

#define MAXSSCSI_REQUEST  12

/* Definitions for the reqflags member */

#define RF_INPUT	    0x00000001
#define RF_OUTPUT	    0x00000002
#define RF_HIPRI	    0x00000004
#define RF_DEVRESET         0x00000008
#define RF_BUSRESET         0x00000010
#define RF_COPYBUF          0x00000020
#define RF_COUNT_IMPRECISE  0x00000040
#define RF_TIMEOUT_EXPECTED 0x00000080
#define RF_ASYNC            0x00000100
#define RF_ERROR_RECOVERY   0x00000200
#define RF_SCATTER_GATHER   0x00000400

/* Definitions for the statflags member 		*/
/* All accesses require that user hold the lock   	*/

#define SF_DONE		0x00000001
#define SF_WAKEUP	0x00000002
#define SF_ERROR        0x00000004


/* definitions of SCSI commands and parameters for Tapes and Disk Drives.
 * Note:
 * SCSI stands for generic scsi structure.
 * SD stands for generic SCSI disk structure.
 * ST stands for generic SCSI tape structure.
 * CI stands for Cipher 1/2 inch tape structure.
 * TA stands for Tandberg 1/4 Cartridge tape stucture.
 * 
 * When the command block or parameter list structure is different, the struct 
 * name will start off the the Above 2 characters for the device it is used
 * for. Generic Tape structs start off with ST and generic Disk structs
 * start off with SD. If the structure can be used for both, is starts off with
 * scsi.
 */

#define SCSI_DISK_DEVICE 0
#define SCSI_TAPE_DEVICE 1

#define ISSCSITAPE(I) ((inquiry->device_type == SCSI_TAPE_DEVICE) ? 1 : 0 )
#define ISSCSIDISK(I) ((inquiry->device_type == SCSI_DISK_DEVICE) ? 1 : 0 )

/* INQUIRY Command 0x12 */

#define SCSI_COMMAND_INQUIRY 0x12

struct scsi_inquirydata
{
    char device_type ;
    char device_qualifier ;
    char version ;
    char res1 ;
    char length ;
    char res2 ;
    char res3 ;
    char res4 ;
    char vendor_id[12] ; 
    /* the rest is not needed in the driver, as it can be returned via
     * a SCSI IOCTL
     */
} ;

#endif KERNEL
/* Generic stuff for a user to send SCSI commands directly to the device. */

#define SCSI_IOC _IOWR(s, 0, SCSIRAWCMD)
#define SD_STATS _IOWR(s, 10, SDSTATS)
#define SD_TEST  _IOWR(s, 20, SDTEST)

typedef struct sdstats
{
    unsigned nrequests;
    int in_progress;
    unsigned transfer_count ;
} SDSTATS;

typedef struct sdtest
{
    unsigned long addr ;
    int len ;
    int type ;
    int count ;
    unsigned long blkno ;
} SDTEST ;

struct sdioc
{
    int        flags;
    char      *cmd;
    int        cmdlen;
    char      *buffer;
    int        xfercount;
    short      bus_retval;
    short      device_retval;
    char sc,asc ;
} ;

#define SCSI_IOC_NODATA    0   /* no data xfer phase       */
#define SCSI_IOC_READ      1   /* xfer device to processor */
#define SCSI_IOC_WRITE     2   /* xfer processor to device */
#define SCSI_IOC_FUNCTION_MASK 0x3

#define SCSI_IOC_IMPRECISE 4   /* xfer count imprecise     */

typedef struct scsi_inquirydata SCSI_INQUIRY ;
typedef struct sdioc SCSIRAWCMD ;

typedef caddr_t scsi_device_map[][BUSSES_PER_ADAPTER][DEVICES_PER_BUS];
#endif ASSEMBLER

#endif _SSCSI_H




