/* $Header: /nfs/columbia/u1/BFMACH.local/mk/butterflydev/RCS/stripe.h,v 1.3 90/12/28 17:00:32 uban Exp $ */

#ifndef _STRIPE_H
#define _STRIPE_H
/*****************************************************************************
* File: stripe.h - Stripe driver definitions
*                                                                            *
* Started 11/14/89 by Robert J. Goguen
*                                                                            *
* Copyright 1988 by BBN Advanced Computers Inc.                              *
* ALL RIGHTS RESERVED                                                        *
*                                                                            *
* No part of this material may be reproduced, stored in a retrieval          *
* system, transmitted or used in any form or by any means, electronic,       *
* mechanical, photocopying, recording, or otherwise, without the prior       *
* written permission of, or express license from BBN Advanced Computers Inc. *
*                                                                            *
* RESTRICTED RIGHTS LEGEND                                                   *
*                                                                            *
* Use, duplication, or disclosure by the Government is subject to            *
* restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to      *
* Technical Data and Computer Software clause at DFARS 52.227-7013.          *
*                                                                            *
* BBN Advanced Computers Inc.                                                *
* 10 Fawcett Street                                                          *
* Cambridge, Ma. 02238                                                       *
* 617-873-6000                                                               *
*****************************************************************************/

#if USER_PROGRAM
typedef int simple_lock_data_t ;
#endif

#define MAX_STRIPE_SIZE 65536

/* Number of bytes in a sector. */
#define SECTOR_SIZE 512

/* Maximum number of stripe groups in a system */
#define MAX_SGROUPS 32

/* Maximum number of disks in a stripe group. */
#define MAX_NDISKS 32
#define NPARTITIONS 8

struct sgroup_entry {
    simple_lock_data_t lock ;          /* Used when probing to protect date. */
    char group_id ;                    /* Index into group_id_table. */
    char group_size ;                  /* Number of logical disks in sgroup. */
    char disks_probed ;                /* Number of disks actually found. */
    struct {
	unsigned long hi;
	unsigned long lo;
    } unique_id ;            /* Unique id for each disk in sgroup */
    int flag ;                         /* Status. */
    unsigned long ssize[NPARTITIONS] ; /* per partition stripe size. */
    unsigned long ssecsize[NPARTITIONS];/* This is the stripe sector size. I.e
					 * the number of sectors in a stripe
					 * in each partition.
					 */
    SMD_PART  partition[NPARTITIONS];  /* Partition information, used to ensure
					* each stripe partition has the same
					* number of physical blocks and to
					* determine the necessity of skipping
					* the first block on the disk.
					*/
    unsigned long llbn[NPARTITIONS];   /* This is an array of the last logical
					* block number for a striped partition.
					* It is calculated at probe time.
					*/
    struct {
	dev_t cdev ;                   /* Physical location of the character */
	dev_t bdev ;                   /* and block devices for a ldisk. */
    } sg_layout[MAX_NDISKS] ;
    short version ;                    /* Version number. */
} ;

/* Definition of flags for the sgroup_entry flag. */

#define SGROUP_COMPLETE  0x00000001
#define SGROUP_ERROR     0x00000002
#define SGROUP_A_ERROR   0x00000004
#define SGROUP_B_ERROR   0x00000008
#define SGROUP_C_ERROR   0x00000010
#define SGROUP_D_ERROR   0x00000020
#define SGROUP_E_ERROR   0x00000040
#define SGROUP_F_ERROR   0x00000080
#define SGROUP_G_ERROR   0x00000100
#define SGROUP_H_ERROR   0x00000200
#define SGROUP_CHANGING  0x00000400

#define SGROUP_PARTITION_SHIFT 0x2
#define SGROUP_PARTITION_MASK  0x3FC

#define sgroup_partition_ok(flag,p) (((1<<(p))<<SGROUP_PARTITION_SHIFT)&(flag))
#define sgroup_partition_bad(flag,p) ((flag) |= \
				      ((1 << (p)) << SGROUP_PARTITION_SHIFT))
/* The minor device of a stripe group dev_t contains
 * an index into the sgroup_id_table and a partition.
 *
 *     Starting from the LSB using a total of 12 bits for the minor.
 *     3 bits partition
 *     2 bits reserved
 *     5 bits stripe group
 *     2 bits reserved
 */


#define SG_PARTITION_SHIFT 0x0
#define SG_ID_SHIFT 0x5
#define SG_PARTITION_MASK 0x7
#define SG_ID_MASK 0x1F

#define dev_to_sgpartition(d) (((d)>>SG_PARTITION_SHIFT) & SG_PARTITION_MASK)
#define dev_to_id(d) (((d)>>SG_ID_SHIFT) & SG_ID_MASK)

#define SG_MAGIC	"sg"
#define SG_MAGIC_SIZE	2

#define LABEL_IS_STRIPED(nx_lb) ((strncmp(SG_MAGIC,(nx_lb)->stripe_magic,SG_MAGIC_SIZE)) == 0)
#define LABEL_VERSION(nx_lb) (atoi((nx_lb)->stripe_magic + SG_MAGIC_SIZE))

#ifdef KERNEL

/* Interface macros to driver routines. */
#define REQUEST_IS_STRIPED(bp) ((bp)->b_flags & B_STRIPE)
extern simple_lock_data_t sgroup_id_table_lock ;
#endif KERNEL
#endif STRIPE_H


