/*******************************************************************************
********************************************************************************
****									    ****
****			Copyright (C) Ciprico Incorporated 1985.	    ****
****									    ****
****				Ciprico Incorporated			    ****
****				 2955 Xenium Lane			    ****
****				 Plymouth, MN 55441			    ****
****				   (612) 559-2034			    ****
****									    ****
****	Module Number:	32000401					    ****
****	Module Name:	tmtreg.h					    ****
****	Package:	Tapemaster 1000 Driver for 4.2BSD UNIX		    ****
****	Date:		(4/26/85)					    ****
****	Module Rev:	(2.4)						    ****
****									    ****
****	Subroutines:	Macros for SWAB, PTOAI, AITOP, SPLTMT.		    ****
****									    ****
****	Description:							    ****
****		This is the include file for the Tapemaster 1000 Driver.    ****
****		It defines the software interface to the board, defining    ****
****		the various bit level patterns used by the TM 1000.	    ****
****		Also defined are the various structures used by the driver  ****
****		and the board.  No actual data allocation is done, only	    ****
****		definition.  Several macros are also defined, mostly having ****
****		to do with byte ordering issues between Motorola & Intel.   ****
****									    ****
********************************************************************************
*******************************************************************************/

/*******************************************************************************
*									       *
*				Revision History			       *
*									       *
********************************************************************************

    Module Number	Date		Author
	Description of Change
--------------------------------------------------------------------------------

    32000401		4/26/85		D. A. Dickey
	Added MAXRBWC to minor device number; changed the UNIT value range
	of 0-7 to 0-3.

    32000400		4/18/85		D. A. Dickey
	Initial Release.

*******************************************************************************/

/*
	The definitions within this file are as follows:

	- Type definitions for 8, 16, and 32 bit quantities.
	- Macros for reordering bytes within 16 and 32 bit quantities.
	- Macros for busting the minor device number.
	- Defines for the Number of IOPB's and RBH's to use.
	- The structure of the TM 1000 I/O registers and how to interpret
		the various bits therein.
	- The IOPB used by the controller (TMTPB) and how to interpret the
		fields within.
	- The structure used for configuring the TM 1000 and its fields.
	- Ring Buffer structures and their defines.
	- Driver internal structures (TMTDEVICE and IOPB).
*/

/****** First off, a few typedefs to make life a little bit easier. ******/
typedef unsigned char byte;	/* Should be an 8 bit quantity. */
typedef unsigned short word;	/* Should be a 16 bit quantity. */
typedef unsigned long Iptr;	/* Should be a 32 bit quantity. */


/******
 *	These macros are used for moving bytes around within a word.  The
 *  SWAB macro can be used for swapping bytes.  For MACH systems the bytes
 *  within a word are already in the correct order, and we just want to end
 *  up with a word size quantity, so, mask off the upper bits.
 *****/
#define	SWAB(x)		((x) & 0xFFFF)
/*
 *	The PTOAI macro is used to convert a Pointer TO an Absolute Intel
 *  pointer.  For a MACH, this means swapping the words within a pointer.  On
 *  some systems it may also mean swapping bytes within a word, hence the
 *  imbedded call to SWAB.
 */
#define	PTOAI(x)	((Iptr) (((SWAB((Iptr) (x))) << 16) | (SWAB(((Iptr) (x)) >> 16))))
/*
 *	The AITOP macro is used to reverse PTOAI.  We want to use an Absolute
 *  Intel pointer as input, and end up with a pointer on the machine we are
 *  using.  On all machines, just make a call to PTOAI, the macro will
 *  reverse itself.
 */
#define	AITOP(x)	((caddr_t) PTOAI(x))


/******
 *	The minor device number is broken down as follows:
 *		Bit:	76543210  (a one in the bit corresponds to on)
 *			^^^^^^^^
 *			|\ /||\/
 *			| | ||Unit Number.
 *			| | |Speed bit, on selects high speed.
 *			| | Density bit, on selects high density.
 *			| Max RBWC Number.
 *			No Rewind at close.
 *
 *	The Unit Number tells the controller what physical drive the
 *	operations are for.  The No Rewind at close bit, when set, will
 *	prevent the driver from rewinding the Unit on the last close.
 *	The Max RBWC number tells the driver how many Ring Buffer Writes must
 *	be linked together to make a record on the tape drive.  The range for
 *	this number is 0 - 7.  The resulting record size on the tape for the
 *	various values is as follows:
 *		MAXRBWC		Record Size (in Kbytes)
 *		   0		    4
 *		   1		    8
 *		   2		    12
 *		   3		    16
 *		   4		    20
 *		   5		    24
 *		   6		    28
 *		   7		    32
 *
 *	These values assume a File System Block Size of 2K bytes.  For other
 *	block sizes, the resulting record size will be scaled accordingly.
 *
 *	And now the macros to bust the minor device number:
 *****/
#define	UNIT(dev)	(minor(dev) & 0x03)	/* Unit Number from device # */
#define	SPEED(dev)	(minor(dev) & 0x04)	/* Speed select */
#define	DENSITY(dev)	(minor(dev) & 0x08)	/* Density select */
#define	MAXRBWC(dev)	(minor(dev) & 0x70)	/* Max RBWC number */
#define	NOREWIND(dev)	(minor(dev) & 0x80)	/* No Rewind Unit */


/******** Miscellaneous Defines. **********/

#define SPLTMT()	splbio()

#define NIOPB	4	/* Number of iopbs per tape drive */
#define	NRBH	64	/* Number of ring buffers per tape drive */

/******************************************************************************/
/******
 *	This structure defines the order and names of the bytes in Multibus
 *  I/O space for the Tapemaster 1000.  The order of the bytes is reversed
 *  on MACH to account for the Motorola - Intel byte order difference.
 *  If you have problems getting the driver to run on systems other than MACH
 *  and the order of I/O space bytes is in question, you might try reversing
 *  the bytes listed here.
 *****/
struct tmtioreg {		/* I/O space registers (at B0) */
	byte	go;		/* +0 Start Operation */
#define	csr	go		/* +0 Controller Status Register */
	byte	reset;		/* +1 Board Reset */
#define	error	reset		/* +1 Error code */
	byte	clri;		/* +2 Clear Interrupt */
#define	dsr	clri		/* +2 Drive Status Register */
	byte	res1;		/* +3 Reserved */
	byte	addr0;		/* +4 IOPB Address bits 0-7 */
	byte	addr1;		/* +5 IOPB Address bits 8-15 */
	byte	addr2;		/* +6 IOPB Address bits 16-23 */
	byte	addr3;		/* +7 IOPB Address bits 24-31 (not used) */
	byte	resv[8];	/* +8-F Reserved */
};

/*
 * csr bits  (Controller Status Register) in struct tmtioreg.
 */
#define	CS_BSY	0x01		/* TM 1000 Busy */
#define	CS_ERR	0x02		/* Error */
#define	CS_RBI	0x04		/* Ring Buffer Interrupt */
#define	CS_CCI	0x08		/* Command Complete Interrupt */

/*
 * dsr bits  (Drive Status Register) in struct tmtioreg.
 *	Also in the upper byte for Drive Status command - see below.
 */
#define	DS_UMASK	0x07		/* Mask for unit selected */
#define	DS_NRDY		0x08		/* Drive Not Ready */
#define	DS_NBOT		0x10		/* Not at Beginning Of Tape */
#define	DS_NEOT		0x20		/* Not at End Of Tape */
#define	DS_NRWD		0x40		/* drive is Not Rewinding */
#define	DS_NONL		0x80		/* drive is Not Online */
/* The following additional defines are returned in the lower byte of the
 * Interface Status Block when the Drive Status command is used.  The above
 * DS_ defines are returned in the upper byte with a slight twist.  Actually,
 * the sense of the bit is negated.  Also, the order of the two bytes is
 * determined by the high order bit in the control byte.
 */
#define	DS_RDY		0x0008		/* Drive Ready */
#define	DS_BOT		0x0010		/* at Beginning Of Tape */
#define DS_EOT		0x0020		/* at End Of Tape */
#define	DS_RWD		0x0040		/* drive is Rewinding */
#define	DS_ONL		0x0080		/* drive is Online */
#define	DS_FBY		0x0100		/* Formatter Busy */
#define	DS_DBY		0x0200		/* Drive Busy */
#define	DS_DEN		0x0400		/* Density (Interface P2, pin 26) */
#define	DS_SPD		0x0800		/* Speed (Interface P2, pin 40) */
#define	DS_FPT		0x1000		/* Drive is Write Protected */
#define	DS_ID		0x2000		/* Identification
					 * (Interface P2, pin 16) */
#define	DS_SGL		0x4000		/* Single head (Interface P2, pin 14) */


/******************************************************************************/
/*****
 *	TMTPB is the actual IOPB used by the controller at execution time.
 ****/
struct TMTPB {
	byte	control;	/* Control */
	byte	cmd;		/* Command  */
	word	count;		/* Byte/Record Count */
	Iptr	sdptr;		/* Source/Destination Pointer */
	Iptr	pblink;		/* Linked Parameter Block Pointer */
	byte	errco;		/* Error Code */
	byte	gate;		/* Gate */
	word	transfer;	/* Record Count */
};

/*
 * cmd - Command Codes  (cmd in struct TMTPB)
 */
#define CONFIGURE	0x00	/* Configure */
#define GIVECONF	0x01	/* Return Configuration */
#define	RESET		0x10	/* Reset */
#define	REWIND		0x11	/* Rewind */
#define	SECURE		0x12	/* Security Erase */
#define	ERASE		0x13	/* Fixed Erase */
#define	ONLINE		0x14	/* Online */
#define	OFFLINE		0x15	/* Offline */
#define	VARERASE	0x16	/* Variable Erase */
#define	DSTATUS		0x17	/* Get Drive Status */
#define	VAREREOT	0x18	/* Variable Erase to EOT */
#define	WFM		0x20	/* Write File Mark */
#define	SFM		0x21	/* Space File Mark */
#define	SPACE		0x22	/* Space Record(s) */
#define	READ		0x30	/* Read */
#define	RBREAD		0x31	/* Ring Buffer Read */
#define	WRITE		0x32	/* Write */
#define	RBWRITE		0x33	/* Ring Buffer Write */
#define	WUR		0x35	/* Ring Buffer Write Unlimited Record Size */
#define	RUR		0x36	/* Ring Buffer Read Unlimited Record Size */
#define	DIAG1		0x40	/* Diagnostic Test 1 */
#define	DIAG2		0x41	/* Diagnostic Test 2 */
#define	NSC		0x50	/* Non-Standard Command */
#define	NSCP		0x51	/* Non-Standard Command with Parameters */
#define	NSRS		0x52	/* Non-Standard Read Status Command */

/*
 * control - Control  (control in struct TMTPB)
 */
#define	C_UNITSEL	0x07	/* Bits for Unit Select */
#define	C_REVERSE	0x08	/* Tape Direction */
#define	C_INT		0x10	/* Interrupt upon command execution */
#define	C_LNK		0x20	/* Linked parameter block follows */
#define	C_EDT		0x40	/* Write Edit */
#define	C_ORDER		0x80	/* Tells the controller what byte order the */
				/* TMTPB is in. */

/*
 * gate - Command Gate  (gate in struct TMTPB)
 */
#define G_ENT		0x01	/* Command Entered */
#define G_CMP		0x02	/* Command Done */
#define G_ERR		0x04	/* Error in command execution */

/*
 * errco - Error Codes  (errco in struct TMTPB)
 */
		/* Recoverable errors: */
#define	BUSTOUT		0x01	/* Bus Timeout */
#define	NOREVBOT	0x02	/* Can't Reverse at BOT */
#define	HITEOT		0x03	/* End Of Tape */
#define	HITFM		0x04	/* Filemark detected */
#define	FBUSY		0x05	/* Formatter Busy */
#define	HARDTERR	0x06	/* Hard Tape Error */
#define	SWPADDR		0x07	/* Invalid Address to start swap (must be even) */
#define	SWPWIDTH	0x08	/* Invalid Bus Width for swap (must be 16) */
#define	SWPCOUNT	0x09	/* Invalid Byte Count to swap (must be even) */
#define	NOCMD		0x0A	/* Invalid Command */
#define	COUNTERR	0x0B	/* Invalid Byte/Record Count */
#define	RETRYERR	0x0C	/* Invalid retry count */
#define	THRTLERR	0x0D	/* Invalid Throttle size */
#define	LARGER		0x0E	/* Actual Record was Larger than requested */
#define	PREVPB		0x0F	/* Linked parameter block error (in prev. PB) */
#define	ONL_US		0x10	/* Online command unsupported */
#define	PARITY		0x11	/* Parity error */
#define	RDOVERRUN	0x12	/* Read Overrun */
#define	REVBOT		0x13	/* Reversed onto BOT */
#define	SMALLER		0x14	/* Actual Record was Smaller than requested */
#define	SOFTTERR	0x15	/* Soft Tape Error */
#define	PROTECTED	0x16	/* Tape File Protected */
#define	DRVBUSY		0x17	/* Drive busy (data busy) */
#define	NOTONL		0x18	/* Drive not Online */
#define	NOTRDY		0x19	/* Drive not Ready */
#define	WFMERR		0x1A	/* Write Filemark Error */
#define	WTUNDERRUN	0x1B	/* Write Underrun */
#define	SIGNALSERR	0x1C	/* Invalid signals parameter in configuration */
#define	BLANK		0x1D	/* Blank tape encountered */
#define	NOGO		0x1E	/* No Go error */
#define	NSCLENGTH	0x1F	/* Invalid non-standard block command length */
#define	NSCBCOUNT	0x20	/* Invalid non-standard block byte count */
#define	TODBNA		0x21	/* Timeout: Data Busy Never Active */
#define	TODBNI		0x22	/* Timeout: Data Busy Never Inactive */
#define	NOWSTROBE	0x23	/* No Write Strobes or incorrect number */
		/* Unrecoverable Errors: */
#define	LOCALMEM	0x80	/* Local memory error */
#define	PLEMPTY		0x81	/* Parameter Latch Empty Error */
#define	PLFULL		0x82	/* Parameter Latch Full Error */
#define	PROMCHK		0x83	/* PROM check error */
#define	PFIFO		0x84	/* Parameter/FIFO test error */
#define	FIFORAM		0x85	/* FIFO Ram test error */
#define	TAPESIDE	0x86	/* Tape Side test error */
#define	UTEA		0x87	/* Underrun Test Error A */
#define	UTEB		0x88	/* Underrun Test Error B */
#define	OTEA		0x89	/* Overrun Test Error A */
#define	OTEB		0x8A	/* Overrun Test Error B */
#define	TPTEA		0x8B	/* Tape Prescale Test Error A */
#define	TPTEB		0x8C	/* Tape Prescale Test Error B */
#define	TPTEC		0x8D	/* Tape Prescale Test Error C */
#define	BPTEA		0x8E	/* Bus Prescale Test Error A */
#define	BPTEB		0x8F	/* Bus Prescale Test Error B */
#define	BTHE		0x90	/* Bus Timeout Hardware Error */
#define	PLHE		0x91	/* Parameter Latch Hardware Error */
#define	PBPRE		0x92	/* Parameter Block Pointer Register Error */
#define	ERE		0x93	/* Error Register Error */
#define	BSRE		0x94	/* Board Status Register Error */
#define	TSRE		0x95	/* Tape Status Register Error */


/******************************************************************************/
/******
 * CONFIGURE - Drive Configuration information
 *	This block is used for the CONFIGURE and GIVECONF commands
 *****/

struct CONFBLOCK {
	byte	tapecntrl;	/* Tape Control */
	byte	buscntrl;	/* Bus Control */
	byte	throttle;	/* Throttle */
	byte	signals;	/* Signals */
	byte	reserved;	/* Reserved */
	byte	retries;	/* Retry Value */
};

/*
 * buscntrl (Bus Control)  (buscntrl in CBLOCK)
 */
#define	WIDTH8		0x01	/* to select 8-Bit Bus Width */
#define	TIMETYPE	0x02	/* to select Time Type Throttle */
#define	SWP		0x04	/* to select Byte Swap during data transfer */
#define	INH		0x08	/* to select Inhibit advancing of addr count */

/*
 * tapecntrl (Tape Control)  (tapecntrl in CBLOCK)
 */
#define	HIGHSPD		0x01	/* Tape Speed */
#define	HIGHDEN		0x02	/* Tape Density */
#define	EXTGAP		0x04	/* Inter-Record Gap Lenth (set for extended) */
#define	INCLUDECHK	0x08	/* Check Character Enable */

/*
 * throttle - At power up, the board starts with 0x8 bytes for the throttle.
 *	Let's retain this for the CONFIGURE within tmtopen.
 */
#define	DEF_THROTTLE	0x8

/*
 * signals - Defines for creating signal values:
 */
#define	P1_36_GAP	0x01
#define	P1_36_DEN	0x02
#define	P1_44_THR	0x04
#define	P1_44_GAP	0x08
#define	P1_16_LOL	0x10
#define	P2_50_SPD	0x20
#define	P2_50_DEN	0x40
/*
 * signals - Here are some signal values for known drives:
 */
#define	CIPHER	(P2_50_SPD)
#define	TELEX	(P2_50_SPD | P1_16_LOL | P1_44_THR | P1_36_GAP)

/*
 * retries - At power up, the board starts with one retry.  Let's retain this
 *	for the CONFIGURE within tmtopen.
 */
#define	DEF_RETRIES	0x1


/******************************************************************************/
/******
 * Non-Standard Command Block - Used for the NSC, NSCP, and NSRS commands.
 *	These commands are not used in this driver.
 *****/

struct NSCBLOCK {
#ifdef BFLY2
        u_short dummy;          /* two bytes padding here, rather than two */
	                        /* bytes padding between clength and sdptr! */
#endif BFLY2
	byte	bcount;		/* Byte Count */
	byte	clength;	/* Command Length */
	Iptr	sdptr;		/* Source/Destination Block Pointer */
	byte	cmd2;		/* Command 2 */
	byte	cmd1;		/* Command 1 */
};

/*
 * cmd1 - Command 1  (cmd1 in NSCBLOCK)
 */
#define	NSC_ERA		0x01	/* Erase */
#define	NSC_EDI		0x02	/* Edit */
#define	NSC_WFM		0x04	/* Write File Mark */
#define	NSC_WRT		0x08	/* Write */
#define	NSC_REV		0x10	/* Reverse */


/******************************************************************************/
/******
 * Ring Buffer Record - Used for the RBREAD, RBWRITE, WUR, and RUR commands.
 *	This is the block used by the controller at execution time.
 *****/

struct RBBLOCK {
	byte	reserved;	/* Reserved */
	byte	gate;		/* Gate */
	word	bcount;		/* Byte Count */
	Iptr	sdptr;		/* Source/Destination Pointer */
	Iptr	next;		/* Next Ring Buffer Record Pointer */
};

/*
 * gate - (Gate of Ring Buffer)  (gate in RBBLOCK)
 */
#define	RDY	0x01	/* Buffer Ready */
#define	BSY	0x02	/* Buffer Busy */
#define	CMP	0x04	/* Buffer Complete */
#define	ERR	0x08	/* Error */
#define	ABT	0x10	/* Abort Ring Buffer Operation */
#define	LST	0x20	/* Last Ring Buffer */
#define	INT	0x40	/* Ring Buffer Interrupt */


/******************************************************************************/
/******
 *  RBH - Ring Buffer Header.  This is used by the driver to keep information
 *	during execution of Ring Buffer commands.
 *****/
struct RBH {
	struct RBBLOCK	rb;		/* The actual Ring Buffer Block */
	struct RBH	*next;		/* Ptr to next RBH */
	struct buf	*bp;		/* Ptr to buffer for this operation */
	struct buf_struct *buffer;	/* ptr to device buffer */
	daddr_t		rb_blkno;	/* blkno for this operation */
	unsigned short	rb_flags;	/* Flagging special cases */
};

/* definitions for rb_flags */
#define RBINUSE 0x1		/* set if RBH in use during block i/o */

/******************************************************************************/
/******
 * TMTDEVICE - driver's per unit struct.
 *	This is used internally to the driver to keep track of information
 *	needed about the particular device.
 *****/
struct TMTDEVICE {
	word	dsr;		/* Drive Status Register
				 * (copy from struct tmtioreg)
				 */
	word	status;		/* Bits used as defined below. */
	daddr_t	blkno;		/* Best approx., drive is at this block */
};

/*
 * TMTDEVICE - Status bits  (status of struct TMTDEVICE)
 */
#define	USED	0x01		/* I/O was performed on unit */
#define	RAWDEV	0x02		/* used as raw device -- do positioning. */
#define	NOREW	0x04		/* no rewind device is open */
#define	WRITTEN	0x08		/* tape was written */
#define	NOP	0x10		/* no operation has been performed */


/******************************************************************************/
/******
 * IOPB	- Drivers internal structure for keeping track of what operations are
 *	presently going on.  These structures can be on the tmtaiopb list, in
 *	which case it is queued up to the controller and will be executed.
 *	They can be on the tmtclist in which case they will be queued for
 *	execution as soon as the current tmtaiopb list is finished.  And, they
 *	can be allocated but not yet queued; this would mean that a process
 *	is building up an operation.
 *****/
struct IOPB {
	struct IOPB	*next;		/* Pointer to next IOPB in list */
	struct TMTPB	pb;		/* actual PB used by the controller */
	int		status;		/* Status bits as defined below */
	daddr_t		blkno;		/* As close as we can tell, the tape */
					/* will be at this block number if */
					/* everything goes as planned. */
	struct RBH	*rbp;		/* Pointer to the RBH for this cmd */
	int		rbc;		/* Number RBH's queued in this IOPB */
	int		retries;	/* Number of retries for this cmd */
};

/*
 * IOPB status bits
 */
#define	PBINUSE		0x01	/* IOPB is being used */
#define	PBWANTED	0x02	/* IOPB wanted (only appears IOPB[0]->status) */
#define	CMD		0x04	/* IOPB is a command */
#define RETRY		0x08	/* IOPB is a retry */
#define SPECIAL		0x10	/* IOPB is from tmtintr() */

/* largest i/o is 63k, a little less than 8 pgs */
#define MAXPG	(((63*1024) + (NBPG-1))/NBPG)

/* number of retries allowed for failed operation */
#define MAX_RETRIES	10	/* max retries for any given IOPB operation */

struct buf_struct {
	caddr_t		bs_buffer;
	unsigned char	bs_alloc;
} buf_pool[MAXPG];

#define BSALLOC		0x01	/* this buffer is allocated */
#define BSWANTED	0x02	/* when somebody is waiting for buffers */
