#if	CMU
/* 
 **********************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 **********************************************************************
 */
 
#endif	CMU
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)callout.h	7.1 (Berkeley) 6/4/86
 */

#ifndef _CALLOUT_H
#define _CALLOUT_H
/*
 * The callout structure is for
 * a routine arranging
 * to be called by the clock interrupt
 * (clock.c) with a specified argument,
 * in a specified amount of time.
 * Used, for example, to time tab
 * delays on typewriters.
 */

#include "lock.h"

typedef struct {
	u_long		high_ticks;
	u_long		low_ticks;
} tick_t;

/* The callout handle must be exactly 32 bits long! */

typedef struct {
	short		callout_index;	/* index into callout table */
	short		node;		/* node on which callout lives */
} callout_handle_t;

typedef struct {
	tick_t			c_time;		/* time in absolute ticks */
	caddr_t			c_arg;		/* argument to routine */
	int			(*c_func)();	/* routine */
	callout_handle_t	* c_handlep;	/* pointer to handle */
} callout_t;

typedef struct {
	int			acallout;	/* numer of active callouts */
	tick_t			boot_ticks;	/* ticks since bootload */
	callout_t		** callout;	/* callout pointer array */
	simple_lock_data_t	callout_lock;	/* per queue lock */
} callout_info_t;

#ifdef KERNEL

#define	MAX_CALLOUT_INFO_SIZE	256

callout_info_t		* callout_info, * master_callout_info;

#define	NCALLOUT	((VM_PAGESIZE - MAX_CALLOUT_INFO_SIZE) /\
				(sizeof (callout_t) + sizeof (callout_t *)))

#endif KERNEL
#endif _CALLOUT_H
