/*ACI:	$Header: /nfs/europa/u1/BFMACH.local/mk/h/RCS/cluster.h,v 1.31 91/01/06 15:12:30 lkaplan Exp $	*/
/*
 *	File:	cluster.h
 *
 *	Copyright (C) 1987, BBN Advanced Computers
 *
*/

#ifndef _CLUSTER_H_		/* Only process this file once, even if */
#define _CLUSTER_H_		/* it is included indirectly more than once. */

#ifdef KERNEL
#include "cpus.h"		/* Since we use NCPUS */
#include "../h/param.h"
#include "../h/machine.h"
#else KERNEL
#include <sys/types.h>
#include <sys/param.h>
#include <sys/machine.h>
#endif KERNEL

#define add_nodes_to_cluster cluster_add_nodes
#define free_nodes_from_cluster cluster_free_nodes

typedef int	cluster_id_t; 
typedef	int	node_type_t; 
typedef	int	cluster_type_t; 
typedef short	cluster_prot_t;

/* cluster type bits */

#ifdef	BFLY1
#define	CLUSTER_NODE_TYPE_MASK	(NODE_TYPE_CPU|NODE_TYPE_VME)
#endif	BFLY1

#ifdef	BFLY2
#define CLUSTER_NODE_TYPE_MASK	(NODE_TYPE_SIZE_MASK|NODE_TYPE_SPEED_MASK|\
				NODE_TYPE_ICACHE_MASK|NODE_TYPE_DCACHE_MASK|\
				NODE_TYPE_CPU_VME_MASK)
#endif	BFLY2

#define	CLUSTER_NODE_HOME_MASK	(NODE_TYPE_CPU|NODE_TYPE_VME)

/* NOTE: The following cluster type bits must not overlap the node type
   bits defined in machine.h */

#define CLUSTER_BARE		0x00008000
#define CLUSTER_PERSIST		0x00004000
#define CLUSTER_DIVISIBLE	0x00002000
#define CLUSTER_INHIBIT_BORROW	0x00001000
#define	CLUSTER_INHIBIT_PAGEOUT	0x00000800

/* cluster protection bits */
#define CLUSTER_ACCESS_OWNER	0x4
#define CLUSTER_ACCESS_GROUP	0x2
#define CLUSTER_ACCESS_PUBLIC	0x1
#define CLUSTER_ACCESS_ALL	CLUSTER_ACCESS_OWNER|CLUSTER_ACCESS_GROUP \
					|CLUSTER_ACCESS_PUBLIC

/* id's of permanent clusters */
#define	SYSTEM_CLUSTER_ID	0
#define	FREE_CLUSTER_ID		1
#define	PUBLIC_CLUSTER_ID	2
#define IO_CLUSTER_ID		3
#define FREE_BPNE_ID		4
#define	FREE_BVME_ID		5

#define	NO_CLUSTER_ID		-2

/* definitions for library calls */

cluster_id_t getclusterbyname();
int setclustername(), addclustername();
char *getclustername();

#define CLUSTERNAMELEN 32

struct clusterent
{
char		cluster_name[CLUSTERNAMELEN];
cluster_id_t	cluster_id;
};

/* return structures for cluster_stat() */
struct	cluster_status1	{
  cluster_id_t	cluster_id;
  uid_t		owner_uid;	/* uid of owner */
  int		creator_pid;	/* pid of process which created cluster */
  int		process_count;	/* # of processes in the cluster */
  int		node_count;	/* # of nodes in the cluster */
};

struct home_node_data {
  short		home_pnn;	/* home node, within the cluster */
  cluster_id_t	home_cluster_id;/* home cluster id */
  short		system_pnn;	/* home node, within the whole system */
  short		physical_pnn;	/* home node, physical number */
};

union cluster_status {
  struct cluster_status1	status_list[MAX_CLUSTERS];
  short				node_list[MAX_CPUS];
  cluster_id_t			cluster_list[MAX_CLUSTERS];
  struct home_node_data		home;
};

/* return structures for cluster_ctl() */
struct cluster_config {
	int	max_clusters;	/* # of clusters configured into O/S */
	int	max_phys_cpus;	/* # of physical cpus configured */
	int	max_log_cpus;	/* # of logical cpus configured */
	int	cluster_info_size;    /* size (bytes) of cluster_info struct */
};

struct	cluster_info
{
cluster_id_t	cluster_id;
uid_t		owner_uid;	/* uid of owner */
gid_t		owner_gid;	/* gid of owner */
cluster_type_t	cluster_type;	/* type (e.g raw/mach nodes etc.) */
int		reclaim_pid;	/* pid of process which reclaims the cluster */
cluster_prot_t	cluster_prot;	/* owner/group/public access bits */
int		node_count;	/* # of nodes in the cluster */
int		process_count;	/* # of processes in the cluster */
int		use_count;	/* number of references to cluster */
};

union cluster_control {
  struct cluster_config		config;
  struct cluster_info		status_list[MAX_CLUSTERS];
  short				node_list[MAX_CPUS];
  cluster_id_t			cluster_list[MAX_CPUS];
  struct home_node_data		home;
  cluster_prot_t		cluster_prot;
  cluster_type_t		cluster_type;
  gid_t				cluster_gid;
  int				reclaim_pid;
  int				delta;
  int				fork_node_strategy;
  cpu_subtype_t			node_type_list[MAX_CPUS];
};

/* convenient parameters to certain cluster system calls */
#define BEST_PNN	-1
#define	HOME_CLUSTER	-1
#define DONT_KILL_PROCESSES  0
#define KILL_PROCESSES  1

/* function types for the cluster_ctl system call */

#define	GET_NODE_LIST		0	/* list of nodes in the cluster */
#define	GET_CLUSTER_STATUS	1	/* status of an single cluster */
#define	GET_PNN_MAP		2	/* map of logical pnn to cluster id */
#define	GET_CLUSTER_MAP		2	/* map of logical pnn to cluster id */
#define GET_PHYS_PNN_MAP	3	/* map of logical pnn to phys pnn */
#define GET_HOME_NODE		4	/* home node structure */
#define GET_CLUSTER_STATUS_LIST	5	/* list of all clusters */
#define GET_HOME_CLUSTER_MAP	6	/* map of logical pnn to home cluster */
#define SET_PROT_BITS		7	/* set owner/group/public bits */
#define SET_CLUSTER_FLAGS	8	/* set divisible/persist bits */
#define CLEAR_CLUSTER_FLAGS	9	/* clear divisible/persist bits */
#define SET_HOME_CLUSTER	10	/* set home cluster for node list */
#define SET_CLUSTER_RECLAIM	11	/* change the cluster reclaim pid */
#define SET_CLUSTER_GID		12	/* change the owner gid of cluster */
#define INC_USE_COUNT		13	/* atomic modify cluster use count */
#define	SET_FORK_NODE_STRATEGY	14	/* set fork choose node strategy */
#define	GET_NODE_TYPE_LIST	15	/* list of logical node types */
#define GET_CLUSTER_CONFIG	16	/* get system configuration */

/* cluster_fork_choose_node strategies -- see local_load() */
#define LOADCALC        	0   /* default */
#define ROUNDROBIN      	1
#define PROCCOUNT       	2
#define MAX_NODE_STRATEGY       2

#ifdef KERNEL

/*	cluster definitions 	*/

/*	# of core cpus in the public cluster */
#define	CORE_CPUS	1

#define	RECLAIM_DEAD	-1

#define	NEXT_CLUSTER_ID_STEP	0x100
#define	CLUSTER_INDEX_MASK	(NEXT_CLUSTER_ID_STEP - 1)
#define	SYSTEM_CLUSTER_INDEX	(SYSTEM_CLUSTER_ID & CLUSTER_INDEX_MASK)
#define	FREE_CLUSTER_INDEX	(FREE_CLUSTER_ID & CLUSTER_INDEX_MASK)
#define	PUBLIC_CLUSTER_INDEX	(PUBLIC_CLUSTER_ID & CLUSTER_INDEX_MASK)
#define	IO_CLUSTER_INDEX	(IO_CLUSTER_ID & CLUSTER_INDEX_MASK)
#define	FREE_BPNE_INDEX		(FREE_BPNE_ID & CLUSTER_INDEX_MASK)
#define	FREE_BVME_INDEX		(FREE_BVME_ID & CLUSTER_INDEX_MASK)
#define LAST_SYSTEM_INDEX	FREE_BVME_INDEX
#define	FIRST_PRIVATE_INDEX	(LAST_SYSTEM_INDEX+1)
#define	CLUSTER_INDEX(cluster_id)	((cluster_id) & CLUSTER_INDEX_MASK)

struct	cluster_block
{ /* per-cluster data */
  int			round_robin;	/* old scheduling hint */
  int			process_count;	
  int			ext_forking;	/* process forking into cluster
					   from outside of cluster */
  uid_t			owner_uid;
  gid_t			owner_gid;
  int			reclaim_pid;
  cluster_id_t		cluster_id;
  cluster_type_t	cluster_type;	/* includes node types */
  int			use_count;	/* user synchronization */
  int			ref_count;	/* # of vm objects ref'ing cluster */
  cluster_prot_t	cluster_prot;
  int			node_strategy;	/* strategy for fork choosing a node */
  short			node_count;
  short			node_list[NCPUS];	/* keep this last */
};

typedef struct cluster_block	cluster_block_t;

struct	cluster_configuration {	/* per-system list of clusters */
	simple_lock_data_t master_cluster_lock;
	cluster_id_t	next_cluster_id;
	cluster_id_t	pnn_to_cluster_map[NCPUS];
	cluster_id_t	pnn_to_home_map[NCPUS];
};

#define HOME_CLUSTER_ID()	(current_proc()->p_home_cluster_id)

struct	cluster_configuration	*cluster_config;
struct  cluster_block		*cluster_table;

/*	returns 1 if cluster is mapped by some mach process 0 otherwise
 *	caller needs to hold the master cluster lock
 */
#define is_cluster_mapped(cur_cluster) (cur_cluster->ref_count)

#endif KERNEL
#endif _CLUSTER_H_
