/* $Header: /nfs/europa/u1/BFMACH.local/mk/h/RCS/domain.h,v 1.13 90/10/15 08:35:00 tappan Exp $ */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)domain.h	7.3 (Berkeley) 6/27/88
 */

#ifdef	KERNEL
#include "lock.h"
#else	KERNEL
#include <sys/lock.h>
#endif	KERNEL

/*
 * Structure per communications domain.
 */
struct	domain {
	int	dom_family;		/* AF_xxx */
	char	*dom_name;
	int	(*dom_init)();		/* initialize domain data structures */
	int	(*dom_externalize)();	/* externalize access rights */
	int	(*dom_dispose)();	/* dispose of internalized rights */
	struct	protosw *dom_protosw, *dom_protoswNPROTOSW;
	struct	domain *dom_next;
	u_char	dom_isserial;		/* if 1, then use lock serialization */
	u_char	dom_islocked;		/* if 1, then the domain is busy */
	u_char	dom_lockiswanted;	/* if 1, then someone wants the lock */
	u_char	dom_parallelized;	/* domain code has been parallelized */
				/* Note that 'isserial' and 'parallelized'
				 * are not the same. The code may have been
				 * parallelized, but it can be forced to run
				 * in serial if any network interfaces use
				 * the if_server.
				 */
	int	dom_softintwaiting;	/* == type if softint wants lock */
	simple_lock_data_t dom_lock;	/* domain acquisition lock */
};

#define	PARALLEL_DOM	0
#define	SERIAL_DOM	1

#ifdef KERNEL
#include "../net/netisr.h"
#include "../machine/cpu.h"

struct	domain *domains;
extern int dom_debug;
extern int netint_active;	/* VECTORIZED */

/*
 * This macro is called at software interrupt time to try and
 * acquire the domain lock. If the domain is busy, then the
 * software interrupt type is set in netisr, the sofware
 * interrupt will be rescheduled when the domain lock is
 * released, and a 0 is returned in gotlock.
 * Otherwise, the lock is taken and a 1 is returned in gotlock.
 *
 * Set the vectorized flag indicating that we are in the interrupt context
 * so that the if_output server can detect whether it should sleep.
 */
#define	DOM_SOFTINT_LOCK(dom, gotlock, type) { \
	int s; \
	if ((dom)->dom_isserial) {\
		s = splhigh(); \
		simple_lock(&(dom)->dom_lock); \
		if ((dom)->dom_islocked) { \
			gotlock = 0; \
			(dom)->dom_softintwaiting = type; \
		} else { \
			(dom)->dom_islocked = 1; \
			gotlock = 1; \
			netint_active++; \
			(dom)->dom_softintwaiting = 0; \
		} \
		simple_unlock(&(dom)->dom_lock); \
		if (dom_debug) \
			printf("DOM_SOFTINT_LOCK: (%s) %s\n", (dom)->dom_name, \
			       gotlock ? "locked" : "resched"); \
		splx(s); \
	} else { \
		gotlock = 1; \
		netint_active++; \
	} \
}

/*
 * This macro is called at software interrupt time to relinquish the
 * lock on a domain. If the domain is wanted, then a wakeup is performed.
 * If a software interrupt is waiting for the lock, then reschedule it.
 *
 * Clear the vectorized flag indicating that we are no longer in the
 * interrupt context.
 */
#define	DOM_SOFTINT_UNLOCK(dom) { \
	int s; \
	if ((dom)->dom_isserial) { \
		s = splhigh(); \
		if (dom_debug) \
			printf("DOM_SOFTINT_UNLOCK: (%s)\n", (dom)->dom_name); \
		simple_lock(&(dom)->dom_lock); \
		if ((dom)->dom_islocked == 0) \
			panic("DOM_SOFTINT_UNLOCK"); \
		(dom)->dom_islocked = 0; \
		if (netint_active <= 0) \
			panic("DOM_SOFTINT_UNLOCK: netint_active"); \
		netint_active--; \
		if ((dom)->dom_lockiswanted) { \
			(dom)->dom_lockiswanted = 0; \
			wakeup((caddr_t)&(dom)->dom_lockiswanted); \
		} \
		simple_unlock(&(dom)->dom_lock); \
		splx(s); \
	} else { \
		if (netint_active <= 0) \
			panic("DOM_SOFTINT_UNLOCK: netint_active"); \
		netint_active--; \
	} \
}

/*
 * This macro is called at timeout time to try and acquire
 * the domain lock. If the domain is busy, then the a 0 is
 * returned in gotlock. Otherwise, the lock is taken and a 1
 * is returned in gotlock.
 */
#define	DOM_TIMEOUT_LOCK(dom, gotlock) { \
	int s; \
	if ((dom)->dom_isserial) { \
		s = splhigh(); \
		simple_lock(&(dom)->dom_lock); \
		if ((dom)->dom_islocked) { \
			gotlock = 0; \
		} else { \
			(dom)->dom_islocked = 1; \
			gotlock = 1; \
		} \
		simple_unlock(&(dom)->dom_lock); \
		splx(s); \
	} else { \
		gotlock = 1; \
	} \
}

/*
 * This macro is called to relinquish the lock on a domain.
 * If the domain is wanted, then a wakeup is performed.
 * This routine is identical to DOM_UNLOCK, except that
 * there is no debugging. This is because timeouts happen
 * frequently and continuously.
 * If a software interrupt is waiting for the lock, then
 * reschedule it.
 */
#define	DOM_TIMEOUT_UNLOCK(dom) { \
	if ((dom)->dom_isserial) { \
		int s = splnet(), l; \
		simple_lock(&(dom)->dom_lock); \
		if ((dom)->dom_islocked == 0) \
			panic("DOM_TIMEOUT_UNLOCK"); \
		(dom)->dom_islocked = 0; \
		if (l = (dom)->dom_softintwaiting) {\
		    schednetisr(l); \
		   (dom)->dom_softintwaiting = 0; \
		} \
		if ((dom)->dom_lockiswanted) { \
			(dom)->dom_lockiswanted = 0; \
			wakeup((caddr_t)&(dom)->dom_lockiswanted); \
		} \
		simple_unlock(&(dom)->dom_lock); \
		splx(s); \
	} \
}

/*
 * This macro is called to relinquish the lock on a domain.
 * If the domain is wanted, then a wakeup is performed. If
 * a software interrupt is waiting for the lock, then
 * reschedule it.
 */
#define	DOM_UNLOCK(dom) { \
	if ((dom)->dom_isserial) { \
           	int s = splnet(), l; \
		if (dom_debug) \
			printf("DOM_UNLOCK: (%s)\n", (dom)->dom_name); \
		simple_lock(&(dom)->dom_lock); \
		if ((dom)->dom_islocked == 0) \
			panic("DOM_UNLOCK"); \
		(dom)->dom_islocked = 0; \
		if (l = (dom)->dom_softintwaiting) { \
                    schednetisr(l); \
		    (dom)->dom_softintwaiting = 0; \
		}\
		if ((dom)->dom_lockiswanted) { \
			(dom)->dom_lockiswanted = 0; \
			wakeup((caddr_t)&(dom)->dom_lockiswanted); \
		} \
		simple_unlock(&(dom)->dom_lock); \
		splx(s); \
	} \
}

/*
 * Flags indicating if the DOM_LOCK should be taken or not.
 */
#define	DO_DOM_LOCK	0
#define	DONT_DOM_LOCK	1

/*
 * This macro performs the pr_usrreq while maintaining serial
 * access to the given domain. The return value from the call
 * is placed in rval.
 */
#define PR_USRREQ(so, req, m, nam, rights, rval) { \
	DOM_LOCK((so)->so_proto->pr_domain); \
	(rval) = \
	    (*(so)->so_proto->pr_usrreq)((so), (req), (m), (nam), (rights)); \
	DOM_UNLOCK((so)->so_proto->pr_domain); \
}

/*
 * This macro performs the pr_ctloutput while maintaining serial
 * access to the given domain. The return value from the call
 * is placed in rval.
 */
#define PR_CTLOUTPUT(req, so, level, nam, rights, rval) { \
	DOM_LOCK((so)->so_proto->pr_domain); \
	(rval) = \
	    (*(so)->so_proto->pr_ctloutput)((req), (so), (level), (nam), \
					    (rights)); \
	DOM_UNLOCK((so)->so_proto->pr_domain); \
}

/*
 * This macro performs the dom_externalize while maintaining serial
 * access to the given domain. The return value from the call
 * is placed in rval.
 */
#define DOM_EXTERNALIZE(so, rights, rval) { \
	DOM_LOCK((so)->so_proto->pr_domain); \
	(rval) = (*(so)->so_proto->pr_domain->dom_externalize)((rights)); \
	DOM_UNLOCK((so)->so_proto->pr_domain); \
}

/*
 * This macro performs the dom_dispose while maintaining serial
 * access to the given domain. The return value from the call
 * is placed in rval.
 */
#define DOM_DISPOSE(so, mb, rval) { \
	DOM_LOCK((so)->so_proto->pr_domain); \
	(rval) = (*(so)->so_proto->pr_domain->dom_dispose)((mb)); \
	DOM_UNLOCK((so)->so_proto->pr_domain); \
}
#endif
