/* $Header: /nfs/columbia/u1/BFMACH.local/mk/h/RCS/lock.h,v 1.12 90/12/18 10:23:53 tappan Exp $ */
/*
 ****************************************************************
 * Mach Operating System
 * Copyright (c) 1986 Carnegie-Mellon University
 *  
 * This software was developed by the Mach operating system
 * project at Carnegie-Mellon University's Department of Computer
 * Science. Software contributors as of May 1986 include Mike Accetta, 
 * Robert Baron, William Bolosky, Jonathan Chew, David Golub, 
 * Glenn Marcy, Richard Rashid, Avie Tevanian and Michael Young. 
 * 
 * Some software in these files are derived from sources other
 * than CMU.  Previous copyright and other source notices are
 * preserved below and permission to use such software is
 * dependent on licenses from those institutions.
 * 
 * Permission to use the CMU portion of this software for 
 * any non-commercial research and development purpose is
 * granted with the understanding that appropriate credit
 * will be given to CMU, the Mach project and its authors.
 * The Mach project would appreciate being notified of any
 * modifications and of redistribution of this software so that
 * bug fixes and enhancements may be distributed to users.
 *
 * All other rights are reserved to Carnegie-Mellon University.
 ****************************************************************
 */

#ifndef	_LOCK_
#define	_LOCK_

#ifdef	KERNEL
#include "cpus.h"
#include "../h/boolean.h"
#else	KERNEL
#include <sys/features.h>
#include <sys/boolean.h>
#endif	KERNEL

#define	SIMPLE_LOCK	0x10000

struct slock {
	int      lock_data;		/* 1 in the high word (SIMPLE_LOCK) */
#ifdef BFLY1
	unsigned long lock_physaddr; /* optimization for certain locks ONLY */
#endif BFLY1
	/* The following are used for debugging only, but are
	 * always present, because the size of a lock structure
	 * must be constant (see proc.h)
	 */
	int	status;
	int	caller;
	int	caller_caller;
	int	cpu_id;
	struct proc	*proc;	/* jam thread address here */
};

typedef struct slock	simple_lock_data_t;
typedef struct slock	*simple_lock_t;

#if	NCPUS > 1
void		simple_lock_init();
void		simple_lock();
void		simple_unlock();
boolean_t	simple_lock_try();

#else	NCPUS > 1

/*
 *	No multiprocessor locking is necessary.
 */
#define simple_lock_init(l)
#define simple_lock(l)
#define simple_unlock(l)
#define simple_lock_try(l)	(1)	/* always succeeds */
#endif	NCPUS > 1

/* The following structure is used in sleep_and_unlock */

struct usw_block
{
  int status ;
  unsigned wchan ;
} ;

/*
 * read/write locks.
 */

/* MAXREADER is a large number, intended to be larger than any 
 * possible concurrent numbers of readers.
 *
 * Note: There are three possible ways a process may be waiting
 * on a rw_lock.
 *   1. Reader waiting for writers to finish. Wait channel = lp->count
 *   2. Writer waiting for readers/writers to finish. chan = lp->writers
 *   3. reader trying to promote to writer. chan = lp->flags
 *
 * rwlocks are "writer priority" which means that as soon as a process
 * has registered a desire for exclusive access, subsequent readers 
 * will sleep.
 */
#define MAXREADERS 0xffffff

typedef struct {
    simple_lock_data_t	latch;
    unsigned		count;	/* count of available reader slots */
    unsigned		writers; /* count of active or waiting writers */
    unsigned short	flags;	/* flags */
#define RW_WWAIT	0x0000001 /* reader waiting on writers */
#define RW_PWAIT	0x0000002 /* promoter waiting */
    unsigned short	pri;	/* sleep priority */
    unsigned		writer;	/* proc of writer */
    unsigned		caller;	/* caller of writer */
} rwlock_data_t, *rwlock_t;

#endif	_LOCK_
