/* $Header: /nfs/europa/u1/BFMACH.local/mk/h/RCS/socketvar.h,v 1.17 90/11/14 10:10:06 tappan Exp $ */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)socketvar.h	7.4 (Berkeley) 6/27/88
 */

#ifdef	KERNEL
#include "lock.h"
#else	KERNEL
#include <sys/lock.h>
#endif	KERNEL

/*
 * Socket state bits.
 *
 * In order to maintain atomicity without sacrificing performance,
 * we allocate a character per flag for the socket state information.
 */
struct sstate {
	u_char SS_NOFDREF;		/* no file table ref any more */
	u_char SS_ISCONNECTED;		/* socket connected to a peer */
	u_char SS_ISCONNECTING;		/* in process of connecting to peer */
	u_char SS_ISDISCONNECTING;	/* in process of disconnecting */
	u_char SS_CANTSENDMORE;		/* can't send more data to peer */
	u_char SS_CANTRCVMORE;		/* can't receive more data from peer */
	u_char SS_RCVATMARK;		/* at mark on input */
	u_char SS_PRIV;			/* privileged for broadcast, raw... */
	u_char SS_NBIO;			/* non-blocking ops */
	u_char SS_ASYNC;		/* async i/o notify */
};

/*
 * Socket queue state.
 *
 * These are used to indicate whether new connections are
 * to be allowed, etc.
 */
#define	SQ_DONT_ACCEPT	0		/* don't allow enqueues */
#define	SQ_ACCEPTCONN	1		/* allow enqueues (SO_ACCEPTCONN) */

/*
 * Kernel structure per socket.
 * Contains send and receive buffer queues,
 * handle on protocol and pointer to protocol
 * private data and error information.
 */
struct socket {
	u_char	so_islocked;		/* if 1, socket is locked */
	u_char	so_lockiswanted;	/* if 1, someone wants the lock */
	short	so_type;		/* generic type, see socket.h */
	short	so_options;		/* from socket call, see socket.h */
	short	so_linger;		/* time to linger while closing */
	struct sstate so_state;		/* internal state flags SS_*, above */
	caddr_t	so_pcb;			/* protocol control block */
	struct	protosw *so_proto;	/* protocol handle */
/*
 * Variables for connection queueing.
 * Socket where accepts occur is so_head in all subsidiary sockets.
 * If so_head is 0, socket is not related to an accept.
 * For head socket so_q0 queues partially completed connections,
 * while so_q is a queue of connections ready to be accepted.
 * If a connection is aborted and it has so_head set, then
 * it has to be pulled out of either so_q0 or so_q.
 * We allow connections to queue up based on current queue lengths
 * and limit on number of queued connections for this socket.
 */
	struct	socket *so_head;	/* back pointer to accept socket */
	struct	socket *so_q0;		/* queue of partial connections */
	struct	socket *so_q;		/* queue of incoming connections */
	u_char	so_qstate;		/* queue state SQ_*, above */
	short	so_q0len;		/* partials on so_q0 */
	short	so_qlen;		/* number of connections on so_q */
	short	so_qlimit;		/* max number queued connections */
	short	so_timeo;		/* connection timeout */
	u_short	so_error;		/* error affecting connection */
	short	so_pgrp;		/* pgrp for signals */
	u_long	so_oobmark;		/* chars to oob mark */
/*
 * Variables for socket buffering.
 */
	struct	sockbuf {
		u_long	sb_cc;		/* actual chars in buffer */
		u_long	sb_hiwat;	/* max actual char count */
		u_long	sb_mbcnt;	/* chars of mbufs used */
		u_long	sb_mbmax;	/* max chars of mbufs to use */
		u_long	sb_lowat;	/* low water mark */
		struct	mbuf *sb_mb;	/* the mbuf chain */
		struct	proc *sb_sel;	/* process selecting read/write */
		short	sb_timeo;	/* timeout */
		u_char  SB_WAKEONE;	/* do wakeup_one when data available */
		u_char  SB_AVAIL; 	/* available flag byte */
		u_char	SB_SEND;	/* someone is in sosend/PR_USRREQ */
		u_char	SB_WAIT;	/* someone is waiting for data/space */
		u_char	SB_SEL;		/* buffer is selected */
		u_char	SB_COLL;	/* collision selecting */
		/* The following variable provides an interlock 
		 * between sofree() and the timeout routine sosendwakeup()
		 */
		int    SB_TIMEOUTACTIVE; /* Count of pending callouts */
		int    sb_spl;		/* saved spl for lock */
		simple_lock_data_t sb_lock;	/* multiprocessor lock */
	} so_rcv, so_snd;
#define	SB_MAX		(64*1024)	/* max chars in sockbuf */
#define	SB_DEFTIMEO	10		/* default sb_timeo value */
};

/*
 * Flags to sofree(), indicating why we think the socket can now be freed
 */
#define SOFREE_PCBGONE	0x01	/* Protocol control block being released */
#define SOFREE_NOFDREF	0x02	/* No more file references */
#define SOFREE_DOMLOCK	0x04	/* Domain lock is held */

/*
 * flags to sorflush
 */
#define SORFLUSH_SOCKLOCK	0x01 /* socket lock is held */
#define SORFLUSH_DOMLOCK	0x02 /* domain lock is held */

/*
 * Macros for sockets and socket buffering.
 */

/* how much space is there in a socket buffer (so->so_snd or so->so_rcv) */
#define	sbspace(sb) \
    (MIN((long)((sb)->sb_hiwat - (sb)->sb_cc),\
	 (long)((sb)->sb_mbmax - (sb)->sb_mbcnt)))

/* do we have to send all at once on a socket? */
#define	sosendallatonce(so) \
    ((so)->so_proto->pr_flags & PR_ATOMIC)

/* can we read something from so? */
#define	soreadable(so) \
    ((so)->so_rcv.sb_cc || (so)->so_state.SS_CANTRCVMORE || \
	(so)->so_qlen || (so)->so_error)

/* can we write something to so? */
#define	sowriteable(so) \
    (sbspace(&(so)->so_snd) > 0 && \
	((so)->so_state.SS_ISCONNECTED || \
	  ((so)->so_proto->pr_flags&PR_CONNREQUIRED)==0) || \
     (so)->so_state.SS_CANTSENDMORE || \
     (so)->so_error)

/* adjust counters in sb reflecting allocation of m */
#define	sballoc(sb, m) { \
	(sb)->sb_cc += (m)->m_len; \
	(sb)->sb_mbcnt += MSIZE; \
	if ((m)->m_off > MMAXOFF) \
		(sb)->sb_mbcnt += MCLBYTES; \
}

/* adjust counters in sb reflecting freeing of m */
#define	sbfree(sb, m) { \
	(sb)->sb_cc -= (m)->m_len; \
	(sb)->sb_mbcnt -= MSIZE; \
	if ((m)->m_off > MMAXOFF) \
		(sb)->sb_mbcnt -= MCLBYTES; \
}

/* SOLOCK is a routine in sys/uipc_socket2.c */
/* release lock on socket so */
#define	SOUNLOCK(so) { \
	int s = splnet(); \
	simple_lock(&socket_lock); \
	if ((so)->so_islocked == 0) \
		panic("SOUNLOCK"); \
	(so)->so_islocked = 0; \
	if ((so)->so_lockiswanted) { \
		(so)->so_lockiswanted = 0; \
		wakeup((caddr_t)&(so)->so_lockiswanted); \
	} \
	simple_unlock(&socket_lock); \
	splx(s); \
}

/*
 * Socket Buffer locking definitions.
 */
#define	SLSB(sb) { \
	int sb_spl = splnet(); \
	simple_lock(&(sb)->sb_lock); \
	(sb)->sb_spl = sb_spl; \
}
#define	SUSB(sb) { \
	int sb_spl = (sb)->sb_spl; \
	simple_unlock(&(sb)->sb_lock); \
	splx(sb_spl); \
}

/*
 * Socket Queue manipulation locking definitions.
 */
#define	SLSQ()		simple_lock(&socket_lock)
#define	SUSQ()		simple_unlock(&socket_lock)

#define	sorwakeup(so)	sowakeup((so), &(so)->so_rcv)
#define	sowwakeup(so)	sowakeup((so), &(so)->so_snd)

#ifdef KERNEL
struct	socket *sonewconn();
extern	simple_lock_data_t socket_lock;
#endif
