/* $Header: /nfs/gemini/u1/BFMACH.local/mk.gibson/h/RCS/termios.h,v 1.3 90/04/03 11:18:37 gibson Exp $ */
/*
 * Copyrighted as an unpublished work.
 * (c) Copyright 1989 INTERACTIVE Systems Corporation
 * All rights reserved.
 *
 * RESTRICTED RIGHTS
 *
 * These programs are supplied under a license.  They may be used,
 * disclosed, and/or copied only as permitted under such license
 * agreement.  Any copy must contain the above copyright notice and
 * this restricted rights notice.  Use, copying, and/or disclosure
 * of the programs is strictly prohibited unless otherwise provided
 * in the license agreement.
 */


/*
 * termios structure.
 */
#ifndef _TERMIOS_H
#define _TERMIOS_H
#ifdef KERNEL
#include "../h/posix.h"
#else KERNEL
#include <sys/posix.h>
#endif KERNEL

#define	NCC	11
#define	NCCS	NCC		/* Number of c_cc characters */

#ifdef _BBN_POSIX_SUPPORT
/* control characters */
#define	VEOF	0
#define	VEOL	1
#define	VERASE	2
#define	VINTR	3
#define	VKILL	4
#define	VMIN	5
#define	VQUIT	6
#define	VSUSP	7
#define	VTIME	8
#define	VSTART	9
#define	VSTOP	10	 
#endif _BBN_POSIX_SUPPORT

/*
 * Types.
 */
typedef	unsigned int	tcflag_t;	/* Terminal control flag type */
typedef	unsigned char	cc_t;		/* Control char type */
typedef		 char	speed_t;	/* Speed type	*/

struct termios	{
	tcflag_t	c_iflag;	/* Input modes	*/
	tcflag_t	c_oflag;	/* Output modes	*/	
	tcflag_t	c_cflag;	/* Control modes */
	tcflag_t	c_lflag;	/* Local modes	*/
	cc_t		c_cc[NCCS];	/* Control chars */
	speed_t		c_ispeed;	/* Input speed	*/
	speed_t		c_ospeed;	/* Ouptut speed	*/
};

#ifdef _BBN_POSIX_SUPPORT
/*
 * Input modes
 */

#define	BRKINT	00000001	/* Signal interrupt on break */
#define	ICRNL	00000002	/* Map CR to NL on input */
#define	IGNBRK	00000004	/* Ignore break condition */
#define	IGNCR	00000010	/* Ignore CR */
#define	IGNPAR	00000020	/* Ignore chars with parity errors */
#define	INLCR	00000040	/* Map NL to CR on input */
#define	INPCK	00000100	/* Enable input parity check */
#define	ISTRIP	00000200	/* Strip character */
#define	IXOFF	00000400	/* Enable start/stop input control */
#define	IXON	00001000	/* Enable start/stop output control */
#define	PARMRK	00002000	/* Mark parity errors */

/*
 * Ouput Modes
 */

#define	OPOST	00000001	/* Perform output processing */

/*
 * Control Modes
 */

#define	CLOCAL	00000001	/* Ignore modem status line */
#define	CREAD	00000002	/* Enable receiver	*/
#define	CSIZE	00000170	/* Number of bits per byte */
#define	CS5	00000010	/* 5 bits */
#define	CS6	00000020	/* 6 bits */
#define	CS7	00000040	/* 7 bits */
#define	CS8	00000100	/* 8 bits */
#define	CSTOPB	00000200	/* Send two stop bits, else one */
#define	HUPCL	00000400	/* Hang up on last close */
#define	PARENB	00001000	/* Parity enable */
#define	PARODD	00002000	/* Odd parity, else even */

/*
 * Local modes		### some common with sys/ioctl.h ###
 */

#define	ECHOE	0x00000002	/* Echoe ERASE */
#define	ECHOK	0x00000004	/* Echo KILL */
#ifndef ECHO
#define	ECHO	0x00000008	/* Enable echo */
#endif
#define	ECHONL	0x00000010	/* Echo '\n' */
#define	ICANON	0x00000020	/* Canonincal input */
#define	IEXTEN	0x00000040	/* Enable extended functions */
#define	ISIG	0x00000100	/* Enable signals */
#ifndef NOFLSH
#define	NOFLSH	0x80000000	/* Disable flush after INT, QUIT, or SUSP */
#endif
#ifndef TOSTOP
#define	TOSTOP	0x00400000	/* Sendt ISGTTOU for background output */
#endif

/*
 * Speed modes
 */
/* these are defined by sys/ttydev.h */
#ifdef NEVER
#define B0      	0	/* Hang up */
#define B50     	1	/* 50 baud */
#define B75     	2
#define B110    	3
#define B134    	4
#define B150    	5
#define B200    	6
#define B300    	7
#define B600    	8	/* ... */
#define B1200   	9
#define B1800   	10
#define B2400   	11
#define B4800   	12
#define B9600   	13
#define B19200   	14
#define B38400   	15
#endif
#ifdef KERNEL
#include "../h/ttydev.h"
#else KERNEL
#include <sys/ttydev.h>
#endif KERNEL

/*
 * tcsetattr() optional actions.
 */
#define	TCSANOW		0	/* Immediately */
#define	TCSADRAIN	1	/* First drain output */
#define	TCSAFLUSH	2	/* First drain output and flush input */
#define	TCSADFLUSH	2	/* First drain output and flush input */

/*
 * tcflush() queue_selector values.
 */
#define	TCIFLUSH	0	/* Flush data received but not read */
#define	TCOFLUSH	1	/* Flush data written but not transmitted */
#define	TCIOFLUSH	2	/* Flush both data received and written */

/*
 * tcflow() action values.
 */
#define	TCOOFF		0	/* Suspend output */
#define	TCOON		1	/* Restart suspended output */
#define	TCIOFF		2	/* Transmit a STOP character */
#define	TCION		3	/* Transmit a START character */

/*
extern	speed_t	cfgetospeed();
extern	speed_t	cfsetospeed();
extern	speed_t	cfgetispeed();
extern	speed_t	cfsetispeed();
extern	int	tcgetattr();
extern	int	tcsetattr();
extern	int	tcsendbreak();
extern	int	tcdrain();
extern	int	tcflush();
extern	int	tcflow();
extern	pid_t	tcgetpgrp();
extern	int	tcsetpgrp();
*/
#endif _BBN_POSIX_SUPPORT

#endif _TERMIOS_H
