#ifndef usd_h
#define usd_h yes
/* outside interface to USD data structures */

/* this file should parallel usd.a68.h, any changes should be made to both */

/* jump table definitions (offsets of routine pointers from eprom) */
#define uputc_addr	0xc	/* (C) write char to console or host */
#define ugetc_addr	0x10	/* (C) get character from console or host */
#define usync_addr	0x14	/* (C) flush output - currently doesn't work */
#define upoll_addr	0x18	/* (C) get char or -1 from console or host */
#define usetq_addr	0x1c	/* (C) set Chrysalis console & host input queues */
#define ureferr_addr	0x20	/* bus/addr error entry to USD when USD on */
#define uqinit_addr	0x24	/* init dual queue header (a0->header, a1=buf) */
#define uPMMUset_addr	0x28	/* (C) change PMMU setup utility routine */
#define uexc_addr	0x2c	/* exception entry to USD (except frame on stk) */
#define uerr_addr	0x30	/* Chrysalis crash entry to USD */
#define uhpoke_addr	0x34	/* poke USD host output driver */
#define ucpoke_addr	0x38	/* poke USD console output driver */
#define uwake_addr	0x3C	/* wakeup USD */
#define umboot_addr 0x40	/* reenter Mach bootstrap	 */
#define uaboot_addr 0x44	/* address of autoboot flag	 */
#define ubparm_addr 0x48	/* address of boot parameter block */


/* async driver control block structure (ADCB) */
/* NOTE: there is currently no way to get at ADCBs outside USD */
struct adcb {
    char * dev_addr;		/* base address of device (must be in global sar) */
    long intf_code;		/* 4 byte code to or into input bytes */
    QH input_QH;		/* input data put on this queue */
    QH output_QH;		/* output data taken from this queue */
    int (*input_done)();	/* call this routine after enqueuing input */
    int (*out_poke)();		/* call this routine to wake up output driver */
    int (*intcpt_rt)();		/* call this routine if input=intercept char */
    char intcpt_ch;		/* intercept character (normally ^C) */
    char tty_mode;		/* flags word for raw mode, flow control, etc. */
#	define TTY_RAW 1	/* bit for raw mode */
#	define TTY_FLOW 2	/* bit for flow control (1 if flow control) */
#	define TTY_STOP 128	/* bit set by ^S flow control to stop output */
};


/* USD per-port data structure */
/*  Other code should not be interested in this structure except for */
/*    u_adcb, u_inq, u_outq, and u_poke */

#define OBUF_SIZE 60
#define IBUF_SIZE 80

struct USD_data {
    short u_flags;		/* USD internal state */
    short u_mode;		/* USD I/O mode control */
    short u_mode2;		/* USD next I/O mode */
    short u_asar;		/* USD simulated AsAr value */
    char * u_olocn;		/* current open location */
    struct adcb * u_adcb;	/* address of ADCB for I/O (may be NULL) */
    char u_cksum;		/* checksum sum */
    char u_seq;			/* command sequence number */
    int (*u_poke)();		/* output poke routine address */
    short u_inarg;		/* # bits left in current arg */
    short u_argn;		/* current arg number */
    long u_arg0;		/* seq#, checksum arg */
    long u_argt[16];		/* USD command args */
#   if BF_PLUS
    short u_pad;		/* pad queues to longword align */
#   endif BF_PLUS
    QH u_inq;			/* USD's input character queue */
    QH u_outq;			/* USD's output goes to this queue */
    long i_buf[IBUF_SIZE];	/* input queue buffer area */
    long o_buf[OBUF_SIZE];	/* output queue buffer area */
};


/* information about this PN */
struct node_info_struct {
    short node_num;		/* node number (=Proc_Num) */
    short node_type;		/* type of node */
#	define PN68000 0	/* normal 68000 node (current BPN2) */
#	define PN68010 1	/* BPN2 with 68010 processor */
#	define PN68020 2	/* BPN2 with 68020 processor daughterboard */
#	define PNFP020 3	/* BPN2 with 68020 and 68881 floating point */
#	define BVME000 0x80     /* BVME node type */

    char sw_cols;		/* # columns in switch */
    char sw_config;		/* setting for switch PNC control reg, includes */
				/*  alternate paths & checksum */
    long owner;			/* (CHRYS) future use for partitioned machine */
    short u_version;		/* microcode version number (copied from PNC) */
    short os_version;		/* (CHRYS) Chrysalis version number */
    short memsize;		/* total 64k banks of memory in node */
#if BPN2
    char membanks[8];		/* 64 bit table of existing memory banks */
    char memgood[8];		/* 64 bit table of working memory banks */
#else BPN2
    char membanks[32];		/* 256 bit table of existing memory banks */
    char memgood[32];		/* 256 bit table of working memory banks */
#endif BPN2
    /* add new fields here */
};

#if BPN2
#define node_info	(* ((struct node_info_struct **) 0xF800F0))
#else not BPN2
#define node_info	(* ((struct node_info_struct **) 0xFE0000F0))
#endif

/* information about USD */
struct USD_info_struct {
    short USD_status;		/* USD test status bits */
    short USD_version;		/* USD version number */
    QH USD_inq;			/* USD character input queue */
    struct USD_data * tusd;	/* pointer to tusd structure (USD use only) */
    struct USD_data * husd;	/* pointer to husd structure (USD use only) */
    char * strarg;		/* pointer to user string arg */
    long args[3];		/* generic user program arg area */
    /* add new fields here */
};

#if BPN2
#define USD_info	(* ((struct USD_info_struct **) 0xF800F4))
#else not BPN2
#define USD_info	(* ((struct USD_info_struct **) 0xFE0000F4))
#endif BPN2


/* information about I/O boards attached to this node */
/* NOTE: this does not describe actual devices (i.e. async ports, Ethernet boards)
   just boards physically attached to the BIOLINK */

/* NOTE: USD only fills in fields about boards it likes (currently BIO and BMA) */
/* others should be filled in by Chrysalis init */

/* room is currently allocated for 4 board entries, but 3 should be current
   max (BMA+2*BIO) */


struct IO_info_struct {
    short NBoards;		/* number of valid entries in following table */
    struct {			/* IO board descriptor entries */
	char brd_type;		/* board type (from board register) */
	char brd_subtype;	/* board subtype (type dependent) */
	long brd_datum;		/* 32-bit datum associated with board (i.e. addr) */
	} IObrds[4];
};

#if BPN2
#define IO_info		(* ((struct IO_info_struct **) 0xF800F8))
#else not BPN2
#define IO_info		(* ((struct IO_info_struct **) 0xFE0000F8))
#endif BPN2


/* error information for node (not well supported yet) */
struct error_info_struct {
    unsigned long par_errcnt;	/* count of parity errors (L6 interrupts) on node */
    short L7_status;		/* status at last L7 interrupt (or'd in) */
    unsigned long L7_cnt;	/* count of L7 interrupts on node */
    unsigned long spurint_cnt;	/* count of spurious interrupts */
    short last_sput;		/* last spurious interrupt vector (if known) */
};

#if BPN2
#define error_info	(* ((struct error_info_struct **) 0xF800FC))
#else not BPN2
#define error_info	(* ((struct error_info_struct **) 0xFE0000FC))
#endif BPN2
#endif usd_h
