/* $Header: /nfs/columbia/u1/BFMACH.local/mk/h/RCS/vfs.h,v 1.10 90/12/18 10:25:01 tappan Exp $ */
#ifndef H_VFS_H
#define H_VFS_H yes
/*	@(#)vfs.h	2.2 88/05/25 4.0NFSSRC SMI; from vfs.h 2.15 87/01/14 SMI	*/

/*
 * File system identifier. Should be unique (at least per machine).
 */
typedef struct {
	long val[2];			/* file system id type */
} fsid_t;

/*
 * File identifier. Should be unique per filesystem on a single machine.
 */
#define	MAXFIDSZ	16
#include "vfs_kmalloctypes.h"
#define	freefid(fidp) \
    KFREE((fidp), KMALLOC_MAKETYPE(KM_MT_VFS, KM_ST_FID));

struct fid {
	u_short		fid_len;		/* length of data in bytes */
	char		fid_data[MAXFIDSZ];	/* data (variable length) */
};

/*
 * paging options
 */
#define POPT_SET_PREF	0x01	/* set preference */
#define POPT_SET_PCNT	0x02	/* set percenet full */
#define POPT_SET_UID	0x04	/* set UID for NFS */
#define POPT_SET_GID	0x08	/* set GID for NFS */

#define POPT_MAX_PREF	9999	/* maximum paging preference */
#define POPT_DEF_PREF	0	/* default paging preference */
#define POPT_MAX_PCNT	100	/* default percenet full */
#define POPT_DEF_PCNT	20	/* default percenet full */
#define POPT_DEF_UID	1	/* default UID for NFS */
#define POPT_DEF_GID	1	/* default GID for NFS */

struct pgr_args {
	int	flags;		/* flags */
	u_short	preference;	/* paging preference (0-N) lower #s preferred */
	u_short	pcnt_full;	/* percent full before choosing a new pager */
	uid_t	nfs_uid;	/* UID to use when paging over NFS */
	gid_t	nfs_gid;	/* GID to use when paging over NFS */
	long	spare[2];	/* reserved */
};

/*
 * Structure per mounted file system.
 * Each mounted file system has an array of
 * operations and an instance record.
 * The file systems are put on a singly linked list.
 */
struct vfs {
	struct vfs	*vfs_next;		/* next vfs in vfs list */
	struct vfsops	*vfs_op;		/* operations on vfs */
	struct vnode	*vfs_vnodecovered;	/* vnode we mounted on */
	int		vfs_flag;		/* flags */
	int		vfs_bsize;		/* native block size */
	fsid_t		vfs_fsid;		/* file system id */
	caddr_t 	vfs_stats;		/* filesystem statistics */
	caddr_t		vfs_data;		/* private data */
	struct pgr_args	vfs_pinfo;		/* paging information */
	struct vfs	*vfs_preflist;		/* bf_choose_fs pref vfs list */
	long		vfs_pcntfree;		/* bf_choose_fs pcnt free */
	long		vfs_minfree;		/* bf_choose_fs min free */
	rwlock_data_t	vfs_rwlock;		/* lookup/unmount interlock */
};

/*
 * vfs flags.
 */
#define VFS_RDONLY	0x0001		/* read only vfs */
#define VFS_NOSUID	0x0008		/* turn off set-uid on exec */
#define VFS_GRPID	0x0010		/* Old BSD group-id on create */
#define VFS_NOSUB	0x0020		/* No mounts allowed beneath this fs */
#define VFS_REMOUNT	0x0040		/* modify mount otions only */
#define VFS_MULTI	0x0080		/* Do multi-component lookup on files */
#define VFS_PAGE_NEVER	0x0100		/* Never use fs for paging */
#define VFS_PAGE_INUSE	0x0200		/* already using this fs for paging */
#define VFS_PAGE_SETOPT	0x0400		/* Set paging options */
#define VFS_PAGE_NOPAGE	0x0800		/* [re]mount fs nopage */
#define VFS_PAGE_PAGE	0x1000		/* [re]mount fs page */

/*
 * Operations supported on virtual file system.
 */
struct vfsops {
	int	(*vfs_mount)();		/* mount file system */
	int	(*vfs_unmount)();	/* unmount file system */
	int	(*vfs_root)();		/* get root vnode */
	int	(*vfs_statfs)();	/* get fs statistics */
	int	(*vfs_sync)();		/* flush fs buffers */
	int	(*vfs_vget)();		/* get vnode from fid */
	int	(*vfs_mountroot)();	/* mount the root filesystem */
};

#define VFS_MOUNT(VFSP, PATH, DATA) \
				 (*(VFSP)->vfs_op->vfs_mount)(VFSP, PATH, DATA)
#define VFS_UNMOUNT(VFSP)	 (*(VFSP)->vfs_op->vfs_unmount)(VFSP)
#define VFS_ROOT(VFSP, VPP)	 (*(VFSP)->vfs_op->vfs_root)(VFSP,VPP)
#define VFS_STATFS(VFSP, SBP)	 (*(VFSP)->vfs_op->vfs_statfs)(VFSP,SBP)
#define VFS_SYNC(VFSP)		 (*(VFSP)->vfs_op->vfs_sync)(VFSP)
#define VFS_VGET(VFSP, VPP, FIDP) (*(VFSP)->vfs_op->vfs_vget)(VFSP, VPP, FIDP)
#define VFS_MOUNTROOT(VFSP, VPP, NM) \
				 (*(VFSP)->vfs_op->vfs_mountroot)(VFSP, VPP, NM)
/*
 * file system statistics
 */
struct statfs {
	long f_type;			/* type of info, zero for now */
	long f_bsize;			/* fundamental file system block size */
	long f_blocks;			/* total blocks in file system */
	long f_bfree;			/* free block in fs */
	long f_bavail;			/* free blocks avail to non-superuser */
	long f_files;			/* total file nodes in file system */
	long f_ffree;			/* free file nodes in fs */
	fsid_t f_fsid;			/* file system id */
	long f_spare[7];		/* spare for later */
};

#ifdef KERNEL
/*
 * Filesystem type switch table
 */
struct vfssw {
	char		*vsw_name;	/* type name string */
	struct vfsops	*vsw_ops;	/* filesystem operations vector */
};

/*
 * public operations
 */
extern void	vfs_mountroot();	/* mount the root */
extern int	vfs_add();		/* add a new vfs to mounted vfs list */
extern void	vfs_remove();		/* remove a vfs from mounted vfs list */
extern int	vfs_lock();		/* lock a vfs */
extern void	vfs_unlock();		/* unlock a vfs */
extern struct vfs *getvfs();		/* return vfs given fsid */
extern struct vfssw *getfstype();	/* find default filesystem type */
extern int vfs_getmajor();		/* get major device # for an fs type */
extern void vfs_putmajor();		/* free major device # for an fs type */
extern int vfs_getnum();		/* get device # for an fs type */
extern void vfs_putnum();		/* release device # for an fs type */

#define VFS_INIT(VFSP, OP, DATA)	{ \
	(VFSP)->vfs_next = (struct vfs *)0; \
	(VFSP)->vfs_op = (OP); \
	(VFSP)->vfs_flag = 0; \
	(VFSP)->vfs_stats = NULL; \
	(VFSP)->vfs_data = (DATA); \
	rwlock_init(&(VFSP)->vfs_rwlock, PVFS); \
}

/*
 * globals
 */
extern struct vfs *rootvfs;		/* ptr to root vfs structure */
extern struct vfssw vfssw[];		/* table of filesystem types */
extern struct vfssw *vfsNVFS;		/* vfs switch table end marker */
#endif
#endif H_VFS_H
