/* $Header: /nfs/europa/u1/BFMACH.local/mk/h/RCS/vm_stat.h,v 1.7 91/01/06 15:12:39 lkaplan Exp $ */

/*****************************************************************************
File:     vm_stat.h
Contents: Public include file for statistics of new vm subsystem.
System:   BBNACI Butterfly OS kernel, virtual memory subsystem.
Created:  9-July-1989
Author:   drb and rwells

Remarks:  These definitions are public since user code needs them, and
are here rather than in vm.h because very little of the kernel needs them,
and they will change fairly often over the next few months as the new VM
system matures; isolating them here should minimize recompiles.

[And now, a few words from the legal department.]

Copyright 1989 by BBN Advanced Computers Inc.
ALL RIGHTS RESERVED

No part of this material may be reproduced, stored in a retrieval
system, transmitted or used in any form or by any means, electronic,
mechanical, photocopying, recording, or otherwise, without the prior
written permission of, or express license from BBN Advanced Computers Inc.

RESTRICTED RIGHTS LEGEND

Use, duplication, or disclosure by the Government is subject to
restrictions as set forth in subparagraph (c)(1)(ii) of the Rights to
Technical Data and Computer Software clause at DFARS 52.227-7013.

BBN Advanced Computers Inc.
10 Fawcett Street
Cambridge, Ma. 02238
617-873-6000
*****************************************************************************/

#ifndef _VM_STAT_H_
#define _VM_STAT_H_

#ifdef KERNEL
#include "../h/vm.h"		/* Include public vm subsystem definitions */
#else
#include <sys/vm.h>		/* Include public vm subsystem definitions */
#endif

/* ----- vm_mapstat codes ------------------------------------------------ */

#define VM_MAPSTAT_REGIONS      1 /* statcode: info on map regions */
#define VM_MAPSTAT_PAGES        2 /* statcode: info on map pages */
#define VM_MAPSTAT_COUNTS       3 /* statcode: map action counts */
#define VM_MAPSTAT_NODECOUNTS   4 /* statcode: map action count sums on node */
#define VM_MAPSTAT_NODEALLOCS   5 /* statcode: node's page allocate actions */
#define VM_MAPSTAT_NODEMEMORY   6 /* statcode: node's memory disposition */
#define VM_MAPSTAT_TOP          7 /* statcode: info on map struct top fields */
#define VM_MAPSTAT_PAGEFRAMES   8 /* statcode: info on all page frames */
#ifdef BFLY2
#define VM_MAPSTAT_INTRLVMEMORY 9 /* statcode: interleaved memory disposition */
#endif
#define VM_MAPSTAT_MAXCODE	9 /* statcode: max valid code */


/* ----- old VM_MAPSTAT_REGIONS definitions ------------------------------ */

typedef struct vm_mapstat *vm_mapstat_t;
typedef struct vm_mapstat
{
    vm_addr_t     address;	/* Starting address */
    vm_size_t     size;		/* Length */
    vm_inherit_t  inherit;	/* Inheritance value */
    vm_prot_t     maxprot;	/* Maximum protection */
    vm_prot_t     curprot;	/* Current protection */
    unsigned      refcnt;	/* (loosely) number of references */
    unsigned short dev;		/* device if file */
    unsigned      inode;	/* inode  if file */
    vm_offset_t   offset;	/* paging offset */
    int           npages;	/* number of resident pages*/
    int           home_node;	/* logical node w/memory */
    int           mapmemflags;	/* flags if mapped file */
    unsigned      ismapmem:1,	/* True for mapped file */
                  wired:1,	/* True if wired down */
                  istext:1,	/* True if text segment */
                  haspager:1;	/* True if object has pager */
} vm_mapstat_data_t;


/* ----- new VM_MAPSTAT_REGIONS definitions ------------------------------ */

typedef struct vm_regionstat *vm_regionstat_t;
typedef struct vm_regionstat
{
    vm_addr_t     address;	/* Entry: Starting address */
    vm_size_t     size;		/* Entry: Length */
    vm_prot_t     curprot;	/* Entry: Current protection */
    vm_prot_t     maxprot;	/* Entry: Maximum protection */
    vm_inherit_t  inherit;	/* Entry: Inheritance value */
    int           caching;	/* Entry: Caching flags */
    int           wirecnt;	/* Entry: Entry wiring count */
    vm_addr_t     object;	/* Entry: Address of object under entry */
    vm_offset_t   offset;	/* Entry: Offset into object */
    int           npages;	/* Object: Number of resident pages */
    int           outcnt;	/* Object: Count of vnode pages not in */
    vm_addr_t     start_used;	/* Object: Offset of first page ever */
    vm_addr_t     end_used;	/* Object: Offset after last page ever */
    vm_addr_t     parmap;	/* Object: Address of parent map */
    int           parcnt;	/* Object: Number of parent map references */
    int           othcnt;	/* Object: Number of other map references */
    int           objnode;	/* Object: node for object struct */
    int           pagenode;	/* Object: prefered node for pages, maybe -1 */
    int           pagestatus;	/* Object: desired page status */
    vm_addr_t     mapping;	/* Object: Address of Mapping, if any */
    int           mapnode;	/* Mapping: table lookup node, if ismapping */
    vm_offset_t   voffset;	/* Mapping: Starting offset in vnode */
    vm_offset_t   vsize;	/* Mapping: size of mapping on vnode */
    vm_addr_t     vnode;	/* Mapping: address of mapped vnode */
    unsigned      dev;		/* VNode: device, if has mapping to file */
    unsigned      inode;	/* VNode: inode, if has mapping to file */
    int           mapcount;	/* VNode: count of VM mappings on vnode */
    char          iscopy;	/* Entry: is marked to be copied as needed? */
    char          istext;	/* Entry: is marked as text, copy on write? */
    char          noborrow;	/* Object: is borrowing inhibited? */
    char          scattered;	/* Object: is page allocation scattered? */
    char          interleaved;	/* Object: use hardware interleaved pages? */
    char          isherd;	/* Object: is object current or past herd? */
    char          ismaptext;	/* Mapping: is marked as text mapping */
    char          isvtext;	/* Vnode: is marked VTEXT */
    char          istruncate;	/* Vnode: is marked for truncate */
} vm_regionstat_data_t;


/* ----- VM_MAPSTAT_PAGES definitions ------------------------------------ */

typedef struct vm_pagestat *vm_pagestat_t;
typedef struct vm_pagestat
{
    unsigned node: 9;
    unsigned frame: 15;
    unsigned flags: 8;
} vm_pagestat_data_t;

	
/* vm_mapstat PAGES bit definitions */

#define VM_VALID    	0x01
#define VM_WRITEPROT 	0x02
#define VM_USED     	0x04
#define VM_MODIFIED 	0x08
#define VM_CACHEABLE 	0x10
#define VM_GLOBAL   	0x20
#define VM_WRITETHRU 	0x40


/* ----- VM_MAPSTAT_NODEMEMORY definitions -------------------------------- */

#define VM_MEM_TOTAL          0	/* # of bytes in total on node */
#define VM_MEM_FREE           1	/* # of bytes in free list */
#define VM_MEM_ZEROED         2	/* # of bytes in zeroed list */
#define VM_MEM_SUSPENDED      3	/* # of bytes in suspended list */
#define VM_MEM_INACTIVE       4	/* # of bytes in inactive list */
#define VM_MEM_ACTIVE         5	/* # of bytes in active list */
#define VM_MEM_IOERROR        6	/* # of bytes in list for write errors */
#define VM_MEM_WIRED          7	/* # of explicitly wired bytes */
#define VM_MEM_KZONE          8	/* # of bytes in kernel zones,proctbl,bufs */
#define VM_MEM_KPTES          9	/* # of bytes in pmap/pte tables */
#define VM_MEM_KTEXT         10	/* # of bytes in kernel text */
#define VM_MEM_KDATA         11	/* # of bytes in kernel data/bss - master */
#define VM_MEM_KSYMBOLS      12	/* # of bytes in kernel kdb symbols - master */
#define VM_MEM_KPAGETS       13 /* # of bytes in kernel vm_page_t's */
#define VM_MEM_KMEM          14	/* # of bytes in kernel kmem_alloc pages */
#define VM_MEM_KELOG         15	/* # of bytes in kernel elog buffers */
#define VM_MEM_KPROC         16	/* # of bytes in kernel proc tables */
#define VM_MEM_KPFREE        17	/* # of bytes in kernel free pmap/pte pages */
#define VM_MEM_INTERLEAVED   18	/* # of bytes for interleaved pages */
#define VM_MEM_KBUF          19	/* # of bytes for buffer pool pages */
#define VM_MEM_IDLECACHE     20 /* # of bytes for idle cached text pages */
#define VM_MEM_KMALLOC	     21 /* # of bytes used by KMALLOC pools */
#define VM_MEM_INACTBUF	     22 /* # of bytes for inactive buffer pages */
#define VM_MEM_COUNT         23	/* Number of memory categories defined */

/* ----- VM_MAPSTAT_COUNTS/VM_MAPSTAT_NODECOUNTS definitions -------------- */

#define VM_CNT_FAULTS         0	/* # of page faults of any sort */
#define VM_CNT_BADADDRS       1	/* # of segment violation faults */
#define VM_CNT_WIRINGS        2	/* # of special wiring faults */
#define VM_CNT_BADPROTS       3	/* # of protection failure faults */
#define VM_CNT_RESUMPTIONS    4	/* # of resumptions for suspended pages */
#define VM_CNT_REACTIVATES    5	/* # of inactive reactivates */
#define VM_CNT_PAGEINS        6	/* # of page ins from back store or file */
#define VM_CNT_ZEROFILLS      7	/* # of zero fill faults */
#define VM_CNT_PRIVATIZE      8	/* # of privatizing faults */
#define VM_CNT_COWFAULTS      9	/* # of privatize that actually do copy */
#define VM_CNT_RVALIDATES    10	/* # of simple read validate faults */
#define VM_CNT_WVALIDATES    11	/* # of simple write validate faults */
#define VM_CNT_MAPPINGLOOKS  12 /* # of lookups in mapping table */
#define VM_CNT_MAPPINGHITS   13	/* # of direct hits in mapping table */
#define VM_CNT_CACHELOOKS    14	/* # of lookups in text mapping cache */
#define VM_CNT_CACHEHITS     15	/* # of hits in text mapping cache alone */
#define VM_CNT_BORROWDEST    16 /* # of pages borrowed for map on node */
#define VM_CNT_FOREIGN       17	/* # of faults on foreign (non-nX) memory */
#define VM_CNT_PREZEROFILLS  18	/* # of zero fill faults met by pre-zeroing */
#define VM_CNT_WASTEALLOC    19 /* # of faults with unnecessary page alloc */
#define VM_CNT_INTRLV	     20	/* # of intrlvd page faults */
#define VM_CNT_INTRLVBORROW  21 /* # of pages borrowed under interleaving */
#define VM_CNT_COUNT	     22	/* # of counts for map (sums on node) */

/* ----- VM_MAPSTAT_NODEALLOCS definitions -------------------------------- */

#define VM_ALLO_BORROWSRC     0 /* # of pages borrowed from node lists */
#define VM_ALLO_FROMACTIVE    1	/* # of pages alloc from active list */
#define VM_ALLO_FROMINACTIVE  2 /* # of pages alloc from inactive list */
#define VM_ALLO_FROMSUSPENDED 3 /* # of pages alloc from suspend list */
#define VM_ALLO_REMOVALS      4	/* # of pages removed by vm_page_remove */
#define VM_ALLO_PAGEOUTS      5	/* # of pageouts outback or to file */
#define VM_ALLO_SUSPENDS      6	/* # of pages suspended */
#define VM_ALLO_INACTIVATES   7	/* # of pages inactivated */
#define VM_ALLO_PAGEBUSY      8 /* # of process waits on busy pages */
#define VM_ALLO_FROMCACHE     9 /* # of pages from idle text cache objects */
#define VM_ALLO_FROMIBUF     10 /* # of pages from inactive buffer pool */
#define VM_ALLO_COUNT        11 /* # of node reclaim action counters defined */

/* ----- VM_MAPSTAT_TOP definitions ---------------------------------------- */

typedef struct vm_topstat *vm_topstat_t;
typedef struct vm_topstat
{
    int         home_node;	/* location of map + entries */
    vm_addr_t   limit_start;	/* 1st addr whose entry can be changed */
    vm_addr_t   limit_end;	/* 1st addr whose entry can't be changed */
    vm_addr_t   anywhere_start; /* min default address for ANYWHERE */
    vm_addr_t   herd[2];	/* Object for newly privatized pages (COWs) */
				/* [0] allows borrowing, [1] doesn't. */
    int         cid;		/* Cluster id for map, for borrow/scatter */
    int         pmap_pagecnt;	/* pages used in map's pmap */
} vm_topstat_data_t;


/* ----- VM_MAPSTAT_PAGEFRAMES definitions -------------------------------- */

typedef struct vm_framestat *vm_framestat_t;
typedef struct vm_framestat
{
    vm_pagestat_data_t pstat;	/* node & frame; zeroed flags */
    vm_addr_t   object;		/* page's object, or 0 */
    vm_offset_t offset;		/* offset in object, if any */
    vm_addr_t   vaddr1;		/* First virtual address in play, or -1 */
    unsigned short locrefs;	/* page scanner local ref count */
    unsigned short rmtrefs;	/* page scanner remote ref count */
    unsigned short wirecnt;	/* wirecnt for page */
    char        status;		/* page status code */
    char        isdirty;	/* 1 if page has been written */
    char        isborrowed;	/* 1 if page is borrowed */
    char        isoutback;	/* 1 if page came from paging disk */
    char        isinterleaved;	/* 1 if page is part of interleaved pool */
    char        padding;
} vm_framestat_data_t;

/* ----- System call function prototypes --------------------------------- */

#ifndef KERNEL			/* if KERNEL is not defined */
#if defined(__STDC__) || defined(c_plusplus) /* if prototypes... */
				/* ANSI C and C++ function prototypes */
vm_errkern_t vm_mapstat(vm_task_t task, vm_addr_t address, vm_size_t size,
			int statcode, int structsize, int maxcnt,
			/*OUT*/ int *actcnt, /*OUT*/ vm_mapstat_t statp);

vm_errkern_t vm_mapstat_pid(int pid, vm_addr_t address, vm_size_t size,
			    int statcode, int structsize, int maxcnt,
			    /*OUT*/ int *actcnt, /*OUT*/ vm_mapstat_t statp);
#else				/* else if not prototypes... */
vm_errkern_t vm_mapstat(/*see above*/);
vm_errkern_t vm_mapstat_pid(/*see above*/);
#endif				/* end if not prototypes */
#endif				/* end if KERNEL is not defined */

/* ----- end of the file... ---------------------------------------------- */

#endif _VM_STAT_H_
