/* $Header: /nfs/europa/u1/BFMACH.local/mk/net/RCS/if.h,v 1.18 90/04/06 09:19:49 tappan Exp $ */
#ifndef _if_h
#define _if_h yes!
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)if.h	7.3 (Berkeley) 6/27/88
 */

/*
 * Structures defining a network interface, providing a packet
 * transport mechanism (ala level 0 of the PUP protocols).
 *
 * Each interface accepts output datagrams of a specified maximum
 * length, and provides higher level routines with input datagrams
 * received from its medium.
 *
 * Output occurs when the routine if_output is called, with three parameters:
 *	(*ifp->if_output)(ifp, m, dst)
 * Here m is the mbuf chain to be sent and dst is the destination address.
 * The output routine encapsulates the supplied datagram if necessary,
 * and then transmits it on its medium.
 *
 * On input, each interface unwraps the data received by it, and either
 * places it on the input queue of a internetwork datagram routine
 * and posts the associated software interrupt, or passes the datagram to a raw
 * packet input routine.
 *
 * Routines exist for locating interfaces by their addresses
 * or for locating a interface on a certain network, as well as more general
 * routing and gateway routines maintaining information used to locate
 * interfaces.  These routines live in the files if.c and route.c
 */

#ifdef	butterfly
#include "../h/lock.h"
#endif	butterfly

/*
 * Structure defining a queue for a network interface.
 *
 * (Would like to call this struct ``if'', but C isn't PL/1.)
 */
#define IFNAMSIZ	16
struct ifnet {
	char	*if_name;		/* name, e.g. ``en'' or ``lo'' */
#ifdef	butterfly
	int	if_lnode;		/* logical node this ethernet is on */
#endif	butterfly
	short	if_unit;		/* sub-unit for lower level driver */
	short	if_mtu;			/* maximum transmission unit */
	short	if_flags;		/* up/down, broadcast, etc. */
	short	if_timer;		/* time 'til if_watchdog called */
	int	if_metric;		/* routing metric (external only) */
	struct	ifaddr *if_addrlist;	/* linked list of addresses per if */
	struct	ifqueue {
		struct	mbuf *ifq_head;
		struct	mbuf *ifq_tail;
#ifdef	butterfly
		simple_lock_data_t ifq_lock;	/* queue multiprocessor lock */
#endif	butterfly
		int	ifq_len;
		int	ifq_maxlen;
		int	ifq_drops;
	} if_snd;			/* output queue */
/* procedure handles */
	int	(*if_init)();		/* init routine */
	int	(*if_output)();		/* output routine */
	int	(*if_ioctl)();		/* ioctl routine */
	int	(*if_reset)();		/* bus reset routine */
	int	(*if_watchdog)();	/* timer routine */
/* generic interface statistics */
	int	if_ipackets;		/* packets received on interface */
	int	if_ierrors;		/* input errors on interface */
	int	if_opackets;		/* packets sent on interface */
	int	if_oerrors;		/* output errors on interface */
	int	if_collisions;		/* collisions on csma interfaces */
/* end statistics */
	struct	ifnet *if_next;
#ifdef butterfly
	char    if_ifname[IFNAMSIZ];	/* user settable name */
	caddr_t	if_softc;	/* Available for driver use */
#endif butterfly
};

#define	IFF_UP		0x1		/* interface is up */
#define	IFF_BROADCAST	0x2		/* broadcast address valid */
#define	IFF_DEBUG	0x4		/* turn on debugging */
#define	IFF_LOOPBACK	0x8		/* is a loopback net */
#define	IFF_POINTOPOINT	0x10		/* interface is point-to-point link */
#define	IFF_NOTRAILERS	0x20		/* avoid use of trailers */
#define	IFF_RUNNING	0x40		/* resources allocated */
#define	IFF_NOARP	0x80		/* no address resolution protocol */
/* next two not supported now, but reserved: */
#define	IFF_PROMISC	0x100		/* receive all packets */
#define	IFF_ALLMULTI	0x200		/* receive all multicast packets */
#define	IFF_PARALLEL	0x400		/* is a parallel interface */
#define IFF_NOBROADCAST 0x800		/* ignore incoming broadcasts */
#define IFF_ARP		0x1000		/* This interface type supports arp */
#define IFF_LDSHARE	0x2000		/* This is a load sharing interface */
/* flags set internally only: */
#define	IFF_CANTCHANGE	(IFF_BROADCAST | IFF_POINTOPOINT | \
			 IFF_RUNNING | IFF_NOBROADCAST | IFF_ARP | IFF_LDSHARE)

/*
 * Output queues (ifp->if_snd) and internetwork datagram level (pup level 1)
 * input routines have queues of messages stored on ifqueue structures
 * (defined above).  Entries are added to and deleted from these structures
 * by these macros, which should be called with ipl raised to splimp().
 */
#define	IF_QFULL(ifq)		((ifq)->ifq_len >= (ifq)->ifq_maxlen)
#define	IF_DROP(ifq)		((ifq)->ifq_drops++)
#ifdef	butterfly
#define	IF_ENQUEUE(ifq, m) { \
	if (simple_lock_is_held(&(ifq)->ifq_lock) == 0) \
		panic("IF_ENQUEUE"); \
	(m)->m_act = 0; \
	if ((ifq)->ifq_tail == 0) \
		(ifq)->ifq_head = m; \
	else \
		(ifq)->ifq_tail->m_act = m; \
	(ifq)->ifq_tail = m; \
	(ifq)->ifq_len++; \
}
#define	IF_PREPEND(ifq, m) { \
	if (simple_lock_is_held(&(ifq)->ifq_lock) == 0) \
		panic("IF_PREPEND"); \
	(m)->m_act = (ifq)->ifq_head; \
	if ((ifq)->ifq_tail == 0) \
		(ifq)->ifq_tail = (m); \
	(ifq)->ifq_head = (m); \
	(ifq)->ifq_len++; \
}
#else	butterfly
#define	IF_ENQUEUE(ifq, m) { \
	(m)->m_act = 0; \
	if ((ifq)->ifq_tail == 0) \
		(ifq)->ifq_head = m; \
	else \
		(ifq)->ifq_tail->m_act = m; \
	(ifq)->ifq_tail = m; \
	(ifq)->ifq_len++; \
}
#define	IF_PREPEND(ifq, m) { \
	(m)->m_act = (ifq)->ifq_head; \
	if ((ifq)->ifq_tail == 0) \
		(ifq)->ifq_tail = (m); \
	(ifq)->ifq_head = (m); \
	(ifq)->ifq_len++; \
}
#endif	butterfly
/*
 * Packets destined for level-1 protocol input routines
 * have a pointer to the receiving interface prepended to the data.
 * IF_DEQUEUEIF extracts and returns this pointer when dequeueing the packet.
 * IF_ADJ should be used otherwise to adjust for its presence.
 */
#define	IF_ADJ(m) { \
	(m)->m_off += sizeof(struct ifnet *); \
	(m)->m_len -= sizeof(struct ifnet *); \
	if ((m)->m_len == 0) { \
		struct mbuf *n; \
		MFREE((m), n); \
		(m) = n; \
	} \
}
#ifdef	butterfly
#define	IF_DEQUEUEIF(ifq, m, ifp) { \
	if (simple_lock_is_held(&(ifq)->ifq_lock) == 0) \
		panic("IF_DEQUEUEIF"); \
	(m) = (ifq)->ifq_head; \
	if (m) { \
		if (((ifq)->ifq_head = (m)->m_act) == 0) \
			(ifq)->ifq_tail = 0; \
		(m)->m_act = 0; \
		(ifq)->ifq_len--; \
		(ifp) = *(mtod((m), struct ifnet **)); \
		IF_ADJ(m); \
	} \
}
#define	IF_DEQUEUE(ifq, m) { \
	if (simple_lock_is_held(&(ifq)->ifq_lock) == 0) \
		panic("IF_DEQUEUE"); \
	(m) = (ifq)->ifq_head; \
	if (m) { \
		if (((ifq)->ifq_head = (m)->m_act) == 0) \
			(ifq)->ifq_tail = 0; \
		(m)->m_act = 0; \
		(ifq)->ifq_len--; \
	} \
}
#else	butterfly
#define	IF_DEQUEUEIF(ifq, m, ifp) { \
	(m) = (ifq)->ifq_head; \
	if (m) { \
		if (((ifq)->ifq_head = (m)->m_act) == 0) \
			(ifq)->ifq_tail = 0; \
		(m)->m_act = 0; \
		(ifq)->ifq_len--; \
		(ifp) = *(mtod((m), struct ifnet **)); \
		IF_ADJ(m); \
	} \
}
#define	IF_DEQUEUE(ifq, m) { \
	(m) = (ifq)->ifq_head; \
	if (m) { \
		if (((ifq)->ifq_head = (m)->m_act) == 0) \
			(ifq)->ifq_tail = 0; \
		(m)->m_act = 0; \
		(ifq)->ifq_len--; \
	} \
}
#endif	butterfly

#define	IFQ_MAXLEN	50
#define	IFNET_SLOWHZ	1		/* granularity is 1 second */

/*
 * The ifaddr structure contains information about one address
 * of an interface.  They are maintained by the different address families,
 * are allocated and attached when an address is set, and are linked
 * together so all addresses for an interface can be located.
 */
struct ifaddr {
	struct	sockaddr ifa_addr;	/* address of interface */
	union {
		struct	sockaddr ifu_broadaddr;
		struct	sockaddr ifu_dstaddr;
	} ifa_ifu;
#define	ifa_broadaddr	ifa_ifu.ifu_broadaddr	/* broadcast address */
#define	ifa_dstaddr	ifa_ifu.ifu_dstaddr	/* other end of p-to-p link */
	struct	ifnet *ifa_ifp;		/* back-pointer to interface */
	struct	ifaddr *ifa_next;	/* next address for interface */
};

/*
 * Interface request structure used for socket
 * ioctl's.  All interface ioctl's must have parameter
 * definitions which begin with ifr_name.  The
 * remainder may be interface specific.
 */
struct	ifreq {
#define	IFNAMSIZ	16
	char	ifr_name[IFNAMSIZ];		/* if name, e.g. "en0" */
	union {
#ifdef butterfly
		struct {
		  char if_driver[12];	/* driver name (eg. "ex")*/
		  short if_iobus;	/* mba */
		  short if_unit;	/* Sub unit */
		} ifru_setifname;
#define ifr_setifname ifr_ifru.ifru_setifname
#endif butterfly
		struct	sockaddr ifru_addr;
		struct	sockaddr ifru_dstaddr;
		struct	sockaddr ifru_broadaddr;
		short	ifru_flags;
		int	ifru_metric;
		caddr_t	ifru_data;
	} ifr_ifru;
#define	ifr_addr	ifr_ifru.ifru_addr	/* address */
#define	ifr_dstaddr	ifr_ifru.ifru_dstaddr	/* other end of p-to-p link */
#define	ifr_broadaddr	ifr_ifru.ifru_broadaddr	/* broadcast address */
#define	ifr_flags	ifr_ifru.ifru_flags	/* flags */
#define	ifr_metric	ifr_ifru.ifru_metric	/* metric */
#define	ifr_data	ifr_ifru.ifru_data	/* for use by interface */
};

/*
 * Structure used in SIOCGIFCONF request.
 * Used to retrieve interface configuration
 * for machine (useful for programs which
 * must know all networks accessible).
 */
struct	ifconf {
	int	ifc_len;		/* size of associated buffer */
	union {
		caddr_t	ifcu_buf;
		struct	ifreq *ifcu_req;
	} ifc_ifcu;
#define	ifc_buf	ifc_ifcu.ifcu_buf	/* buffer address */
#define	ifc_req	ifc_ifcu.ifcu_req	/* array of structures returned */
};

#ifdef KERNEL
#include "../net/if_arp.h"
struct	ifqueue rawintrq;		/* raw packet input queue */
struct	ifnet *ifnet;
struct	ifaddr *ifa_ifwithaddr(), *ifa_ifwithnet();
struct	ifaddr *ifa_ifwithdstaddr();
#else KERNEL
#include <net/if_arp.h>
#endif KERNEL

#ifdef	butterfly
#ifdef	KERNEL
#include "../h/queue.h"

struct ifrpc {
	struct queue_entry entry;
	simple_lock_data_t lock;
	struct ifnet *ifp;
	union {
		struct mbuf *m0;
		int cmd;
	} u_arg1;
	union {
		struct sockaddr dst;
		caddr_t data;
	} u_arg2;
	int flags;
	int rval;
};

/*
 * rval specifies the type of operation
 */
#define	IF_RPC_OUTPUT	0	/* have if server do output */
#define	IF_RPC_WATCHDOG	1	/* have if server do watchdog */
#define	IF_RPC_IOCTL	2	/* have if server do ioctl */
#define	IF_RPC_RESET	3	/* have if server do reset */
#define	IF_RPC_INIT	4	/* have if server do init */

/*
 * flags
 */
#define	IF_RPC_DONE	0x01	/* if 1, if_via_rpc complete */
#define	IF_RPC_SLEEP	0x02	/* if 1, if_via_rpc may sleep */

#define	IF_RPC_PAGES	8	/* max zone pages for rpc cookies */
#define	SLIFQ(ifq)	simple_lock(&(ifq)->ifq_lock);
#define	SUIFQ(ifq)	simple_unlock(&(ifq)->ifq_lock);
#endif	KERNEL

#ifdef notdef
/*
 * definitions for extracting unit and bmaid information from ifr_name
 *
 * the ifr_name is constructed as follows:
 *	IF_VECBIT | (bmaid << log2(IF_UNITBITS)) | unit
 */
#define	IF_UNITPERNODE	4	/* max # of (ex) interfaces per bma */
#define	IF_UNITBITS	2
#define	IF_UNITMASK	0x0003
#define IF_UNIT(num)	((num) & IF_UNITMASK)
#define	IF_VECBITS	1
#define	IF_VECBIT	(1 << (IF_UNITBITS + IF_BMAIDBITS))
#define	IF_VEC(num)	((num) & IF_VECBIT)
#define	IF_BMAIDSHIFT	IF_UNITBITS
#define	IF_BMAIDBITS	((sizeof(short) * 8) - IF_UNITBITS - IF_VECBITS)
#define	IF_BMAIDMASK	0x7ffc
#define IF_BMAID(num)	(((num) & IF_BMAIDMASK) >> IF_BMAIDSHIFT)
#else
/*
 * New format for ifr_name
 *
 * xx<bmaid:2><unit:1>
 *
 * For example
 *  ex000 :== Unit 0 on BMA 0
 *
 * As a special case xx<unit:1> is interpreted on input as 
 * xx<master_bmaid:2><unit:1>
 *
 * On output, if if_lnode == IF_ALLNODES then output is xx<unit:1>
 * else xx<bmaid:2><unit:1>
 */
#define IF_BMAID(num) 	((num) / 10)
#define IF_UNIT(num)	((num) % 10)
#define IF_ALLNODES -1
#endif
#endif	butterfly
#endif _if_h
