/* $Header: route.h,v 1.9 90/02/09 08:30:16 tappan Exp $ */
/*
 * Copyright (c) 1980, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)route.h	7.4 (Berkeley) 6/27/88
 */

/*
 * Kernel resident routing tables.
 * 
 * The routing tables are initialized when interface addresses
 * are set by making entries for all directly connected interfaces.
 */

/*
 * A route consists of a destination address and a reference
 * to a routing entry.  These are often held by protocols
 * in their control blocks, e.g. inpcb.
 */
struct route {
	struct	rtentry *ro_rt;
	struct	sockaddr ro_dst;
};

/*
 * We distinguish between routes to hosts and routes to networks,
 * preferring the former if available.  For each route we infer
 * the interface to use from the gateway address supplied when
 * the route was entered.  Routes that forward packets through
 * gateways are marked so that the output routines know to address the
 * gateway rather than the ultimate destination.
 */
struct rtentry {
	u_long	rt_hash;		/* to speed lookups */
	struct	sockaddr rt_dst;	/* key */
	struct	sockaddr rt_gateway;	/* value */
	short	rt_flags;		/* up/down?, host/net */
	short	rt_refcnt;		/* # held references */
	u_long	rt_use;			/* raw # packets forwarded */
	struct	ifnet *rt_ifp;		/* the answer: interface to use */
};
/*
 * There are some internal fields to the route entry. For reasons
 * of backward compatability with ioctl (the ioctl code contains
 * the argument structure size) these are kept in a 
 * structure which overlays the struct rtentry above.
 *
 * When allocating a route entry for use in the routing table
 * you should make sure that it is actually as big as the rtentry_internal
 */
struct rtentry_internal {
    struct rtentry rt_rtentry;	/* The real entry */
    struct rtentry_internal *rt_ldshare; /* Load sharing routes */
/*
 * Load sharing routes are routes with exactly the same characteristics
 * which lead to the same destination. Examples might be multiple interfaces
 * connected to the same network, or multiple gateways between two networks.
 * All routing entries are linked in a circular list to other load sharing
 * routes. In the single route case this degenerates to the rtentry
 * pointing to itself.
 */
    struct rtentry_internal *rt_next; /* linked list of routes */
};
#define RT_LDSHARE(rt) \
    ((struct rtentry *)((struct rtentry_internal *)rt)->rt_ldshare)
#define RT_NEXT(rt) \
    (((struct rtentry_internal *)(rt))->rt_next)
#define RT_I(rt) ((struct rtentry_internal *)(rt))
#define RT_X(rti) ((struct rtentry *)(rti))

#define	RTF_UP		0x1		/* route useable */
#define	RTF_GATEWAY	0x2		/* destination is a gateway */
#define	RTF_HOST	0x4		/* host entry (net otherwise) */
#define	RTF_DYNAMIC	0x10		/* created dynamically (by redirect) */
#define	RTF_MODIFIED	0x20		/* modified dynamically (by redirect) */

/*
 * Routing statistics.
 */
struct	rtstat {
	short	rts_badredirect;	/* bogus redirect calls */
	short	rts_dynamic;		/* routes created by redirects */
	short	rts_newgateway;		/* routes modified by redirects */
	short	rts_unreach;		/* lookups which failed */
	short	rts_wildcard;		/* lookups satisfied by a wildcard */
};

#ifdef KERNEL
#define	RTFREE(rt) \
        if (Atomic_fetch16(&(rt)->rt_refcnt) == 1) \
		rtfree((rt)); \
        else \
    	    Atomic_add16(&(rt)->rt_refcnt, -1)

#ifdef	GATEWAY
#define	RTHASHSIZ	64
#else
#define	RTHASHSIZ	8
#endif
#if	(RTHASHSIZ & (RTHASHSIZ - 1)) == 0
#define RTHASHMOD(h)	((h) & (RTHASHSIZ - 1))
#else
#define RTHASHMOD(h)	((h) % RTHASHSIZ)
#endif
struct	rtentry *rthost[RTHASHSIZ];
struct	rtentry *rtnet[RTHASHSIZ];
struct	rtstat	rtstat;
#endif
