/* $Header: in_pcb.h,v 1.8 90/01/22 17:40:22 tappan Exp $ */
/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that the above copyright notice and this paragraph are
 * duplicated in all such forms and that any documentation,
 * advertising materials, and other materials related to such
 * distribution and use acknowledge that the software was developed
 * by the University of California, Berkeley.  The name of the
 * University may not be used to endorse or promote products derived
 * from this software without specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 *	@(#)in_pcb.h	7.3 (Berkeley) 6/29/88
 */

/*
 * Common structure pcb for internet protocol implementation.
 * Here are stored pointers to local and foreign host table
 * entries, local and foreign socket numbers, and pointers
 * up (to a socket structure) and down (to a protocol-specific)
 * control block.
 */
struct inpcb {
	struct	inpcb *inp_next,*inp_prev;
					/* pointers to other pcb's */
	struct	inpcb *inp_head;	/* pointer back to chain of inpcb's
					   for this protocol */
	struct	in_addr inp_faddr;	/* foreign host table entry */
	struct	in_addr inp_laddr;	/* local host table entry */
	u_short	inp_fport;		/* foreign port */
	u_short	inp_lport;		/* local port */
	struct	socket *inp_socket;	/* back pointer to socket */
	caddr_t	inp_ppcb;		/* pointer to per-protocol pcb */
	struct	route inp_route;	/* placeholder for routing entry */
	struct	mbuf *inp_options;	/* IP options */
#ifdef butterfly
	simple_lock_data_t inp_lock; /* Per pcb lock */
	u_short inp_flags;	/* Flag bits */
#define INPF_TIMERPASS 0x01
#define INPF_DELETING 0x02	/* Someone is trying to delete the pcb */
#define INPF_CONNECTING 0x04	/* pcb is temporarily unlocked (bind/connect)*/
	u_short inp_spl;	/* saved spl */
#endif butterfly
};

#ifdef KERNEL
#ifdef butterfly

/*
 * Locking protocol.
 * Each in_pcb has a lock and each queue of in_pcb's has a queue lock
 * (in the in_pcb which is the queue head).
 *
 * The individual lock must be held to manipulate an in_pcb.
 * The queue lock must be held to add to, remove from, or search a queue.
 * The socket lock must be held to set the pcb pointer in the socket.
 *
 * To avoid deadlock, an individual in_pcb lock cannot be held while
 * trying to lock a queue lock or the socket lock.
 *
 * When an in_pcb is about to be deleted the 'in_deleting' flag will be set. 
 * Anyone who acquires an in_pcb with the deleting flag set should free
 * the lock immediatly.
 * 
 * A process must remain at splnet() or higher while any of the above locks
 * are held.
 */


/*
 * Pointer to lock/unlock functions. 
 * simple_lock_IFNP (IF Network Parallel)
 * simple_unlock_IFNP
 * These are set to point to simple_(un)lock if  the network code runs
 * in parallel otherwise they point to a nop.
 */
extern void (*simple_lock_IFNP)(), (*simple_unlock_IFNP)();

extern int my_cpu_number;
/*
 * UNLOCKINPCB - Unlocks the in_pcb and returns to the previous spl
 */
#define UNLOCKINPCB(inp) \
{ int s = (inp)->inp_spl; \
  (*simple_unlock_IFNP)(&(inp)->inp_lock); \
  splx(s); \
}

#endif butterfly

#define	INPLOOKUP_WILDCARD	1
#define	INPLOOKUP_SETLOCAL	2

#ifdef butterfly
#define INPLOOKUP_NOLOCK	4

#define sotoinpcb(so)	lockinpcb((struct inpcb *)((so)->so_pcb))
#else
#define	sotoinpcb(so)	((struct inpcb *)(so)->so_pcb)
#endif butterfly
#endif KERNEL

#ifdef KERNEL
struct	inpcb *in_pcblookup();
#ifdef butterfly
struct inpcb *lockinpcb();
struct inpcb *in_pcbdisconnect();
struct inpcb *in_pcbdetach();
#endif butterfly
#endif


